Package: PathwaySpace
Type: Package
Title: Spatial Projection of Network Signals along Geodesic Paths
Version: 1.3.0
Authors@R: 
    c(person("Sysbiolab Team", role = "aut"),
      person("Victor", "Apolonio", role = "ctb"),
      person("Jonathan", "Back", role = "ctb"),
      person("Lana", "Querne", role = "ctb"),
      person("Vinicius", "Chagas", role = "ctb"),
      person("Bahar", "Tercan", role = "ctb"),
      person("Mauro", "Castro", role = c("cre"), 
              email = "mauro.a.castro@gmail.com", 
              comment = c(ORCID = "0000-0003-4942-8131"))
      )
Depends: R(>= 4.5), methods, RGraphSpace(>= 1.2)
Imports: grDevices, stats, scales, RANN, igraph, ggplot2, ggrepel,
        grid, colorspace, patchwork, lifecycle
Suggests: knitr, rmarkdown, testthat, RedeR
Description: For a given graph containing vertices, edges, and a signal 
    associated with the vertices, the 'PathwaySpace' package performs a convolution 
    operation, which involves a weighted combination of neighboring vertices and 
    their associated signals. The package uses a decay function to project these 
    signals, creating geodesic paths on a 2D-image space. 'PathwaySpace' has various 
    applications, such as visualizing network data in a graphical format that 
    highlights the relationships and signal strengths between vertices. By 
    combining graph theory, signal processing, and visualization, 'PathwaySpace' 
    provides a way of representing graph data on a continuous projection space.
    Based on methods introduced in Tercan et al. (2025) 
    <doi:10.1016/j.xpro.2025.103681> and Ellrott et al. (2025) 
    <doi:10.1016/j.ccell.2024.12.002>.
License: Artistic-2.0
VignetteBuilder: knitr
URL: https://sysbiolab.github.io/PathwaySpace/,
        https://github.com/sysbiolab/PathwaySpace
BugReports: https://github.com/sysbiolab/PathwaySpace/issues
Collate: pspaceChecks.R pspaceSupplements.R pspaceMisc.R
        pspaceWatershed.R pspaceDecay.R pspaceClasses.R
        pspaceGenerics.R pspaceMethods.R pspacePlots.R
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-23 14:40:01 UTC; maac
Author: Sysbiolab Team [aut],
  Victor Apolonio [ctb],
  Jonathan Back [ctb],
  Lana Querne [ctb],
  Vinicius Chagas [ctb],
  Bahar Tercan [ctb],
  Mauro Castro [cre] (ORCID: <https://orcid.org/0000-0003-4942-8131>)
Maintainer: Mauro Castro <mauro.a.castro@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-23 15:20:02 UTC
Built: R 4.5.2; ; 2026-05-25 02:33:35 UTC; unix

# Generated by roxygen2: do not edit by hand

export(buildPathwaySpace)
export(expDecay)
export(getNearestNode)
export(linearDecay)
export(pathDistances)
export(plotPathDistances)
export(polarDecay)
export(pspace.cols)
export(pspace.pals)
export(signalAggregation)
export(signalDecay)
export(summitWatershed)
export(weibullDecay)
exportClasses(PathwaySpace)
exportMethods("gs_edge_attr<-")
exportMethods("gs_vertex_attr<-")
exportMethods("vertexDecay<-")
exportMethods("vertexSignal<-")
exportMethods(circularProjection)
exportMethods(getPathwaySpace)
exportMethods(plotPathwaySpace)
exportMethods(polarProjection)
exportMethods(silhouetteMapping)
exportMethods(summitMapping)
exportMethods(vertexDecay)
exportMethods(vertexSignal)
import(methods)
importClassesFrom(RGraphSpace,GraphSpace)
importFrom(RANN,nn2)
importFrom(RGraphSpace,"gs_edge_attr<-")
importFrom(RGraphSpace,"gs_vertex_attr<-")
importFrom(RGraphSpace,GraphSpace)
importFrom(RGraphSpace,getGraphSpace)
importFrom(RGraphSpace,gs_ecount)
importFrom(RGraphSpace,gs_edge_attr)
importFrom(RGraphSpace,gs_vcount)
importFrom(RGraphSpace,gs_vertex_attr)
importFrom(RGraphSpace,normalizeGraphSpace)
importFrom(RGraphSpace,plotGraphSpace)
importFrom(RGraphSpace,theme_gspace_coords)
importFrom(colorspace,lighten)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,annotation_raster)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,geom_area)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_raster)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,rel)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_gray)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,as.raster)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,convertColor)
importFrom(grDevices,rgb)
importFrom(grid,unit)
importFrom(igraph,'E<-')
importFrom(igraph,'V<-')
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,as_edgelist)
importFrom(igraph,degree)
importFrom(igraph,distances)
importFrom(igraph,ecount)
importFrom(igraph,is_directed)
importFrom(igraph,is_igraph)
importFrom(igraph,simplify)
importFrom(igraph,vcount)
importFrom(igraph,which_mutual)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecate_stop)
importFrom(lifecycle,deprecated)
importFrom(lifecycle,is_present)
importFrom(patchwork,wrap_plots)
importFrom(scales,rescale)
importFrom(stats,density)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,weighted.mean)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)

## ----Load packages - quick start, eval=TRUE, message=FALSE--------------------
#--- Load required packages for this section
library(igraph)
library(ggplot2)
library(RGraphSpace)
library(PathwaySpace)

## ----Making a toy igraph - 1, eval=TRUE, message=FALSE------------------------
# Make a 'toy' igraph object, either a directed or undirected graph
gtoy1 <- make_star(5, mode="undirected")

# Assign 'x' and 'y' coordinates to each vertex
# ..this can be an arbitrary unit in (-Inf, +Inf)
V(gtoy1)$x <- c(0, 2, -2, -4, -8)
V(gtoy1)$y <- c(0, 0,  2, -4,  0)

# Assign a 'name' to each vertex (here, from n1 to n5)
V(gtoy1)$name <- paste0("n", 1:5)

## ----GraphSpace constructor - 1, eval=TRUE, message=FALSE---------------------
# Check graph validity
gs1 <- GraphSpace(gtoy1)
# Normalize node coordinates 
gs1 <- normalizeGraphSpace(gs1, mar = 0.2)

## ----GraphSpace constructor - 2, eval=FALSE, message=FALSE, out.width="100%"----
# # Check the graph layout
# plotGraphSpace(gs1, add.labels = TRUE)

## ----PathwaySpace constructor - 1, eval=TRUE, message=FALSE-------------------
# Run the PathwaySpace constructor
p_space1 <- buildPathwaySpace(gs1)

## ----PathwaySpace constructor - 2, eval=TRUE, message=FALSE, results='hide'----
# Check the number of vertices in the PathwaySpace object
length(p_space1)
## [1] 5

# Check vertex names
names(p_space1)
## [1] "n1" "n2" "n3" "n4" "n5"

# Check signal (initialized with '0')
vertexSignal(p_space1)
## n1 n2 n3 n4 n5 
##  0  0  0  0  0

## ----PathwaySpace constructor - 3, eval=TRUE, message=FALSE, results='hide'----
# Set new signal to all vertices
vertexSignal(p_space1) <- c(1, 4, 2, 4, 3)

# Set a new signal to the 1st vertex
vertexSignal(p_space1)[1] <- 2

# Set a new signal to vertex "n1"
vertexSignal(p_space1)["n1"] <- 6

# Check updated signal values
vertexSignal(p_space1)
## n1 n2 n3 n4 n5 
##  6  4  2  4  3

## ----Circular projection - 1, eval=FALSE, message=FALSE, out.width="70%"------
# # Run signal projection
# p_space1 <- circularProjection(p_space1, k = 1,
#   decay.fun = weibullDecay(pdist = 0.4))
# 
# # Plot a PathwaySpace image
# plotPathwaySpace(p_space1, add.marks = TRUE)

## ----Circular projection - 3, eval=FALSE, message=FALSE, out.width="70%"------
# # Re-run signal projection, adjusting Weibull's shape
# p_space1 <- circularProjection(p_space1, k = 2,
#   decay.fun = weibullDecay(shape = 2, pdist = 0.2))
# 
# # Plot PathwaySpace
# plotPathwaySpace(p_space1, marks = "n1", theme = "th2")

## ----Polar projection - 1, eval=TRUE, message=FALSE, out.width="100%"---------
# Load a pre-processed directed igraph object
data("gtoy2", package = "RGraphSpace")
# Check graph validity
gs2 <- GraphSpace(gtoy2)
gs2 <- normalizeGraphSpace(gs2, mar = 0.2)

## ----Polar projection - 2, eval=FALSE, message=FALSE, out.width="100%"--------
# # Check the graph layout
# plotGraphSpace(gs2, add.labels = TRUE)

## ----Polar projection - 3, eval=TRUE, message=FALSE---------------------------
# Build a PathwaySpace for the 'gs2'
p_space2 <- buildPathwaySpace(gs2)

# Set '1s' as vertex signal
vertexSignal(p_space2) <- 1

## ----Polar projection - 4, eval=TRUE, message=FALSE---------------------------
# Modify decay function
# ..for all vertices
vertexDecay(p_space2) <- weibullDecay(shape=2, pdist = 1)
# ..for individual vertices
vertexDecay(p_space2)[["n6"]] <- weibullDecay(shape=3, pdist = 1)

## ----Polar projection - 5, eval=FALSE, message=FALSE, out.width="70%"---------
# # Run signal projection using polar coordinates
# p_space2 <- polarProjection(p_space2, beta = 10)
# 
# # Plot PathwaySpace
# plotPathwaySpace(p_space2, theme = "th2", add.marks = TRUE)

## ----Polar projection - 6, eval=FALSE, message=FALSE, out.width="70%"---------
# # Re-run signal projection using 'directional = TRUE'
# p_space2 <- polarProjection(p_space2,
#   beta = 10, directional = TRUE)
# 
# # Plot PathwaySpace
# plotPathwaySpace(p_space2, theme = "th2",
#   marks = c("n1","n3","n4","n5"))

## ----Signal types, eval=FALSE, message=FALSE, out.width="70%"-----------------
# # Set a negative signal to vertices "n3" and "n4"
# vertexSignal(p_space1)[c("n3","n4")] <- c(-2, -4)
# 
# # Check updated signal vector
# vertexSignal(p_space1)
# # n1 n2 n3 n4 n5
# #  6  4 -2 -4  3
# 
# # Re-run signal projection
# p_space1 <- circularProjection(p_space1,
#   decay.fun = weibullDecay(shape = 2))
# 
# # Plot PathwaySpace
# plotPathwaySpace(p_space1, bg.color = "white",
#   font.color = "grey20", add.marks = TRUE,
#   mark.color = "magenta", theme = "th3")

## ----label='Session information', eval=TRUE, echo=FALSE-----------------------
sessionInfo()

## ----Load packages - quick start, eval=TRUE, message=FALSE--------------------
#--- Load required packages for this section
library(igraph)
library(ggplot2)
library(RGraphSpace)
library(PathwaySpace)

## ----Making a toy igraph - 1, eval=TRUE, message=FALSE------------------------
# Make a 'toy' igraph object, either a directed or undirected graph
gtoy1 <- make_star(5, mode="undirected")

# Assign 'x' and 'y' coordinates to each vertex
# ..this can be an arbitrary unit in (-Inf, +Inf)
V(gtoy1)$x <- c(0, 2, -2, -4, -8)
V(gtoy1)$y <- c(0, 0,  2, -4,  0)

# Assign a 'name' to each vertex (here, from n1 to n5)
V(gtoy1)$name <- paste0("n", 1:5)

## ----GraphSpace constructor - 1, eval=TRUE, message=FALSE---------------------
# Check graph validity
gs1 <- GraphSpace(gtoy1)
# Normalize node coordinates 
gs1 <- normalizeGraphSpace(gs1)

## ----GraphSpace constructor - 2, eval=FALSE, message=FALSE, out.width="100%"----
# # Check the graph layout
# plotGraphSpace(gs1, add.labels = TRUE)

## ----PathwaySpace constructor - 1, eval=TRUE, message=FALSE-------------------
# Run the PathwaySpace constructor
p_space1 <- buildPathwaySpace(gs1)

## ----PathwaySpace constructor - 2, eval=TRUE, message=FALSE, results='hide'----
# Check the number of vertices in the PathwaySpace object
length(p_space1)
## [1] 5

# Check vertex names
names(p_space1)
## [1] "n1" "n2" "n3" "n4" "n5"

# Check signal (initialized with '0')
vertexSignal(p_space1)
## n1 n2 n3 n4 n5 
##  0  0  0  0  0

## ----PathwaySpace constructor - 3, eval=TRUE, message=FALSE, results='hide'----
# Set new signal to all vertices
vertexSignal(p_space1) <- c(1, 4, 2, 4, 3)

# Set a new signal to the 1st vertex
vertexSignal(p_space1)[1] <- 2

# Set a new signal to vertex "n1"
vertexSignal(p_space1)["n1"] <- 6

# Check updated signal values
vertexSignal(p_space1)
## n1 n2 n3 n4 n5 
##  6  4  2  4  3

## ----Circular projection - 1, eval=FALSE, message=FALSE, out.width="70%"------
# # Run signal projection
# p_space1 <- circularProjection(p_space1, k = 1,
#   decay.fun = weibullDecay(pdist = 0.4))
# 
# # Plot a PathwaySpace image
# plotPathwaySpace(p_space1, add.marks = TRUE)

## ----Circular projection - 3, eval=FALSE, message=FALSE, out.width="70%"------
# # Re-run signal projection, adjusting Weibull's shape
# p_space1 <- circularProjection(p_space1, k = 2,
#   decay.fun = weibullDecay(shape = 2, pdist = 0.2))
# 
# # Plot PathwaySpace
# plotPathwaySpace(p_space1, marks = "n1", theme = "th2")

## ----Polar projection - 1, eval=TRUE, message=FALSE, out.width="100%"---------
# Load a pre-processed directed igraph object
data("gtoy2", package = "RGraphSpace")
# Check graph validity
gs2 <- GraphSpace(gtoy2)
gs2 <- normalizeGraphSpace(gs2, mar = 0.2)

## ----Polar projection - 2, eval=FALSE, message=FALSE, out.width="100%"--------
# # Check the graph layout
# plotGraphSpace(gs2, add.labels = TRUE)

## ----Polar projection - 3, eval=TRUE, message=FALSE---------------------------
# Build a PathwaySpace for the 'gs2'
p_space2 <- buildPathwaySpace(gs2)

# Set '1s' as vertex signal
vertexSignal(p_space2) <- 1

## ----Polar projection - 4, eval=TRUE, message=FALSE---------------------------
# Modify decay function
# ..for all vertices
vertexDecay(p_space2) <- weibullDecay(shape=2, pdist = 1)
# ..for individual vertices
vertexDecay(p_space2)[["n6"]] <- weibullDecay(shape=3, pdist = 1)

## ----Polar projection - 5, eval=FALSE, message=FALSE, out.width="70%"---------
# # Run signal projection using polar coordinates
# p_space2 <- polarProjection(p_space2, beta = 10)
# 
# # Plot PathwaySpace
# plotPathwaySpace(p_space2, theme = "th2", add.marks = TRUE)

## ----Polar projection - 6, eval=FALSE, message=FALSE, out.width="70%"---------
# # Re-run signal projection using 'directional = TRUE'
# p_space2 <- polarProjection(p_space2,
#   beta = 10, directional = TRUE)
# 
# # Plot PathwaySpace
# plotPathwaySpace(p_space2, theme = "th2",
#   marks = c("n1","n3","n4","n5"))

## ----Signal types, eval=FALSE, message=FALSE, out.width="70%"-----------------
# # Set a negative signal to vertices "n3" and "n4"
# vertexSignal(p_space1)[c("n3","n4")] <- c(-2, -4)
# 
# # Check updated signal vector
# vertexSignal(p_space1)
# # n1 n2 n3 n4 n5
# #  6  4 -2 -4  3
# 
# # Re-run signal projection
# p_space1 <- circularProjection(p_space1,
#   decay.fun = weibullDecay(shape = 2))
# 
# # Plot PathwaySpace
# plotPathwaySpace(p_space1, bg.color = "white",
#   font.color = "grey20", add.marks = TRUE,
#   mark.color = "magenta", theme = "th3")

## ----label='Session information', eval=TRUE, echo=FALSE-----------------------
sessionInfo()

#' @title  Constructor of PathwaySpace-class Objects
#' 
#' @description \code{buildPathwaySpace} is a constructor of
#' PathwaySpace-class objects.
#' 
#' @param gs A \code{\link[RGraphSpace]{GraphSpace}} object. Alternatively, 
#' an \code{\link[igraph]{igraph}} object with node coordinates assigned 
#' to \code{x} and \code{y} vertex attributes, and node labels assigned 
#' to \code{name} vertex attribute.
#' @param nrc A single positive integer indicating the number of rows and 
#' columns (in pixels) for a square image matrix. This argument will 
#' affect the resulting image size and resolution.
#' @param verbose A logical value specifying to display detailed 
#' messages (when \code{verbose=TRUE}) or not (when \code{verbose=FALSE}).
#' @param g Deprecated from PathwaySpace 1.0.1; use 'gs' instead.
#' @return A pre-processed \linkS4class{PathwaySpace} class object.
#' @author Sysbiolab Team
#' @seealso \code{\link[igraph]{undirected_graph}}
#' @examples
#' # Load a demo igraph
#' data('gtoy1', package = 'RGraphSpace')
#' 
#' # Check graph validity
#' gs <- GraphSpace(gtoy1)
#' 
#' gs <- normalizeGraphSpace(gs)
#' 
#' # Create a new PathwaySpace object
#' ps <- buildPathwaySpace(gs, nrc = 100)
#' # note: adjust 'nrc' to increase image resolution
#' 
#' @importFrom igraph degree vcount ecount which_mutual is_igraph
#' @importFrom igraph as_edgelist as_adjacency_matrix
#' @importFrom igraph simplify V E 'V<-' 'E<-' is_directed
#' @importFrom stats quantile sd
#' @importFrom scales rescale
#' @importFrom RGraphSpace GraphSpace getGraphSpace normalizeGraphSpace
#' @importFrom RANN nn2
#' @aliases buildPathwaySpace
#' @export
#' 
buildPathwaySpace <- function(gs, nrc = 500, verbose = TRUE, 
  g = deprecated()) {
  if(verbose) message("Validating arguments...")
  #--- validate argument types
  .validate.ps.args("singleInteger", "nrc", nrc)
  .validate.ps.args("singleLogical", "verbose", verbose)
  ### deprecate
  if (lifecycle::is_present(g)) {
    gs <- g
  }
  ###
  #--- validate argument values
  if (nrc < 2) {
    stop("'nrc' should be >=2", call. = FALSE)
  }
  #--- validate the graph object
  if(is_igraph(gs)){
    if(verbose) message("Validating the 'igraph' object...")
    gs <- GraphSpace(gs, verbose=FALSE)
    gs <- normalizeGraphSpace(gs)
  }
  .validate.gspace(gs)
  if(!gs@pars$is.normalized){
    gs <- normalizeGraphSpace(gs)
  }
    
  #--- build PathwaySpace-class
  ps <- .buildPathwaySpace(gs, nrc, verbose)
  ps <- .updateStatus(ps, "Preprocess")
  return(ps)
}

#' @title Circular Projection of Graph-Associated Signals
#'
#' @description \code{circularProjection} implements a convolution
#' algorithm to project signals onto a 2D-coordinate system.
#'
#' @param ps A \linkS4class{PathwaySpace} class object.
#' @param k A single positive integer determining the k-top signals for the 
#' convolution operation.
#' @param decay.fun A signal decay function. Available options include 
#' 'Weibull', 'exponential', and 'linear' (see \code{\link{weibullDecay}}).
#' Users may also define a custom decay model with at least two arguments, 
#' e.g., \code{function(x, signal) \{ ... \}}, which should returns a vector of  
#' projected signals of the same length as \code{x}. Additional arguments may  
#' include any variable available as a graph vertex attribute.
#' @param aggregate.fun A function used to aggregate the projected signals. 
#' It must be provided as a unary function, e.g., \code{function(x) { ... }}, 
#' which should aggregate a vector of signals to a scalar value. 
#' Available options include 'mean', 'wmean', 'log.wmean', and 'exp.wmean' 
#' (See \code{\link{signalAggregation}}).
#' @param rescale A logical value indicating whether to rescale 
#' the signal. If the signal \code{>=0}, then it will be rescaled to 
#' \code{[0, 1]}; if the signal \code{<=0}, then it will be rescaled to 
#' \code{[-1, 0]}; and if the signal in \code{(-Inf, +Inf)}, then it will be 
#' rescaled to \code{[-1, 1]}.
#' @param verbose A logical value specifying to display detailed 
#' messages (when \code{verbose=TRUE}) or not (when \code{verbose=FALSE}).
#' @param pdist Deprecated as of PathwaySpace 1.0.2; this parameter is now 
#' passed internally through \code{decay.fun}.
#' @return A preprocessed \linkS4class{PathwaySpace} class object.
#' @author Sysbiolab Team
#' @seealso \code{\link{buildPathwaySpace}},  \code{\link{weibullDecay}},  
#' \code{\link{expDecay}}, \code{\link{linearDecay}}
#' @examples
#' # Load a demo igraph
#' data('gtoy1', package = 'RGraphSpace')
#'
#' # Create a new PathwaySpace object
#' ps <- buildPathwaySpace(gtoy1, nrc = 100)
#' # note: adjust 'nrc' to increase image resolution
#' 
#' # Set '1s' as vertex signal
#' vertexSignal(ps) <- 1
#' 
#' # Create a 2D-landscape image
#' ps <- circularProjection(ps)
#' 
#' @import methods
#' @importFrom lifecycle deprecated deprecate_soft is_present deprecate_stop
#' @docType methods
#' @rdname circularProjection-methods
#' @aliases circularProjection
#' @export
#'
setMethod("circularProjection", "PathwaySpace", function(ps, 
  k = 8, decay.fun = weibullDecay(), 
  aggregate.fun = signalAggregation(),
  rescale = TRUE, verbose = TRUE, 
  pdist = deprecated()) {
  ### deprecate
  if (lifecycle::is_present(pdist)) {
    deprecate_soft("1.0.2", "polarProjection(pdist)", 
      "polarProjection(decay.fun)")
  }
  #--- validate the pipeline status
  if (!.checkStatus(ps, "Preprocess")) {
    stop("NOTE: the 'ps' object needs preprocessing!", call. = FALSE)
  }
  if(verbose) message("Validating arguments...")
  #--- validate argument types
  .validate.ps.args("singleInteger", "k", k)
  .validate.ps.args("function", "aggregate.fun", aggregate.fun)
  .validate.ps.args("function", "decay.fun", decay.fun)
  .validate.ps.args("singleLogical", "rescale", rescale)
  .validate.ps.args("singleLogical", "verbose", verbose)
  #--- validate argument values
  if (k < 1) {
    stop("'k' should be >=1", call. = FALSE)
  }
  n <- gs_vcount(ps)
  if (k > n) k <- n
  #--- validate functions
  if(!missing(decay.fun)){
    gs_vertex_attr(ps, "decayFunction") <- decay.fun
  }
  .validate_aggregate_fun(aggregate.fun)
  
  #--- pack args
  pars <- list(k = k, rescale = rescale, 
    aggregate.fun = aggregate.fun, projection = "Circular")
  for (nm in names(pars)) {
    ps@pars$ps[[nm]] <- pars[[nm]]
  }
  #--- run ps pipeline
  ps <- .circularProjection(ps, verbose)
  ps <- .updateStatus(ps, "CircularProjection")
  if (.checkStatus(ps, "PolarProjection")) {
    if(verbose) message("-- polar projection replaced by circular.")
    ps <- .updateStatus(ps, "PolarProjection", FALSE)
  }
  return(ps)
})

#' @title Polar Projection of Graph-Associated Signals
#'
#' @description \code{polarProjection} implements a convolution algorithm
#' to project signals across a 2D-coordinate system.
#'
#' @param ps A \linkS4class{PathwaySpace} class object.
#' @param k A single positive integer determining the k-top signals for the 
#' convolution operation.
#' @param beta An exponent (in \code{[0, +Inf)}) used in the polar 
#' projection functions (see \code{\link{polarDecay}}). It controls the  
#' shape of the polar projection by modulating the angular span.
#' For example, \eqn{beta = 0} yields a circular projection, \eqn{beta = 1} 
#' produces a cardioid-like shape, and \code{beta > 1} progressively narrows 
#' the projection along a reference edge axis.
#' @param decay.fun A signal decay function. Available options include 
#' 'Weibull', 'exponential', and 'linear' (see \code{\link{weibullDecay}}).
#' Users may also define a custom decay model with at least two arguments, 
#' e.g., \code{function(x, signal) \{ ... \}}, which should returns a vector of  
#' projected signals of the same length as \code{x}. Additional arguments may  
#' include any variable available as a graph vertex attribute.
#' @param aggregate.fun A function used to aggregate the projected signals. 
#' It must be provided as a unary function, e.g., \code{function(x) { ... }}, 
#' which should aggregate a vector of signals to a scalar value. 
#' Available options include 'mean', 'wmean', 'log.wmean', and 'exp.wmean' 
#' (See \code{\link{signalAggregation}}).
#' @param polar.fun A polar decay function (see \code{\link{polarDecay}}).
#' @param directional If directional edges are available, this argument can 
#' be used to orientate the signal projection on directed graphs.
#' @param edge.norm Scale distances based on edge lengths 
#' (when \code{edge.norm=TRUE}) or based on full coordinate space 
#' (when \code{edge.norm=FALSE}).
#' @param rescale A logical value indicating whether to rescale 
#' the signal. If the signal \code{>=0}, then it will be rescaled to 
#' \code{[0, 1]}; if the signal \code{<=0}, then it will be rescaled to 
#' \code{[-1, 0]}; and if the signal in \code{(-Inf, +Inf)}, then it will be 
#' rescaled to \code{[-1, 1]}.
#' @param verbose A logical value specifying to display detailed 
#' messages (when \code{verbose=TRUE}) or not (when \code{verbose=FALSE}).
#' @param theta Deprecated as of PathwaySpace 1.0.2; use 'beta' instead.
#' @param pdist Deprecated as of PathwaySpace 1.0.2; this parameter is now 
#' passed internally through \code{decay.fun}.
#' @return A preprocessed \linkS4class{PathwaySpace} class object.
#' @author Sysbiolab Team
#' @seealso \code{\link{buildPathwaySpace}}
#' @examples
#' # Load a demo igraph
#' data('gtoy2', package = 'RGraphSpace')
#' 
#' # Create a new PathwaySpace object
#' ps <- buildPathwaySpace(gtoy2, nrc = 100)
#' # note: adjust 'nrc' to increase image resolution
#' 
#' # Set '1s' as vertex signal
#' vertexSignal(ps) <- 1
#' 
#' # Set edge weight
#' # gs_edge_attr(ps, "weight") <- c(-1, 1, 1, 1, 1, 1)
#' 
#' # Create a 2D-landscape image
#' ps <- polarProjection(ps, pdist=1)
#' 
#' @import methods
#' @docType methods
#' @rdname polarProjection-methods
#' @aliases polarProjection
#' @export
#'
setMethod("polarProjection", "PathwaySpace", function(ps, 
  k = 2, beta = 10,
  decay.fun = weibullDecay(pdist = 1),
  aggregate.fun = signalAggregation(), 
  polar.fun = polarDecay(), 
  directional = FALSE,
  edge.norm = TRUE,
  rescale = TRUE, 
  verbose = TRUE, 
  theta = deprecated(),
  pdist = deprecated()) {
  #--- validate the pipeline status
  if (!.checkStatus(ps, "Preprocess")) {
    stop("NOTE: the 'ps' object needs preprocessing!", call. = FALSE)
  }
  ### deprecate
  if (lifecycle::is_present(theta)) {
    deprecate_soft("1.0.2", "polarProjection(theta)", 
      "polarProjection(beta)")
  }
  if (lifecycle::is_present(pdist)) {
    deprecate_soft("1.0.2", "polarProjection(pdist)", 
      "polarProjection(decay.fun)")
  }
  ###
  if(verbose) message("Validating arguments...")
  .validate.ps.args("singleInteger", "k", k)
  .validate.ps.args("singleNumber", "beta", beta)
  .validate.ps.args("function", "decay.fun", decay.fun)
  .validate.ps.args("function", "aggregate.fun", aggregate.fun)
  .validate.ps.args("function", "polar.fun", polar.fun)
  .validate.ps.args("singleLogical", "directional", directional)
  .validate.ps.args("singleLogical", "edge.norm", edge.norm)
  .validate.ps.args("singleLogical", "rescale", rescale)
  .validate.ps.args("singleLogical", "verbose", verbose)
  if (k < 1) {
    stop("'k' should be >=1", call. = FALSE)
  }
  n <- gs_vcount(ps)
  if (k > n) k <- n
  if (beta < 0) {
    msg <- paste0("'beta' should be an exponent in [0,+Inf)")
    stop(msg, call. = FALSE)
  }
  
  #--- validate functions
  if(!missing(decay.fun)){
    gs_vertex_attr(ps, "decayFunction") <- decay.fun
  }
  .validate_aggregate_fun(aggregate.fun)
  .validate_polar_fun(polar.fun)
  
  #--- pack args
  pars <- list(k = k, beta = beta,
    edge.norm = edge.norm, rescale = rescale, directional = directional,
    polar.fun = polar.fun, aggregate.fun = aggregate.fun, 
    projection = "Polar")
  for (nm in names(pars)) {
    ps@pars$ps[[nm]] <- pars[[nm]]
  }
  ps <- .polarProjection(ps, verbose)
  ps <- .updateStatus(ps, "PolarProjection")
  if (.checkStatus(ps, "CircularProjection")) {
    if(verbose) message("-- circular projection replaced by polar.")
    ps <- .updateStatus(ps, "CircularProjection", FALSE)
  }
  return(ps)
})

#' @title Decorating PathwaySpace Images with Graph Silhouettes
#'
#' @description \code{silhouetteMapping} constructs an image baseline used
#' to outline the graph layout in a PathwaySpace image.
#'
#' @param ps A \linkS4class{PathwaySpace} class object.
#' @param pdist A term (in \code{[0,1]}) determining a distance unit for the
#' silhouette projection.
#' @param baseline A fraction (in \code{[0,1]}) of the silhouette projection,
#' representing the level over which a silhouette will outline the graph layout.
#' When \code{baseline = 0} (i.e. lower level of the projection), the 
#' silhouette will extend over the entire image space, so no outline will 
#' be visible.
#' @param fill.cavity A logical value specifying to fill cavities 
#' in the silhouette mask (when \code{fill.cavity=TRUE}) or not 
#' (when \code{fill.cavity=FALSE}).
#' @param verbose A logical value specifying to display detailed 
#' messages (when \code{verbose=TRUE}) or not (when \code{verbose=FALSE}).
#' @return A preprocessed \linkS4class{PathwaySpace} class object.
#' @author Sysbiolab Team
#' @seealso \code{\link{circularProjection}}
#' @examples
#' # Load a demo igraph
#' data('gtoy1', package = 'RGraphSpace')
#'
#' # Create a new PathwaySpace object
#' ps <- buildPathwaySpace(gtoy1, nrc = 100)
#' # note: adjust 'nrc' to increase image resolution
#' 
#' # Set '1s' as vertex signal
#' vertexSignal(ps) <- 1
#'
#' # Map graph silhouette
#' ps <- silhouetteMapping(ps, pdist = 0.1)
#'
#' @import methods
#' @docType methods
#' @rdname silhouetteMapping-methods
#' @aliases silhouetteMapping
#' @export
#'
setMethod("silhouetteMapping", "PathwaySpace", function(ps,
  pdist = 0.05, baseline = 0.01, fill.cavity = TRUE, verbose = TRUE) {
  #--- validate the pipeline status
  if (!.checkStatus(ps, "Preprocess")) {
    stop("NOTE: the 'ps' object needs preprocessing!", call. = FALSE)
  }
  if(verbose) message("Validating arguments...")
  #--- validate argument types
  .validate.ps.args("singleNumber", "pdist", pdist)
  .validate.ps.args("singleNumber", "baseline", baseline)
  .validate.ps.args("singleLogical", "fill.cavity", fill.cavity)
  .validate.ps.args("singleLogical", "verbose", verbose)
  #--- validate argument values
  if (baseline < 0 || baseline > 1) {
    stop("'baseline' should be in [0,1]", call. = FALSE)
  }
  if (pdist < 0 || pdist > 1) {
    stop("'pdist' should be in [0,1]", call. = FALSE)
  }
  #--- pack args (for default projection)
  k <- min(8, gs_vcount(ps))
  pars <- list(baseline = baseline, pdist = pdist, k = k, 
    fill.cavity = fill.cavity, 
    decay.fun = weibullDecay(pdist=1))
  for (nm in names(pars)) {
    ps@pars$ps$silh[[nm]] <- pars[[nm]]
  }
  #--- run ps pipeline
  if(verbose) message("Mapping graph silhouette...")
  ps <- .silhouetteCircular(ps, verbose)
  ps <- .updateStatus(ps, "Silhouette")
  return(ps)
})

#' @title Mapping Summits on PathwaySpace Images
#'
#' @description The \code{summitMapping} method implements a segmentation
#' strategy to identify summits on a 2D-landscape image 
#' (see \code{\link{summitWatershed}}).
#'
#' @param ps A \linkS4class{PathwaySpace} class object.
#' @param maxset A single positive integer indicating the maximum number 
#' of summits to be returned by the segmentation function.
#' @param minsize A single positive integer indicating the minimum size 
#' of the summits.
#' @param threshold A threshold provided as a fraction (in \code{[0,1]}) of the
#' max signal intensity.
#' @param verbose A logical value specifying to display detailed 
#' messages (when \code{verbose=TRUE}) or not (when \code{verbose=FALSE}).
#' @param segm_fun A segmentation function used to detect summits
#' (see \code{\link{summitWatershed}}).
#' @param ... Additional arguments passed to the segmentation function.
#' @return A preprocessed \linkS4class{PathwaySpace} class object.
#' @author Sysbiolab Team
#' @seealso \code{\link{circularProjection}}
#' @examples
#' # Load a large igraph
#' data("PCv12_pruned_igraph", package = "PathwaySpace")
#' 
#' # Continue this example from the PathwaySpace vignette,
#' # in the 'PathwaySpace decoration' section
#'
#' @import methods
#' @docType methods
#' @rdname summitMapping-methods
#' @aliases summitMapping
#' @export
#'
setMethod("summitMapping", "PathwaySpace", function(ps, maxset = 30, 
    minsize = 30, threshold = 0.5, verbose = TRUE, 
    segm_fun = summitWatershed, ...) {
    #--- validate the pipeline status
    if (!.checkStatus(ps, "Projection")) {
        msg <- paste0("NOTE: the 'ps' object needs to be\n",
            "evaluated by a 'projection' method!")
        stop(msg, call. = FALSE)
    }
    #--- validate argument types
    .validate.ps.args("singleInteger", "maxset", maxset)
    .validate.ps.args("singleInteger", "minsize", minsize)
    .validate.ps.args("singleNumber", "threshold", threshold)
    .validate.ps.args("singleLogical", "verbose", verbose)
    .validate.ps.args("function", "segm_fun", segm_fun)
    #--- validate argument values
    if (maxset < 1) {
        stop("'maxset' should be >=1", call. = FALSE)
    }
    if (minsize < 1) {
        stop("'minsize' should be >=1", call. = FALSE)
    }
    if (threshold < 0 || threshold > 1) {
        stop("'threshold' should be in [0,1]", call. = FALSE)
    }
    #--- pack args
    pars <- list(maxset = maxset, minsize = minsize,
        summit_threshold = threshold, segm_fun = segm_fun,
        segm_arg = list(...=...))
    for (nm in names(pars)) {
        ps@pars$ps$summit[[nm]] <- pars[[nm]]
    }
    #--- run ps pipeline
    ps <- .summitMapping(ps, verbose, ...=...)
    ps <- .updateStatus(ps, "Summits")
    return(ps)
})

#' @title Accessors for Fetching Slots from a PathwaySpace Object
#'
#' @description \code{getPathwaySpace} retrives information from
#' individual slots available in a PathwaySpace object.
#'
#' @param ps A preprocessed \linkS4class{PathwaySpace} class object
#' @param what A character value specifying which information should 
#' be retrieved from the slots.
#' Options: "nodes", "edges", "graph", "image", "pars", "misc", 
#' "signal","projections", "status", "silhouette", "summits", 
#' "summit_mask", "summit_contour"
#' @return Content from slots in the \linkS4class{PathwaySpace} object.
#' @examples
#' # Load a demo igraph
#' data('gtoy1', package = 'RGraphSpace')
#'
#' # Create a new PathwaySpace object
#' ps <- buildPathwaySpace(gtoy1, nrc = 100)
#' # note: adjust 'nrc' to increase image resolution
#'
#' # Get the 'status' slot in ps
#' status <- getPathwaySpace(ps, what = 'status')
#'
#' @import methods
#' @docType methods
#' @rdname getPathwaySpace-methods
#' @aliases getPathwaySpace
#' @export
setMethod("getPathwaySpace", "PathwaySpace", function(ps, what = "status") {
    opts <- c("nodes", "edges", "graph", "image", "pars", "misc", 
      "projections", "status", "signal", "silhouette", "summits", 
      "summit_mask", "summit_contour")
    if (!what %in% opts) {
        opts <- paste0(opts, collapse = ", ")
        stop("'what' must be one of:\n", opts, call. = FALSE)
    }
    if (what == "nodes") {
        obj <- ps@nodes
    } else if (what == "edges") {
        obj <- ps@edges
    } else if (what == "graph") {
      obj <- ps@graph   
    } else if (what == "image") {
      obj <- ps@image   
    } else if (what == "pars") {
      obj <- ps@pars
    } else if (what == "misc") {
      obj <- ps@misc
    } else if (what == "projections") {
        obj <- ps@projections
    } else if (what == "status") {
        obj <- ps@status
    } else if (what == "signal") {
      obj <- gs_vertex_attr(ps, "signal")
    } else if (what == "silhouette") {
        obj <- ps@projections$xfloor
    } else if (what == "summits") {
        obj <- ps@projections$summits$lset
    } else if (what == "summit_mask") {
        obj <- ps@projections$summits$mset
    } else if (what == "summit_contour") {
        obj <- ps@projections$summits$cset
    }
    return(obj)
})

################################################################################
### Accessors
################################################################################

#-------------------------------------------------------------------------------
# show summary information on screen
setMethod("show", "PathwaySpace", function(object) {
  message("A PathwaySpace-class object for:")
  summary(getGraphSpace(object, what = "graph"))
  cat("+ status:", .summariseStatus(object))
})

#-------------------------------------------------------------------------------
#' @title Accessor Functions for PathwaySpace Objects
#'
#' @description Get or set 'signal' and 'decay' functions in a 
#' \linkS4class{PathwaySpace} class object.
#'
#' @param x A \linkS4class{PathwaySpace} class object.
#' @param value The new value of the attribute.
#' @return Updated \linkS4class{PathwaySpace} object.
#' @examples
#' data('gtoy1', package = 'RGraphSpace')
#' ps <- buildPathwaySpace(gtoy1, nrc = 100)
#' 
#' # Check vertex names
#' names(ps)
#' 
#' # Access signal values from all vertices
#' vertexSignal(ps)
#' 
#' # Modify signal value of a specific vertex
#' vertexSignal(ps)[1] <- 1
#' 
#' # Modify signal value of specific vertices
#' vertexSignal(ps)[c("n2","n3")] <- 1
#' 
#' # Set '1s' to all vertices
#' vertexSignal(ps) <- 1
#' 
#' #----
#' 
#' # Access decay function of a specific vertex
#' vertexDecay(ps)[["n3"]]
#' 
#' # Modify decay function of a specific vertex
#' vertexDecay(ps)[["n3"]] <- linearDecay()
#' 
#' # Modify decay functions of two vertices
#' vertexDecay(ps)[c("n1","n3")] <- list( weibullDecay() )
#' 
#' # Modify decay functions of all vertices
#' vertexDecay(ps) <- weibullDecay(shape = 2)
#' 
#' @import methods
#' @docType methods
#' @rdname vertexSignal-accessors
#' @aliases vertexSignal
#' @aliases vertexSignal<-
#' @aliases vertexDecay
#' @aliases vertexDecay<-
#' @export
setMethod("vertexSignal", "PathwaySpace", function(x){
  gs_vertex_attr(x, "signal")
})

#' @rdname vertexSignal-accessors
#' @export
setMethod("vertexSignal<-", "PathwaySpace",
  function(x, value) {
    if (!is.numeric(value) || !is.vector(value)){
      stop("'value' must be a numeric vector or scalar.", call. = FALSE)
    }
    gs_vertex_attr(x, "signal") <- value
    return(x)
  }
)

#' @rdname vertexSignal-accessors
#' @export
setMethod("vertexDecay", "PathwaySpace", function(x){
  gs_vertex_attr(x, "decayFunction")
})

#' @rdname vertexSignal-accessors
#' @export
setMethod("vertexDecay<-", "PathwaySpace",
  function(x, value) {
    gs_vertex_attr(x, "decayFunction") <- value
    return(x)
  }
)

#-------------------------------------------------------------------------------
#' @title Accessor Functions for PathwaySpace Objects
#'
#' @description Get or set edge and vertex attributes in
#' \linkS4class{PathwaySpace} class object.
#'
#' @param x A \linkS4class{PathwaySpace} class object.
#' @param name Name of the attribute.
#' @param value The new value of the attribute.
#' @param ... Additional arguments passed to igraph methods.
#' @return Updated \linkS4class{PathwaySpace} object.
#' @examples
#' data('gtoy1', package = 'RGraphSpace')
#' ps <- buildPathwaySpace(gtoy1, nrc = 100)
#' 
#' # Get vertex count
#' gs_vcount(ps)
#' 
#' # Get edge count
#' gs_ecount(ps)
#' 
#' # Access a specific vertex attribute
#' gs_vertex_attr(ps, "signal")
#' 
#' # Replace an entire vertex attribute
#' gs_vertex_attr(ps, "signal") <- 1
#' 
#' # Modify a single value within a vertex attribute
#' gs_vertex_attr(ps, "signal")["n1"] <- 1
#'
#' # Access a specific edge attribute
#' gs_edge_attr(ps, "weight")
#' 
#' # Replace an entire edge attribute
#' gs_edge_attr(ps, "weight") <- 1
#' 
#' @rdname PathwaySpace-accessors
#' @importFrom RGraphSpace gs_vertex_attr<- gs_edge_attr<-
#' @importFrom RGraphSpace gs_vertex_attr gs_edge_attr
#' @importFrom RGraphSpace gs_ecount gs_vcount
#' @aliases gs_vertex_attr<-
#' @export
setReplaceMethod(
  "gs_vertex_attr","PathwaySpace", function(x, name, ..., value) {
    
    # Call the GraphSpace method first
    x <- callNextMethod()
    
    x <- .validate_ps_containers(x)
    
    return(x)
  }
)

#' @rdname PathwaySpace-accessors
#' @aliases gs_edge_attr<-
#' @export
setReplaceMethod(
  "gs_edge_attr","PathwaySpace", function(x, name, ..., value) {
    
    # Call the GraphSpace method first
    x <- callNextMethod()
    
    x <- .validate_ps_containers(x)
    
    return(x)
  }
)

#-------------------------------------------------------------------------------
.validate_ps_containers <- function(ps) {
  ps <- .validate_signal(ps)
  ps <- .validate_weights(ps)
  ps <- .validate_decayFunction(ps)
  return(ps)
}

#-------------------------------------------------------------------------------
.validate_signal <- function(ps) {
  signal <- gs_vertex_attr(ps, "signal")
  if(is.null(signal)){
    stop("'signal' vertex attribute must be available.", call. = FALSE)
  }
  if (!is.numeric(signal)){
    stop("vertex 'signal' variable must be numeric.", call. = FALSE)
  }
  ps@nodes$signal <- .revise_signal(signal)
  return(ps)
}
.revise_signal <- function(x){
  x[is.nan(x)] <- NA
  x[x == Inf] <- NA
  x[x == -Inf] <- NA
  if (all(is.na(x))) x[] <- 0
  return(x)
}

#-------------------------------------------------------------------------------
.validate_weights <- function(ps) {
  if(gs_ecount(ps)>0){
    weight <- gs_edge_attr(ps, "weight")
    if(is.null(weight)){
      stop("'weight' edge attribute must be available.", call. = FALSE)
    }
    if (!is.numeric(weight)){
      stop("edge 'weight' variable must be numeric.", call. = FALSE)
    }
    ps@edges$weight <- .revise_weights(weight)
  }
  return(ps)
}
.revise_weights <- function(wt){
  if (all(is.na(wt))) wt[] <- 1
  if (sd(wt, na.rm = TRUE) != 0) {
    wt <- wt/max(abs(wt), na.rm = TRUE)
    wt[is.na(wt)] <- 0
  } else {
    wt[] <- 1
  }
  return(wt)
}

#-------------------------------------------------------------------------------
.validate_decayFunction <- function(ps){
  att <- names(gs_vertex_attr(ps))
  if(! "decayFunction" %in% att){
    stop("Missing a vertex 'decayFunction' attribute.", call. = FALSE)
  }
  decayFunction <- gs_vertex_attr(ps, "decayFunction")
  # check function, args, and vertex attributes
  lg <- unlist(lapply(decayFunction, is.function))
  if(!all(lg)){
    msg1 <- "Each vertex 'decay function' must be a function, e.g.,\n"
    msg2 <- "function(x, signal) { ... }"
    stop(msg1, msg2, call. = FALSE)
  }
  not_used <- lapply(decayFunction, .check_decay_args, nodes=ps@nodes)
  if(.all_equal_fun(decayFunction)){
    dfun <- attributes(decayFunction[[1]])$name
    dfun <- ifelse(.is_singleString(dfun), dfun, "customized")
    ps@pars$ps$decay$fun <- dfun
    ps@pars$ps$decay$info <- "global-defined-decay"
  } else {
    ps@pars$ps$decay$fun <- "customized"
    ps@pars$ps$decay$info <- "local-defined-decay"
  }
  ps@pars$ps$decay$is_default_args <- .is_default_args(decayFunction)
  return(ps)
}
.check_decay_args <- function(decay_fun, nodes, args = c("x","signal")){
  fargs <- formalArgs(args(decay_fun))
  missing_args <- setdiff(args, fargs)
  if (length(missing_args) > 0) {
    msg <- paste0("Invalid 'decay function':")
    msg <- paste0(msg, " expected arguments 'x' and 'signal' not found.")
    stop(msg, call. = FALSE)
  }
  decay_args <- c(args, setdiff(fargs, args))
  
  if(!all(decay_args %in% colnames(nodes))){
    extra_args <- decay_args[!decay_args %in% colnames(nodes)]
    msg1 <- "Each 'decay function' argument must correspond to a vertex attribute.\n"
    msg2 <- paste0(sQuote(extra_args, q=FALSE), collapse = ", ")
    msg2 <- paste0("The following argument(s) do not match any vertex attribute: ",
      msg2)
    stop(msg1, msg2, call. = FALSE)
  }
  TRUE
}
.is_default_args <- function(decayFunction, args = c("x","signal")){
  fargs <- lapply(decayFunction, formalArgs)
  n_args <- unlist(lapply(fargs, length))
  if(all(n_args==length(args))){
    fargs <- as.character(unlist(fargs))
    is_default <- all( fargs %in% args )
  } else {
    is_default <- FALSE
  }
  return(is_default)
}
.all_equal_fun <- function(lst){
  all(unlist(lapply(lst[-1], function(x) identical(x, lst[[1]]))))
}

# Tests for PathwaySpace-class methods
test_that("Check PathwaySpace-class methods", {
  data("gtoy1", package = "RGraphSpace")
  pts <- buildPathwaySpace(gtoy1, nrc = 30)
  expect_true(is(pts, "PathwaySpace"))
})

################################################################################
### Main constructor of PathwaySpace-class objects
################################################################################
.buildPathwaySpace <- function(gs, nrc = 500, verbose = TRUE) {
  
    #--- update pars
    gs@pars$ps$nrc <- nrc
    
    #--- initialize vertex signal
    att <- names(gs_vertex_attr(gs))
    if( ! "signal" %in% att ){
      gs_vertex_attr(gs, "signal") <- 0
    }
    
    #--- initialize vertex function
    att <- names(gs_vertex_attr(gs))
    if( ! "decayFunction" %in% att ){
      gs_vertex_attr(gs, "decayFunction") <- weibullDecay()
    }
    
    #--- initialize edge weight
    att <- names(gs_edge_attr(gs))
    if( ! "weight" %in% att ){
      gs_edge_attr(gs, "weight") <- 1
    }
    
    #--- create a PathwaySpace object
    if(verbose) message("Creating a 'PathwaySpace' object...")
    ps <- as(gs, "PathwaySpace")
        
    #--- initialize status
    pnames <- c("Preprocess", "CircularProjection", "PolarProjection",
      "Silhouette", "Summits")
    status <- rep("[ ]", length(pnames))
    names(status) <- pnames
    ps@status <- status
    
    ps <- .validate_ps_containers(ps)
    
    return(ps)
}

################################################################################
### Main circularProjection calls
################################################################################
.circularProjection <- function(ps, verbose = TRUE) {
  
  if(verbose) message("Using circular projection...")
  pars <- getPathwaySpace(ps, "pars")
  nodes <- getPathwaySpace(ps, "nodes")
  pars$ps$zscale <- .get_signal_scale(nodes$signal)
  # baseline will put a stop to signal decay (in [0,1])
  # ...used in the .get_near_points() function
  pars$ps$baseline <- 0.01
  
  if(verbose) message("Mapping 'x' and 'y' coordinates...")
  gxy <- .rescale_coord(nodes, pars$ps$nrc)
  lpts <- .get_points_in_matrix(pars$ps$nrc)
  nnpg <- .get_near_points(lpts, gxy, nodes, pars)
  
  # project signal
  if(verbose) message("Running signal convolution...")
  xsig <- array(0, c(pars$ps$nrc, pars$ps$nrc))
  if (nrow(nodes) > 0) {
    xsig[lpts[, c("Y", "X")]] <- .get_ldsig(nodes, pars, nnpg, lpts)
  }
  # image(.transpose_and_flip(xsig))
  
  # add projections
  ps@projections$gxy <- gxy
  ps@projections$xsig <- xsig
  ps <- .update_projections(ps, pars)
  return(ps)
}

#-------------------------------------------------------------------------------
.get_ldsig <- function(nodes, pars, nnpg, lpts) {
  if (pars$ps$zscale$maxsig == 0) {
    return(0)
  }
  nsig <- list()
  if(pars$ps$decay$is_default_args){
    for(i in seq_len(nrow(nodes))){
      decay_fun <- nodes$decayFunction[[i]]
      signal <- nodes$signal[i] / pars$ps$zscale$maxsig
      x <- nnpg$dist[[i]] / pars$ps$nrc
      nsig[[i]] <- decay_fun(x=x, signal=signal)
    }
  } else {
    # Use do.call (slower, but flexible)
    for(i in seq_len(nrow(nodes))){
      decay_fun <- nodes$decayFunction[[i]]
      decay_args <- formalArgs(decay_fun)
      args_list <- as.list(nodes[i, decay_args, drop=FALSE])
      args_list$x <- nnpg$dist[[i]] / pars$ps$nrc
      args_list$signal <- args_list$signal / pars$ps$zscale$maxsig
      nsig[[i]] <- do.call(decay_fun, args_list)
    }
  }
  # vectorize and sort projected signals
  nsig <- unlist(nsig)
  nn <- unlist(nnpg$nn)
  idx <- order(-abs(nsig))
  nsig <- nsig[idx]
  nn <- nn[idx]
  # pack into a list
  nnu <- unique(nn)
  nnl <- .find_nnu_positions(nnu, nn)
  dsig_lt <- as.list(rep(0, times = nrow(lpts)))
  for(i in seq_along(nnl)){
    dsig_lt[[nnu[i]]] <- nsig[nnl[[i]]]
  }
  # aggregate
  Z <- .summ_dsig_lt(dsig_lt, pars)
  return(Z)
}

#-------------------------------------------------------------------------------
#--- aggregate projected signals
.summ_dsig_lt <- function(dsig_lt, pars) {
  m <- max(unlist(lapply(dsig_lt, length)))
  k <- min(pars$ps$k, m)
  Z <- sapply(dsig_lt, function(lt){
    n <- length(lt)
    if(n < k){
      lt <- c(lt, integer(k - n) )
    } else {
      lt <- lt[seq_len(k)]
    }
    z <- pars$ps$aggregate.fun(lt)
    return(z)
  })
  # Note: NaNs may result from user's customized functions when signal 
  # values are only 0s
  Z[is.na(Z)] <- 0
  return(Z)
}

#-------------------------------------------------------------------------------
#--- aggregate projected signals
.summ_dsig_mt <- function(dsig_mt, pars) {
  if (pars$ps$k > 1) {
    k <- min(pars$ps$k, ncol(dsig_mt))
    dsig_mt <- dsig_mt[, seq_len(k), drop = FALSE]
    Z <- apply(dsig_mt, 1, pars$ps$aggregate.fun)
  } else {
    Z <- dsig_mt[, 1]
  }
  # Note: NaNs may result from user's customized functions when signal 
  # values are only 0s
  Z[is.na(Z)] <- 0
  return(Z)
}

#-------------------------------------------------------------------------------
# Find positions of unique 'nnu' in 'nn'
.find_nnu_positions <- function(nnu, nn) {
  .find_positions <- function(i, nn_idx, starts, ends){
    nn_idx[seq(starts[i], ends[i])]
  }
  nn_idx <- order(nn)
  nn_ord <- nn[nn_idx]
  nn_rev <- rev(nn_ord)
  starts <- match(nnu, nn_ord)
  ends <- length(nn_rev) - match(nnu, nn_rev) + 1
  lapply(seq_along(nnu), function(i) {
    .find_positions(i, nn_idx, starts, ends)
  } )
}

#-------------------------------------------------------------------------------
#--- get vertex-to-point distances for circular projections
# .get_near_neighbours <- function(lpts, gxy, nodes, nrc, pdist){
#   eradius <- .estimate_radius(nodes, pdist)
#   eradius <- nrc * eradius
#   ksearch <- ceiling(pi * max(eradius)^2)
#   nnpg <- RANN::nn2(lpts[, c("X", "Y")], gxy[, c("X", "Y")], k = ksearch)
#   names(nnpg) <- c("nn", "dist")
#   nnpg$nn <- lapply(seq_len(nrow(nnpg$nn)), function(i){nnpg$nn[i,]} )
#   nnpg$dist <- lapply(seq_len(nrow(nnpg$dist)), function(i){nnpg$dist[i,]} )
#   return(nnpg)
# }

#-------------------------------------------------------------------------------
#--- get vertex-to-point distances for circular and polar projections
.get_near_points <- function(lpts, gxy, nodes, pars){
  eradius <- .estimate_radius(nodes, pars$ps$baseline)
  eradius <- pars$ps$nrc * eradius
  nnpg <- list(nn = list(), dist = list())
  lpts <- as.data.frame(lpts)
  blocks <- unique(lpts$X)
  first <- match(blocks, lpts$X)
  last  <- length(lpts$X) - match(blocks, rev(lpts$X)) + 1
  blocks <- data.frame(value = blocks, first, last)
  for(i in seq_len(nrow(gxy))){
    r <- eradius[i]
    if(r>0){
      p <- gxy[i, ]
      x_low <- max(ceiling(p["X"] - r), 1)
      x_high <- min(floor(p["X"] + r), nrow(blocks))
      lpts_filt <- lpts[seq(blocks[x_low, "first"], blocks[x_high, "last"]), ]
      d2 <- (lpts_filt$X - p["X"])^2 + (lpts_filt$Y - p["Y"])^2
      within_radius <- which(d2 <= r^2)
      if(length(within_radius)>0){
        nn_dists <- sqrt(d2[within_radius])
        nearby_idx <- lpts_filt[within_radius, ]
        nn_idx <- with(nearby_idx, (X - 1) * pars$ps$nrc + Y)
      } else {
        nn_idx <- numeric()
        nn_dists <- numeric()
      }
    } else {
      nn_idx <- numeric()
      nn_dists <- numeric()
    }
    nnpg$nn[[i]] <- nn_idx
    nnpg$dist[[i]] <- nn_dists
  }
  return(nnpg)
}
.estimate_radius <- function(nodes, baseline = 0.01){
  x <- seq(1, 0, length.out=100)
  fun <- nodes$decayFunction
  signal <- abs(nodes$signal)
  signal[is.na(signal)] <- 0
  mx <- max(signal)
  if(mx!=0) signal <- signal/mx
  eradius <- vapply(seq_along(fun), function(i){
    f <- fun[[i]]
    s <- signal[i]
    if(s==0){
      r <- 0
    } else {
      xs <- vapply(x, function(l){ f(l, s) }, numeric(1))
      idx <- findInterval(baseline, xs)[1]
      r <- ifelse(idx>0, x[idx], x[1])
    }
    return(r)
  }, numeric(1))
  return(eradius)
}

################################################################################
### Main polarProjection calls
################################################################################
.polarProjection <- function(ps, verbose = TRUE) {
  
    nodes <- getPathwaySpace(ps, "nodes")
    edges <- getPathwaySpace(ps, "edges")
    if(nrow(edges)==0){
        msg <- paste0("Note, this 'PathwaySpace' object does not ",
            "contain edges.\nThe 'polarProjection' method requires ",
            "at least one edge for projection.")
        stop(msg)
    }
    pars <- getPathwaySpace(ps, "pars")
    signal <- .get_edge_signal(nodes, edges)
    pars$ps$zscale <- .get_signal_scale(signal)
    # baseline will put a stop to signal decay (in [0,1])
    # ...used in the .get_near_points() function
    pars$ps$baseline <- 0.01
    
    if(pars$ps$directional){
        if(pars$is.directed){
            message("Using polar projection on directed graph...")
        } else {
            stop("'directional' used with undirected graph.")
        }
    } else {
        if(verbose) message("Using polar projection on undirected graph...")
    }
    
    if(verbose) message("Mapping 'x' and 'y' coordinates...")
    gxy <- .rescale_coord(nodes, pars$ps$nrc)
    
    # 'weight' will adjust signals when variable or different from default
    if (.is_variable(edges$weight) || any(edges$weight != 1)) {
      if(verbose) message("Scaling projection to edge weight...")
      pars$eweight <- TRUE
    } else {
      pars$eweight <- FALSE
    }
    
    if(verbose) message("Computing linear and angular distances...")
    # for polar, 'dist' is scaled to edge dist and polar coordinates
    edges$edist <- .get_edge_dist(edges, gxy)
    edges$emode <- .get_emode(edges$arrowType)
    lpts <- .get_points_in_matrix(pars$ps$nrc)
    nnpg <- .get_near_points(lpts, gxy, nodes, pars)
    nnpg <- .get_angular_dist(nnpg, lpts, gxy, edges, pars)
    nnpg <- .scale_dist_polar(nnpg, pars)
    
    # project signal
    if(verbose) message("Running signal convolution...")
    xsig <- array(0, c(pars$ps$nrc, pars$ps$nrc))
    if (nrow(gxy) > 0) {
      xsig[lpts[, c("Y", "X")]] <- .get_ldsig_polar(nodes, pars, nnpg, lpts)
    }
    # image(.transpose_and_flip(xsig))
    
    # add projections
    ps@projections$gxy <- gxy
    ps@projections$xsig <- xsig
    ps <- .update_projections(ps, pars)
    return(ps)
}
.get_edge_signal <- function(nodes, edges){
  c( nodes[edges$vertex1,"signal"] * edges$weight,
  nodes[edges$vertex2,"signal"] * edges$weight)
}

#-------------------------------------------------------------------------------
.get_emode <- function(arrow_type){
  emode <- abs(arrow_type)
  emode[emode>3] <- 3
  return(emode)
}

#-------------------------------------------------------------------------------
.get_ldsig_polar <- function(nodes, pars, nnpg, lpts) {
  if (pars$ps$zscale$maxsig == 0) {
    return(0)
  }
  nsig <- list()
  if(pars$ps$decay$is_default_args){
    for(i in seq_len(nrow(nodes))){
      decay_fun <- nodes$decayFunction[[i]]
      signal <- nodes$signal[i]
      x <- nnpg$dist[[i]] / (nnpg$dist_dth[[i]] * pars$ps$nrc)
      signal <- signal / pars$ps$zscale$maxsig
      if(pars$eweight) signal <- signal * nnpg$edwght[[i]]
      nsig[[i]] <- decay_fun(x=x, signal=signal)
    }
  } else {
    # Use do.call (slower, but flexible)
    for(i in seq_len(nrow(nodes))){
      decay_fun <- nodes$decayFunction[[i]]
      decay_args <- formalArgs(decay_fun)
      args_list <- as.list(nodes[i, decay_args, drop=FALSE])
      args_list$x <- nnpg$dist[[i]] / (nnpg$dist_dth[[i]] * pars$ps$nrc)
      args_list$signal <- args_list$signal / pars$ps$zscale$maxsig
      if(pars$eweight) args_list$signal <- args_list$signal * nnpg$edwght[[i]]
      nsig[[i]] <- do.call(decay_fun, args_list)
    }
  }
  # vectorize and sort projected signals
  nsig <- unlist(nsig)
  nn <- unlist(nnpg$nn)
  idx <- order(-abs(nsig))
  nsig <- nsig[idx]
  nn <- nn[idx]
  # arrange into a matrix
  nnu <- unique(nn)
  nnl <- .find_nnu_positions(nnu, nn)
  n <- max(sapply(nnl, length))
  dsig_mt <- matrix(0, nrow = nrow(lpts), ncol = n)
  for(i in seq_along(nnl)){
    s <- nsig[nnl[[i]]]
    dsig_mt[nnu[i], seq_along(s) ] <- s
  }
  # aggregate
  Z <- .summ_dsig_mt(dsig_mt, pars)
  return(Z)
}

#-------------------------------------------------------------------------------
.get_angular_dist <- function(nnpg, lpts, gxy, edges, pars){
  # get polar coords
  edlist <- .edge_list(edges, gxy)
  etheta <- .edge_list_theta(edlist, gxy)
  edleng <- .edge_attr(edges, gxy, "edist")
  edwght <- .edge_attr(edges, gxy, "weight")
  if(pars$ps$directional){
    emode <- .edge_mode(edges, gxy)
    for(i in seq_along(emode)){
      idx <- emode[[i]] == 1
      edlist[[i]] <- edlist[[i]][idx]
      etheta[[i]] <- etheta[[i]][idx]
      edleng[[i]] <- edleng[[i]][idx]
      edwght[[i]] <- edwght[[i]][idx]
    }
  }
  # get normalized point-to-edge angular distances
  minlen <- min(unlist(edleng))
  for(i in seq_len(length(nnpg$nn))){
    p_idx <- nnpg$nn[[i]]
    p_dst <- nnpg$dist[[i]]
    # get edge's theta and dist
    p_et <- etheta[[i]]
    p_el <- edleng[[i]]
    p_wt <- edwght[[i]]
    # compute theta for p vs. i
    dx <- lpts[p_idx, "X"] - gxy[i, "X"]
    dy <- lpts[p_idx, "Y"] - gxy[i, "Y"]
    p_theta <- atan2(dy, dx)
    # get delta theta (multi directional)
    if(length(p_et)>0){
      # p_theta <- ifelse(p_theta < 0, p_theta + 2*pi, p_theta)
      # p_et <- ifelse(p_et < 0, p_et + 2*pi, p_et)
      mdr <- .get_delta_theta(p_theta, p_et, p_el, p_wt, pars)
      dth <- mdr[, 1]
      eln <- mdr[, 2]
      ewt <- mdr[, 3]
    } else {
      ## Estimate dth for isolated nodes
      if(pars$ps$directional){
        dth_iso <- 0
      } else {
        ## For isolated nodes, 'dth_iso' will aim the area of a cardioid,
        ## as result from the adjusted 'dist', computed in the expression: 
        ## dist_dth = dist * dth_iso^beta
        ## 1) cf: geometric factor to adjust circle's radius for a cardioid area
        cf <- sqrt(3/8)
        ## 2) dth_iso: rescaled 'dth'; here the effect of 'beta' is removed
        ## from 'cf', proportional to 'cf', so when later raised to beta
        ## (i.e. dth_iso^beta) the moderation is applied only once.
        dth_iso <- cf^(1 / (pars$ps$beta^cf) )
      }
      dth <- rep(dth_iso, length(p_dst))
      eln <- rep(minlen, length(p_dst))
      ewt <- rep(1, length(p_dst))
    }
    nnpg$dtheta[[i]] <- dth
    nnpg$edleng[[i]] <- eln
    nnpg$edwght[[i]] <- ewt
  }
  return(nnpg)
}
.get_delta_theta <- function(p_theta, p_et, p_el, p_wt, pars) {
  if (length(p_et) > 1){
    C <- ( 1 + ( 1 - .angular_evenness(p_et) ) ) ^ 2
  } else {
    C <- 1
  }
  mdr <- t(vapply(seq_along(p_theta), function(i) {
    dth <- abs(p_theta[i] - p_et)
    dth <- abs(dth - pi)/pi
    if (length(dth) > 1) {
      #-- Note1: 'p' skews mean toward larger or smaller weights
      #-- Note2: 'C' adjusts 'p' to account for angular variability
      # Aggregate edge length weighted by angular distances
      eln <- .weighted_mean(p_el, dth, p = pars$ps$beta * C)
      # Aggregate edge weight weighted by angular distances
      ewt <- .weighted_mean(p_wt, dth, p = pars$ps$beta * C)
      # Aggregate angular distances weighted by itself
      dth <- .lehmer_mean(dth, p = pars$ps$beta * C)
      ##-- Old1 aggregations
      # eln <- sum(p_el * (dth / sum(dth)))
      # dth2 <- dth ^ (pars$ps$beta * C)
      # ewt <- sum(p_wt * (dth2 / sum(dth2)))
      # dth <- sum(dth * (dth2 / sum(dth2)))
      ##-- Old2 aggregations
      #eln <- sum(p_el * (dth / sum(dth)))
      #idx <- which.max(dth * abs(p_wt))
      #dth <- dth[idx]
      #ewt <- p_wt[idx]
    } else {
      eln <- p_el
      ewt <- p_wt
    }
    c(dth, eln, ewt)
  }, numeric(3)))
  return(mdr)
}
.lehmer_mean <- function(x, p=2){
  sum(x^(p+1)) / sum(x^p)
}
.weighted_mean <- function(x, w, p=1){
  w <- w^p
  sum(w * x) / sum(w)
}
.angular_evenness <- function(x) {
  n <- length(x)
  x <- sort(x %% (2*pi))
  gaps <- c(diff(x), 2*pi - (x[n] - x[1]))
  max_gap <- max(gaps)
  min_gap <- 2*pi / n
  c <- 1 - (max_gap - min_gap) / (2*pi - min_gap)
  return(c)
}

#-------------------------------------------------------------------------------
.scale_dist_polar <- function(nnpg, pars) {
  pfun <- pars$ps$polar.fun
  for(i in seq_len(length(nnpg$nn))){
    if(pars$ps$edge.norm){
      edleng <- (nnpg$edleng[[i]] / pars$ps$nrc)
    } else {
      edleng <- 1
    }
    nnpg$dist_dth[[i]] <- edleng * pfun(nnpg$dtheta[[i]], pars$ps$beta)
  }
  return(nnpg)
}

#-------------------------------------------------------------------------------
.edge_list <- function(edges, gxy) {
    nms <- rownames(gxy)
    el <- lapply(nms, function(nm) {
        e <- c(edges$name2[edges$name1==nm],
            edges$name1[edges$name2==nm])
        which(nms %in% e)
    })
    names(el) <- nms
    return(el)
}
.edge_attr <- function(edges, gxy, eattr = "edist") {
    nms <- rownames(gxy)
    el <- lapply(nms, function(nm) {
        idx1 <- edges$name1 == nm
        att1 <- edges[[eattr]][idx1]
        names(att1) <- edges$name2[idx1]
        idx2 <- edges$name2 == nm
        att2 <- edges[[eattr]][idx2]
        names(att2) <- edges$name1[idx2]
        e <- c(att1, att2)
        e[order(match(names(e), nms))]
    })
    names(el) <- nms
    return(el)
}
.edge_mode <- function(edges, gxy) {
    nms <- rownames(gxy)
    el <- lapply(nms, function(nm) {
        idx1 <- edges$name1 == nm
        att1 <- edges$emode[idx1]
        names(att1) <- edges$name2[idx1]
        att1[] <- as.numeric(att1>0)
        idx2 <- edges$name2 == nm
        att2 <- edges$emode[idx2]
        names(att2) <- edges$name1[idx2]
        att2[] <- as.numeric(att2==2)
        e <- c(att1, att2)
        e[order(match(names(e), nms))]
    })
    names(el) <- nms
    return(el)
}

#-------------------------------------------------------------------------------
#--- compute theta between vertices
.edge_list_theta <- function(edlist, gxy) {
  theta <- lapply(seq_along(edlist), function(e1) {
    e2 <- edlist[[e1]]
    if (length(e2) > 0) {
      res <- .edges_theta(e1, e2, gxy)
    } else {
      res <- numeric()
    }
    res
  })
  names(theta) <- rownames(gxy)
  return(theta)
}
.edges_theta <- function(e1, e2, gxy) {
  p1 <- gxy[e1, c("X", "Y")]
  p2 <- gxy[e2, c("X", "Y"), drop = FALSE]
  dx <- p2[,"X"] - p1["X"]
  dy <- p2[,"Y"] - p1["Y"]
  dist <- sqrt(dx^2 + dy^2)
  theta <- ifelse(dist < 1e-07, NA, atan2(dy, dx))
  names(theta) <- rownames(p2)
  return(theta)
}

#-------------------------------------------------------------------------------
.deg2rad <- function(x) {
  x * pi / 180
}
.rad2deg <- function(x) {
  x * 180 / pi
}

################################################################################
### Rescale projections
################################################################################
# If 'rescale = F', it will rescale 'xsig' to the original range
.update_projections <- function(ps, pars, update.config=TRUE) {
  xfloor <- ps@projections$xfloor
  xsig <- ps@projections$xsig
  if(is.null(pars$ps$rescale)) pars$ps$rescale <- TRUE
  if (!is.null(xfloor)) xsig[xfloor == 0] <- NA
  bl <- all(range(xsig, na.rm = TRUE) == 0)
  if(bl){
    gxyz <- xsig
  } else {
    if (!pars$ps$rescale) {
      gxyz <- scales::rescale(xsig, to = pars$ps$zscale$range)
    } else if (pars$ps$zscale$signal.type == "binary") {
      gxyz <- scales::rescale(xsig, to = pars$ps$zscale$scaling)
    } else {
      endpoints <- c(0.01, 1e-04)
      if (pars$ps$zscale$scale.type == "negpos") {
        to <- pars$ps$zscale$range/pars$ps$zscale$maxsig
        gxyz <- .rescale_negpos(xsig, pars, to, endpoints = endpoints)
      } else if (pars$ps$zscale$scale.type == "pos") {
        to <- pars$ps$zscale$scaling
        gxyz <- .rescale_pos(xsig, pars, to, endpoints = endpoints)
      } else if (pars$ps$zscale$scale.type == "neg") {
        to <- pars$ps$zscale$scaling
        gxyz <- .rescale_neg(xsig, pars, to, endpoints = endpoints)
      }
      #- export 'endpoints' in future versions;
      #- it will cut noise and outliers at the endpoints;
      #- outliers outside that range are moved to the nearby endpoint;
      #- noise are set to background (i.e. zero).
    }
  }
  if(update.config || is.null(pars$ps$configs)){
    pars$ps$configs <- .get_configs(pars)
  }
  ps@projections$pars <- pars
  ps@projections$gxyz <- gxyz
  return(ps)
}
.get_configs <- function(pars){
  zconfig <- pars$ps$zscale
  if(is.null(pars$ps$rescale)){
    zconfig$rescaled <- TRUE
  } else {
    zconfig$rescaled <- pars$ps$rescale
  }
  if(zconfig$rescaled){
    zconfig$zlim <- zconfig$scaling
  } else {
    # mx <- zconfig$maxsig
    # if(mx==0) mx <- 1
    # if(zconfig$scale.type=="negpos"){
    #   zlim <- c(-mx, mx)
    # } else if(zconfig$scale.type=="neg"){
    #   zlim <- c(-mx, 0)
    # } else {
    #   zlim <- c(0, mx)
    # }
    zconfig$zlim <- zconfig$range
  }
  return(zconfig)
}
.rescale_negpos <- function(xsig, pars, to, endpoints) {
  pr <- c(endpoints[1], 1 - endpoints[2])
  # pos
  xp <- xsig
  xp[xp <= 0] <- NA
  qt <- quantile(as.numeric(xp), na.rm = TRUE, probs = pr)
  if(!any(is.na(qt))){
    xp[xp < qt[1]] <- qt[1]
    xp[xp > qt[2]] <- qt[2]
    xp <- scales::rescale(xp, to = c(0, to[2]))
  }
  # neg
  pr <- c(endpoints[2], 1 - endpoints[1])
  xn <- xsig
  xn[xn >= 0] <- NA
  qt <- quantile(as.numeric(xn), na.rm = TRUE, probs = pr)
  if(!any(is.na(qt))){
    xn[xn < qt[1]] <- qt[1]
    xn[xn > qt[2]] <- qt[2]
    xn <- scales::rescale(xn, to = c(to[1], 0))
  }
  # update xsig
  xsig[!is.na(xn)] <- xn[!is.na(xn)]
  xsig[!is.na(xp)] <- xp[!is.na(xp)]
  return(xsig)
}
.rescale_pos <- function(xsig, pars, to, endpoints) {
  pr <- c(endpoints[1], 1 - endpoints[2])
  xp <- xsig
  xp[xp <= 0] <- NA
  qt <- quantile(as.numeric(xp), na.rm = TRUE, probs = pr)
  xp[xp < qt[1]] <- qt[1]
  xp[xp > qt[2]] <- qt[2]
  xp <- scales::rescale(xp, to = to)
  xsig[!is.na(xp)] <- xp[!is.na(xp)]
  return(xsig)
}
.rescale_neg <- function(xsig, pars, to, endpoints) {
  pr <- c(endpoints[2], 1 - endpoints[1])
  xn <- xsig
  xn[xn >= 0] <- NA
  qt <- quantile(as.numeric(xn), na.rm = TRUE, probs = pr)
  xn[xn < qt[1]] <- qt[1]
  xn[xn > qt[2]] <- qt[2]
  xn <- scales::rescale(xn, to = to)
  xsig[!is.na(xn)] <- xn[!is.na(xn)]
  return(xsig)
}

################################################################################
### Map silhouette
################################################################################
.silhouetteCircular <- function(ps, verbose = TRUE) {
  
    nodes <- getPathwaySpace(ps, "nodes")
    pars <- getPathwaySpace(ps, "pars")
    pars$ps$zscale <- .get_signal_scale(nodes$signal)
    
    if(verbose) message("Mapping 'x' and 'y' coordinates...")
    gxy <- .rescale_coord(nodes, pars$ps$nrc)
    lpts <- .get_points_in_matrix(pars$ps$nrc)
    
    #--- get point-to-vertex distances for silhouettes
    nnbg <- .get_silhouette_dists(lpts, gxy, pars$ps$silh$k)
    
    #--- project floor
    xfloor <- array(0, c(pars$ps$nrc, pars$ps$nrc))
    if (nrow(nodes)>0 & pars$ps$silh$pdist > 0) {
        xfloor[lpts[, c("Y", "X")]] <- .get_ldfloor(pars, nnbg)
    }
    # image(.transpose_and_flip(xfloor))
    xfloor <- .cutfloor(xfloor, pars)
    xfloor[gxy[, c("Yint", "Xint")]] <- 1
    
    #---return silhouette
    nbg <- sum(xfloor == 1)
    sz <- round(nbg / prod(dim(xfloor)) * 100, 2)
    if(verbose) message("Silhouette: ", sz, "% of the landscape area!")
    
    #--- add projections
    ps@projections$gxy <- gxy
    ps@projections$xfloor <- xfloor
    if (!.checkStatus(ps, "Projection")){
      ps@projections$xsig <- array(0, c(pars$ps$nrc, pars$ps$nrc))
    }
    ps <- .update_projections(ps, pars, update.config=FALSE)
    return(ps)
}

#-------------------------------------------------------------------------------
#--- get point-to-vertex distances for silhouettes
.get_silhouette_dists <- function(lpts, gxy, k = 10){
  min.ksearch <- min(max(k, 10), nrow(gxy))
  nnbg <- RANN::nn2(gxy[, c("X", "Y")], lpts[, c("X", "Y")],
    k = min.ksearch)
  names(nnbg) <- c("nn", "dist")
  return(nnbg)
}

#-------------------------------------------------------------------------------
.get_ldfloor <- function(pars, nnbg) {
    decay_fun <- pars$ps$silh$decay.fun
    nn <- ncol(nnbg$nn)
    sfloor <- vapply(seq_len(nrow(nnbg$nn)), function(ii) {
        p_dst <- nnbg$dist[ii, ]
        #--- scaling projection on pdist
        pdist <- pars$ps$silh$pdist
        #--- get floor of an ideal (max) signal
        x <- p_dst / (pars$ps$nrc * pdist)
        sfloor <- decay_fun(x=x, signal=1)
        return(sfloor)
    }, numeric(nn))
    sfloor <- t(sfloor)
    sfloor <- matrix(sfloor[order(row(sfloor), -sfloor)],
        nrow = nrow(sfloor), byrow = TRUE)
    sfloor <- sfloor[, seq_len(pars$ps$silh$k), drop = FALSE]
    Z <- apply(sfloor, 1, mean)
    return(Z)
}

#-------------------------------------------------------------------------------
.cutfloor <- function(xfloor, pars) {
    size <- pi * (pars$ps$silh$pdist * 2 * pars$ps$nrc)^2
    rg <- range(xfloor, na.rm = TRUE)
    if (rg[1] != rg[2]) {
        xfloor <- xfloor - rg[1]
        xfloor <- xfloor / max(xfloor, na.rm = TRUE)
        mask <- xfloor
        mask[mask < pars$ps$silh$baseline] <- 0
        mask[mask > 0] <- 1
        if(pars$ps$silh$fill.cavity){
          mask <- .fillCavity(mask)
        } else {
          mask <- .fillSpots(mask, spot.size = size)
        }
        xfloor[mask == 0] <- 0
        xfloor[mask > 0] <- 1
    } else {
      xfloor[, ] <- 0
    }
    xfloor[1, ] <- 0
    xfloor[, 1] <- 0
    xfloor[, ncol(xfloor)] <- 0
    xfloor[nrow(xfloor), ] <- 0
    return(xfloor)
}

################################################################################
### Map summits for enrichment analysis
################################################################################
.summitMapping <- function(ps, verbose = TRUE, ...) {
    pars <- getPathwaySpace(ps, "pars")
    gxy <- ps@projections$gxy
    gxyz <- ps@projections$gxyz
    ps@projections$summits <- .find_summits(
        gxy = gxy, gxyz = gxyz, 
        maxset = pars$ps$summit$maxset, 
        minsize = pars$ps$summit$minsize,
        threshold = pars$ps$summit$summit_threshold,
        segm_fun = pars$ps$summit$segm_fun, ...=...)
    return(ps)
}

#-------------------------------------------------------------------------------
.find_summits <- function(gxy, gxyz, maxset, minsize, threshold,
    segm_fun, ...) {
    #-- get coords
    lpts <- as.matrix(gxy[, c("X", "Y")])
    lpts[, "X"] <- round(lpts[, "X"])
    lpts[, "Y"] <- round(lpts[, "Y"])
    #-- apply th
    smt <- gxyz
    smt[smt < threshold] <- 0
    #-- run watershed
    # smt <- summitWatershed(smt, tolerance=tolerance, ext=1)
    # smt <- base::do.call(segm_fun, c(list(x = smt), pars$ps$summit$segm_arg))
    smt <- segm_fun(smt, ...=...)
    xx <- .openPxEdges(smt > 0)
    smt[xx == 0] <- 0
    
    #--- retrive itens
    lset <- .retrive_items(smt, lpts)
    
    #--- apply minsize
    len <- unlist(lapply(lset, length))
    len <- which(len < minsize)
    nset <- length(lset)
    if (length(len) > 0) {
        lset <- lset[-len]
        smt[smt %in% len] <- 0
        nset <- nset - length(len)
        smt <- .relabel(smt)
        if (length(lset) > 0)
            names(lset) <- seq_along(lset)
    }
    
    #--- relabel by size and apply maxset
    # smt <- .relabelBySize(smt)
    # smt[smt > maxset] <- 0
    
    #--- relabel by signal and apply maxset
    smt <- .relabelBySignal(gxyz, smt)
    smt[smt > maxset] <- 0
    
    #--- retrive itens
    lset <- .retrive_items(smt, lpts)
    nset <- length(lset)
    
    #--- get summit outlines
    cset <- .findOutlines(smt)
    
    return(list(lset = lset, cset = cset, mset = smt, nset = nset))
}
.retrive_items <- function(smt, lpts){
  nset <- length(table(as.numeric(smt))) - 1
  lset <- lapply(seq_len(nset), function(i) {
    xi <- smt == i
    idx <- xi[lpts[, c("Y", "X")]]
    rownames(lpts)[idx]
  })
  names(lset) <- seq_along(lset)
  return(lset)
}

################################################################################
### Diverse internal accessors
################################################################################
.updateStatus <- function(ps, name, check = TRUE) {
  ps@status[name] <- ifelse(check, "[x]", "[ ]")
  return(ps)
}
.checkStatus <- function(ps, name) {
  if(name=="Projection"){
    projections <- c("PolarProjection","CircularProjection")
    sts <- any(ps@status[projections] == "[x]")
    names(sts) <- "Projection"
  } else {
    sts <- ps@status[name] == "[x]"
    names(sts) <- name
  }
  sts[is.na(sts)] <- FALSE
  sts
}
.summariseStatus <- function(ps){
  sts <- c(.checkStatus(ps, "Preprocess"),
    .checkStatus(ps, "Projection"),
    .checkStatus(ps, "Silhouette"),
    .checkStatus(ps, "Summits"))
  sts[] <- ifelse(sts, "[x]", "[ ]")
  sts <- paste(names(sts), sts, collapse = "  ", sep="")
  sts
}
.removeSummits <- function(ps, verbose=TRUE){
  if (.checkStatus(ps, "Summits")) {
    if(verbose) message("-- available summits removed.")
    i <- which(names(ps@projections) == "summits")
    ps@projections <- ps@projections[-i]
    ps <- .updateStatus(ps, "Summits", FALSE)
  } 
  return(ps)
}
.removeSilhouette <- function(ps, verbose=TRUE){
  if (.checkStatus(ps, "Silhouette")) {
    if(verbose) message("-- available silhouette removed.")
    i <- which(names(ps@projections) == "xfloor")
    ps@projections <- ps@projections[-i]
    ps <- .updateStatus(ps, "Silhouette", FALSE)
    ps <- .update_projections(ps)
  }
  return(ps)
}
.get_signal_scale <- function(signal) {
  zscale <- list(range = range(signal, na.rm = TRUE))
  zscale$maxsig <- max(abs(zscale$range))
  if (.all_binaryValues(signal)) {
    # "binary"
    zscale$signal.type <- "binary"
    zscale$scale.type <- "pos"
    zscale$range <- c(0, 1)
    zscale$scaling <- c(0, 1)
  } else if (zscale$range[1] == zscale$range[2]) {
    # when one single value; it will be treated as binary
    zscale$signal.type <- "binary"
    if (zscale$range[1] >= 0) {
      zscale$range[1] <- 0
      zscale$scaling <- c(0, 1)
      zscale$scale.type <- "pos"
    } else {
      zscale$range[2] <- 0
      zscale$scaling <- c(-1, 0)
      zscale$scale.type <- "neg"
    }
  } else if (sum(sign(zscale$range)) == 0) {
    # continuous in (-Inf,Inf)
    zscale$signal.type <- "continuous"
    zscale$scale.type <- "negpos"
    zscale$scaling <- c(-1, 1)
  } else if (all(zscale$range >= 0)) {
    # continuous >=0
    zscale$signal.type <- "continuous"
    zscale$scale.type <- "pos"
    zscale$scaling <- c(0, 1)
  } else if (all(zscale$range <= 0)) {
    # continuous <=0
    zscale$signal.type <- "continuous"
    zscale$scale.type <- "neg"
    zscale$scaling <- c(-1, 0)
  }
  return(zscale)
}
.rescale_coord <- function(nodes, nrc, from = c(0, 1)){
  gxy <- nodes[, c("x", "y")]
  colnames(gxy) <- c("X", "Y")
  rownames(gxy) <- nodes$name
  gxy$X <- scales::rescale(gxy$X, to = c(1, nrc), from = from)
  gxy$Y <- scales::rescale(gxy$Y, to = c(1, nrc), from = from)
  gxy$Xint <- round(gxy$X)
  gxy$Yint <- round(gxy$Y)
  gxy <- as.matrix(gxy)
  return(gxy)
}
.is_variable <- function(weight){
  if (length(weight) > 1 && sd(weight, na.rm = TRUE) > 0) {
    wscale <- TRUE
  } else {
    wscale <- FALSE
  }
  return(wscale)
}
.get_edge_dist <- function(edges, gxy) {
  dts <- sqrt((gxy[edges$name1, "X"] - gxy[edges$name2, "X"])^2 +
      (gxy[edges$name1, "Y"] - gxy[edges$name2, "Y"])^2)
  return(dts)
}

#-------------------------------------------------------------------------------
.validate_aggregate_fun <- function(aggregate.fun){
  fargs <- formalArgs(args(aggregate.fun))
  fargs <- fargs[ !fargs %in% c("...", "na.rm")]
  if(length(fargs)>1){
    stop("'aggregate.fun' must be a unary function, e.g, function(x) {...}",
      call. = FALSE)
  }
  x <- seq(0,10)/10
  result <- tryCatch({
    res <- aggregate.fun(x)
    res <- ifelse(length(res)==1, TRUE, FALSE)
    res
  }, 
    warning = function(w) { FALSE }, 
    error = function(e) { FALSE }
  )
  if(!result){
    ms1 <-"The 'aggregate.fun' failed validation: "
    ms2 <-"it should take a numeric vector and return a single scalar value."
    stop(ms1,ms2, call. = FALSE)
  }
  
  TRUE
}
#-------------------------------------------------------------------------------
.validate_polar_fun <- function(polar.fun){
  fargs <- formalArgs(args(polar.fun))
  fargs <- fargs[ !fargs %in% c("x", "beta")]
  if(length(fargs)>1){
    stop("the 'polar.fun' must have the signature: function(x, beta) {...}",
      call. = FALSE)
  }
  TRUE
}
#-------------------------------------------------------------------------------
.get_points_in_matrix <- function(nrc) {
  d <- c(nrc,nrc)
  lpts <- arrayInd(seq_len(prod(d)), d, useNames = TRUE)
  colnames(lpts) <- c("Y", "X")
  return(lpts)
}

################################################################################
### Other accessors
################################################################################

#-------------------------------------------------------------------------------
# pairwise vertice-to-vertice dists 
# .get_pairwise_dists <- function(gxy){
#     dx <- outer(gxy[, "X"], gxy[, "X"], FUN = "-")
#     dy <- outer(gxy[, "Y"], gxy[, "Y"], FUN = "-")
#     nngg <- sqrt(dx^2 + dy^2)
#     return(nngg)
# }

#-------------------------------------------------------------------------------
# .transpose_and_flip <- function(mtx) {
#     mtx <- t(mtx[nrow(mtx):1, ])
#     return(mtx)
# }

################################################################################
### Package documentation
################################################################################
#' @keywords internal
#' @author
#' The Cancer Genome Atlas Analysis Network
#'
#' @description
#' For a given graph containing vertices, edges, and a signal associated with
#' the vertices, the PathwaySpace package performs a convolution operation,
#' which involves a weighted combination of neighboring vertices and their
#' associated signals. The package then uses a decay function to propagate
#' these signals, creating geodesic paths on a 2D-image space.
#'
#' @details
#'
#' \tabular{ll}{
#' Package: \tab PathwaySpace\cr
#' Type: \tab Software\cr
#' License: \tab Artistic-2.0\cr
#' Maintainer: \tab Mauro Castro \email{mauro.a.castro@@gmail.com}\cr
#' }
#'
#' @section Index:
#' \tabular{ll}{
#' \link{PathwaySpace-class}: 
#' \tab An S4 class for signal propagation on pathway spaces.\cr
#' \link{buildPathwaySpace}: 
#' \tab Constructor of PathwaySpace-class objects.\cr
#' \link{circularProjection}: 
#' \tab Creating 2D-landscape images from graph objects.\cr
#' \link{polarProjection}: 
#' \tab Creating 2D-landscape images from graph objects.\cr
#' \link{silhouetteMapping}: 
#' \tab Mapping graph silhouettes on PathwaySpace images.\cr
#' \link{summitMapping}: 
#' \tab Mapping summits on a 2D-landscape image.\cr
#' \link{getPathwaySpace}: 
#' \tab Accessory method for fetching slots from a PathwaySpace object.\cr
#' \link{plotPathwaySpace}: 
#' \tab Plotting 2D-landscape images for the PathwaySpace package.\cr
#' }
#' Further information is available in the vignettes by typing
#' \code{vignette('PathwaySpace')}. Documented topics are also available in
#' HTML by typing \code{help.start()} and selecting the PathwaySpace package
#' from the menu.
#'
#' @references
#' The Cancer Genome Atlas Analysis Network (2023). PathwaySpace: Spatial propagation 
#' of network signals along geodesic paths. R package version 0.99.
#'
"_PACKAGE"
#> [1] '_PACKAGE'

################################################################################
### Documentation for the 'gimage' dataset
################################################################################
#' @title An image matrix
#'
#' @description An image matrix used for workflow demonstrations.
#'
#' @format matrix
#'
#' @usage data(gimage)
#'
#' @source This package.
#'
#' @docType data
#' @keywords gimage
#' @name gimage
#' @aliases gimage
#' @return An image matrix.
#' @examples
#' data(gimage)
NULL

################################################################################
### Documentation for the 'PCv12_pruned_igraph' dataset
################################################################################
#' @title A pruned and laid out igraph object from Pathway Commons V12
#'
#' @description
#' This igraph object was created from a 'sif' file available from 
#' the Pathway Commons V12 (Rodchenkov et al., 2020), which was filtered to 
#' keep interactions from the following sources: CTD, Recon, HumanCyc, 
#' DrugBank, MSigDB, DIP, BioGRID, IntAct, BIND, and PhosphoSite. The igraph 
#' was additionally pruned and laid out by a force-directed algorithm aiming 
#' signal projection on PathwaySpace's images. Edges with the smallest 
#' betweenness centrality were pruned using 'backward elimination' and
#' 'forward selection' strategies. The resulting graph represents the 
#' main connected component with the minimum number of edges.
#'
#' @format igraph
#'
#' @usage data(PCv12_pruned_igraph)
#'
#' @source Pathway Commons V12.
#'
#' @author Chris Wong, Mauro Castro, and TCGA Network.
#'
#' @references
#' Rodchenkov et al. Pathway Commons 2019 Update: integration, analysis and
#' exploration of pathway data. Nucleic Acids Research 48(D1):D489–D497, 2020.
#' \doi{10.1093/nar/gkz946}
#'
#' @docType data
#' @keywords PCv12_pruned_igraph
#' @name PCv12_pruned_igraph
#' @aliases PCv12_pruned_igraph
#' @aliases gdist.toy
#' @return An igraph object.
#' @examples
#' data(PCv12_pruned_igraph)
#' ## Suggestion to vizualize this igraph in R:
#' library(RGraphSpace)
#' plotGraphSpace(PCv12_pruned_igraph)
NULL

################################################################################
### Documentation for the 'CGC_20211118' dataset
################################################################################
#' @title COSMIC-CGC genes mapped to PathwaySpace images
#'
#' @description
#' A data frame listing 'GeneSymbol' and 'Entrez' IDs from the
#' COSMIC-CGC database (Sondka et al., 2020). These genes are used to
#' demonstrate the PathwaySpace's summit mapping pipeline, which assigns 
#' summits to an image space.
#'
#' @format data.frame
#'
#' @usage data(CGC_20211118)
#'
#' @source COSMIC-CGC database (release v95, tier 1 collection).
#'
#' @references
#' Sondka et al. The COSMIC Cancer Gene Census: describing genetic
#' dysfunction across all human cancers. Nat Rev Cancer 18, 696-705, 2018.
#' Doi: 10.1038/s41568-018-0060-1.
#'
#' @docType data
#' @keywords CGC_20211118
#' @name CGC_20211118
#' @aliases CGC_20211118
#' @return A data.frame object.
#' @examples
#' data(CGC_20211118)
NULL


################################################################################
### Documentation for the 'Hallmarks_v2023_1_Hs_symbols' dataset
################################################################################
#' @title A list with Hallmark gene sets (v2023.1)
#'
#' @description
#' A list with Human gene symbols from the MSigDB's Hallmark gene set 
#' collection (Liberzon et al., 2015). These gene sets are used to
#' demonstrate the PathwaySpace's summit mapping pipeline, which assigns 
#' summits to an image space.
#'
#' @format list
#'
#' @usage data(Hallmarks_v2023_1_Hs_symbols)
#'
#' @source MSigDB database (v2023.1).
#'
#' @references
#' Liberzon et al. The Molecular Signatures Database (MSigDB) hallmark 
#' gene set collection. Cell Systems 1(5):417-425, 2015
#' Doi: 10.1016/j.cels.2015.12.004
#'
#' @docType data
#' @keywords Hallmarks_v2023_1_Hs_symbols
#' @name Hallmarks_v2023_1_Hs_symbols
#' @aliases Hallmarks_v2023_1_Hs_symbols
#' @aliases hallmarks
#' @return A list object.
#' @examples
#' data(Hallmarks_v2023_1_Hs_symbols)
NULL


################################################################################
### Wrapper around 
################################################################################
#' @title getNearestNode
#' 
#' @description
#' Retrieves the nearest neighbor for each node from a 
#' \linkS4class{PathwaySpace} object using Euclidean distance.
#' 
#' @param ps Either a \linkS4class{PathwaySpace} or 
#' \link[RGraphSpace]{GraphSpace} object.
#' @seealso \code{\link[RANN]{nn2}}
#' @examples
#' # See examples in the PathwaySpace's tutorials:
#' # https://sysbiolab.github.io/PathwaySpace/
#' 
#' @importFrom RANN nn2
#' @aliases getNearestNode
#' @export
getNearestNode <- function(ps){
  if(!inherits(ps, "GraphSpace")){
    stop("'ps' should be either a 'PathwaySpace' or 'GraphSpace' object.")
  }
  nodes <- getGraphSpace(ps, "nodes")
  nnpg <- nn2(nodes[,c("x","y")], nodes[,c("x","y")], k=2)
  nn.idx <- nnpg$nn.idx[,2]
  nn.dists <- nnpg$nn.dists[,2]
  nn <- data.frame(from=nodes$name, to=nodes$name[nn.idx], dist=nn.dists)
  return(nn)
}

################################################################################
### Colors for PathwaySpace
################################################################################
#' @title Create interpolated color palettes for PathwaySpace images
#' 
#' @description
#' Creates mixed color palettes by interpolating and offsetting hues, useful for
#' generating transitions between hues.
#' 
#' @param colors A vector of five base colors used to construct the custom
#' diverging palette. These colors are interpolated according to the
#' `trim.colors` values.
#' @param trim.colors A vector of five positive integers that control the
#' relative weight of each hue in the five-color diverging palette.
#' @param offset Adjusts brightness by shifting hues toward the center,
#' either brighter (`offset > 0`) or darker (` offset < 0`).
#' @param n The number of colors to generate in the output palette.
#' @return A vector with hexadecimal color codes.
#' @seealso \code{\link{plotPathwaySpace}}
#' @examples
#' pspace.pals()
#'
#' @importFrom colorspace lighten
#' @aliases pspace.pals
#' @export
#'
pspace.pals <- function(
    colors = c("#303f9d","#578edb","#63b946","#f3930c","#a60d0d"),
  trim.colors = c(3, 2, 1, 2, 3), offset = 0.5, n = 25) {
  .validate.colors("allColors","colors", colors)
  .validate.ps.args("allInteger", "trim.colors", trim.colors)
  .validate.ps.args("singleInteger", "n", n)
  .validate.ps.args("singleNumber", "offset", offset)
  if(any(trim.colors < 0)){
    stop("'trim.colors' should be a vector with 5 positive integers.",
      call. = FALSE)
  }
  if(length(colors)!=5){
    colors <- colorRampPalette(colors)(5)
  }
  if(length(trim.colors)!=5){
    if(length(trim.colors)==1){
      trim.colors <- rep(trim.colors, 5)
    } else {
      stop("'trim.colors' should be a vector with 5 positive integers.",
        call. = FALSE)
    }
  }
  colors <- .pspace_pals(colors, trim.colors, offset)
  colorRampPalette(colors)(n)
}

.pspace_pals <- function(colors, trim.colors, offset) {
  tms <- trim.colors^2
  center <- ceiling(length(colors)/2)
  cols <- vector("list", 5)
  idx <- seq(1, center-1)
  cols[idx] <- lapply(idx, function(i){
    cl <- colorspace::lighten(colors[i], amount = offset)
    colorRampPalette(c(colors[i], cl))(tms[i])
  })
  idx <- seq(center+1, length(colors))
  cols[idx] <- lapply(idx, function(i){
    cl <- colorspace::lighten(colors[i], amount = offset)
    colorRampPalette(c(cl, colors[i]))(tms[i])
  })
  cl <- colorspace::lighten(colors[center], amount = offset)
  cols[[center]] <- colorRampPalette(c(colors[center], cl , colors[center]))(tms[center])
  cols <- unlist(cols)
  cols <- colorspace::lighten(cols, amount = -offset/5)
  cols <- colorRampPalette(cols)(25)
  return(cols)
}


################################################################################
### Colors for PathwaySpace
################################################################################
#' @title A simple vector of colors for PathwaySpace images
#'
#' @param n The number of colors to generate in the output palette.
#' @param ... Additional arguments (not used).
#' @return A vector with hexadecimal color codes.
#' @seealso \code{\link{plotPathwaySpace}}, \code{\link{pspace.pals}}
#' @examples
#' pspace.cols()
#'
#' @importFrom colorspace lighten
#' @aliases pspace.cols
#' @export
#'
pspace.cols <- function(n = 25, ...) {
  .validate.ps.args("singleInteger", "n", n)
  colors <- c("#303f9d","#578edb","#63b946","#f3930c","#a60d0d")
  colors <- .pspace_cols(colors)
  colorRampPalette(colors)(n)
}

.pspace_cols <- function(colors) {
  trim.colors <- c(3, 2, 1, 2, 3)
  tms <- trim.colors * 3
  offset <- list()
  offset[[1]] <- c(0.03, 0.09, 0.07, 0)
  offset[[2]] <- c(0.12, 0.26, 0.08, 0)
  offset[[3]] <- c(0.25, 0.11, 0.09, 0)
  offset[[4]] <- c(0.00, 0.31, 0.10, 0)
  offset[[5]] <- c(0.31, 0.30, 0.09, 0)
  cols <- lapply(seq_along(colors), function(i){
    cl <- adjustcolor(colors[i], offset = offset[[i]])
    colorRampPalette(c(colors[i], cl))(tms[i])
  })
  cols[[4]] <- rev(cols[[4]])
  cols[[5]] <- rev(cols[[5]])
  cols <- unlist(cols)
  return(cols)
}

################################################################################
### Pathway space distance
################################################################################
#' @title Calculate a pathway space distance between two vectors
#' 
#' @param gdist A distance matrix computed by the igraph's \code{distances} 
#' function. Rows and columns must be named with vertex labels as listed in
#' the 'igraph' object.
#' @param from A vector with valid vertex names.
#' @param to A vector with valid vertex names.
#' @param nperm Number of permutations.
#' @param verbose A single logical value specifying to display detailed 
#' messages (when \code{verbose=TRUE}) or not (when \code{verbose=FALSE}).
#' @return A list with pathway space distances and a 'ggplot' object.
#' @seealso \code{\link{plotPathwaySpace}}
#' @examples
#' 
#' # Load a vertex-wise distance matrix (distance between nodes in a graph)
#' data("gdist.toy", package = "PathwaySpace")
#'
#' # Get two vertex lists
#' from <- sample(colnames(gdist.toy), 50)
#' to <- sample(colnames(gdist.toy), 50)
#' 
#' # Calculate distances between lists, and between random lists
#' res <- pathDistances(gdist.toy, from, to)
#' names(res)
#' # "p_dist"  "z_score"
#' 
#' @importFrom igraph distances
#' @importFrom utils setTxtProgressBar txtProgressBar
#' @aliases pathDistances
#' @export
#'
pathDistances <- function(gdist, from, to, nperm = 1000, verbose=TRUE){
  .validate.ps.args("square_numeric_mtx", "gdist", gdist)
  .validate.ps.args("allCharacter", "from", from)
  .validate.ps.args("allCharacter", "to", to)
  .validate.ps.args("singleInteger", "nperm", nperm)
  .validate.ps.args("singleLogical", "verbose", verbose)
  if(!all(from %in% rownames(gdist))){
    stop("All names in 'from' should be listed in 'gdist' rownames.")
  }
  if(!all(to %in% colnames(gdist))){
    stop("All names in 'to' should be listed in 'gdist' colnames.")
  }
  res <- .psdist(gdist, from, to, nperm, verbose)
  return(res)
}
.psdist <- function(gdist, from, to, nperm, verbose){
  n.total <- ncol(gdist)
  n.from <- length(from)
  n.to <- length(to)
  if(verbose) message("Computing pathway distance...")
  obs <- gdist[from, to]
  obs <- mean(apply(obs, 1, min, na.rm=T))
  if(verbose) message("Running permutation analysis...")
  if (verbose) pb <- txtProgressBar(style = 3)
  null <- vapply(seq_len(nperm), function(i){
    if (verbose) setTxtProgressBar(pb, i / nperm)
    r.from <- sample(n.total, n.from)
    r.to <- sample(n.total, n.to)
    res <- gdist[r.from, r.to]
    mean(apply(res, 1, min, na.rm=T))
  }, numeric(1))
  p_dist <- list(obs=obs, null=null)
  z_score <- list()
  sd <- sd(null)
  md <- median(null)
  z_score$obs <- (obs-md)/sd
  z_score$null <- (null - md)/sd
  res <- list(p_dist=p_dist, z_score=z_score)
  return(res)
}

################################################################################
### Plot pathway space distances
################################################################################
#' @title Accessory function to plot pathway space distances
#' 
#' @param pdist A list generated by the \code{\link{pathDistances}} function.
#' @param z.transform A single logical value specifying to convert pathway 
#' distances into z-score values.
#' @return A 'ggplot' object.
#' @examples
#' 
#' # Load a vertex-wise distance matrix (distance between nodes in a graph)
#' data("gdist.toy", package = "PathwaySpace")
#' 
#' # Get two gene lists
#' from <- sample(colnames(gdist.toy), 50)
#' to <- sample(colnames(gdist.toy), 50)
#' 
#' # Calculate distances between lists, and between random lists
#' res <- pathDistances(gdist.toy, from, to)
#' 
#' # Plot observed and null distances
#' plotPathDistances(res)
#' 
#' @importFrom stats density median density
#' @importFrom ggplot2 geom_area ylab geom_segment geom_line
#' @importFrom ggplot2 xlab theme_minimal
#' @aliases plotPathDistances
#' @export
#'
plotPathDistances <- function(pdist, z.transform=FALSE){
  .validate.ps.args("singleLogical", "z.transform", z.transform)
  if(z.transform){
    gg <- .plot_pdist(pdist$z_score, xlab = "z-score")
  } else {
    gg <- .plot_pdist(pdist$p_dist, xlab = "steps")
  }
  return(gg)
}
.plot_pdist <- function(pdist, xlab = "steps"){
  #---
  obs <- pdist$obs
  null <- pdist$null
  nperm <- length(pdist$null)
  #---
  dt <- data.frame(ord=seq_len(nperm), null=sort(null, decreasing = FALSE))
  probs <- seq(0, 1, 0.01)
  quant <- quantile(dt$null, prob=probs)
  dt$quant <- probs[findInterval(dt$null, quant)]
  thr1 <- ceiling(nperm*c(0.05, 0.5))
  thr1 <- dt[thr1,]
  #---
  dens <- density(dt$null, adjust=2, from = min(dt$null), to=max(dt$null))
  dens <- data.frame(x=dens$x, y=dens$y)
  dens$y <- dens$y/max(dens$y)
  dens$thr <- findInterval(dens$x, thr1$null)
  thr2 <- dens[match(c(1,2),dens$thr),]
  #---
  xlab <- paste0("Distance between gene lists in pathway space (",xlab,")")
  lim <- range(pretty(c(obs, null)))
  dlim <- lim[2]-lim[1]
  x <- y <- NULL
  gg <- ggplot(dens, aes(x, y)) + 
    geom_area(fill = "grey95") + 
    geom_area(data = dens[dens$x < thr2$x[1], ], fill = "grey40") + 
    scale_x_continuous(limits = lim) + xlab(xlab) + ylab("Density") +
    geom_segment(x=thr2$x[2], y=0, xend = thr2$x[2], yend = thr2$y[2], 
      linewidth=0.25, lty="22", color="grey") + geom_line(linewidth=0.25) + 
    geom_segment(x=thr2$x[1], y=0, xend = thr2$x[1], yend = 0.5, 
      linewidth=0.4, lty="22", color="black") + theme_minimal()
  gg <- gg + annotate("point", x = obs, y = 0.03, pch=25, size=3, fill="red") +
    annotate("text", x = obs, y = 0.15, label="Obs. distance",
      color="red", hjust=0.1, vjust=0.5) +
    annotate("text", x = thr2$x[1], y = 0.5, label="Alpha 0.05",
      color="black", hjust=1, vjust=0) +
    annotate("text", x = lim[2]-dlim*0.1, 
      y = thr2$y[2], label="Distances between\nrandom gene lists",
      color="grey40", hjust=1, vjust=1)
  gg
}


################################################################################
##########################  misc. accessory functions  #########################
################################################################################

################################################################################
### Count the number of local neighbors
################################################################################
.countPxNeighbors <- function(x) {
    x <- xx <- .expandMask(x)
    nr <- nrow(x)
    nc <- ncol(x)
    ii <- jj <- c(-1, 0, 1)
    for (i in 2:(nr - 1)) {
        for (j in which(x[i, ] == 1)) {
            xx[i, j] <- sum(x[ii + i, jj + j] == 1) - 1
        }
    }
    xx <- xx[-c(1, nr), -c(1, nc)]
    return(xx)
}
.expandMask <- function(x, val = 0) {
    xx <- array(val, dim = dim(x) + 2)
    nr <- nrow(xx)
    nc <- ncol(xx)
    xx[-c(1, nr), -c(1, nc)] <- x
    return(xx)
}
.reduceMask <- function(x) {
    nr <- nrow(x)
    nc <- ncol(x)
    x <- x[-c(1, nr), -c(1, nc)]
    return(x)
}

################################################################################
### Find outlines in a labeled image mask (labels near bg's '0s')
################################################################################
.findOutlines <- function(mask) {
    ids <- sort(unique(as.numeric(mask)), decreasing = TRUE)
    ids <- ids[ids > 0]
    x <- mask
    x[, ] <- 0
    for (id in ids) {
        tp <- .findPxEdges(mask == id)
        x[tp == 1] <- id
    }
    x
}

################################################################################
### Erode and dilatate pixels of a mask
################################################################################
.openPxEdges <- function(mask, kernel = .kernelMask(2)) {
    mask <- .erodePxEdges(mask, kernel)
    mask <- .dilatePxEdges(mask, kernel)
    return(mask)
}

################################################################################
### Dilatate pixels of a mask
################################################################################
.dilatePxEdges <- function(mask, kernel = .kernelMask(2)) {
    ids <- as.numeric(names(table(mask)))
    ids <- ids[ids > 0]
    for (id in ids) {
        tp <- .dledge(mask == id, kernel = kernel)
        mask[tp == 1] <- id
    }
    mask
}
.dledge <- function(dx, kernel) {
    nr <- nrow(dx)
    nc <- ncol(dx)
    n <- (dim(kernel)[1] - 1)/2
    ed <- .findPxEdges(dx)
    idx <- which(ed == 1, arr.ind = TRUE)
    if (nrow(idx) > 0) {
        for (id in seq_len(nrow(idx))) {
            i <- idx[id, 1]
            j <- idx[id, 2]
            ii <- (i - n):(i + n)
            jj <- (j - n):(j + n)
            bi <- ii > 0 & ii <= nr
            bj <- jj > 0 & jj <= nc
            dx[ii[bi], jj[bj]] <- dx[ii[bi], jj[bj]] + kernel[bi, bj]
        }
        dx[dx > 0] <- 1
    }
    dx
}

################################################################################
### Erode pixels of a mask
################################################################################
.erodePxEdges <- function(mask, kernel = .kernelMask(2)) {
    x <- mask
    x[x != 0] <- 1
    tp <- .eledge(x, kernel = kernel)
    mask[tp == 0] <- 0
    mask
}
.eledge <- function(ex, kernel) {
    nr <- nrow(ex)
    nc <- ncol(ex)
    n <- (dim(kernel)[1] - 1)/2
    ed <- .findBgEdges(ex)
    idx <- which(ed == 1, arr.ind = TRUE)
    if (nrow(idx) > 0) {
        for (id in seq_len(nrow(idx))) {
            i <- idx[id, 1]
            j <- idx[id, 2]
            ii <- (i - n):(i + n)
            jj <- (j - n):(j + n)
            bi <- ii > 0 & ii <= nr
            bj <- jj > 0 & jj <= nc
            ex[ii[bi], jj[bj]] <- ex[ii[bi], jj[bj]] - kernel[bi, bj]
        }
        ex[ex < 0] <- 0
    } else {
        ex[] <- 0
    }
    ex
}

################################################################################
### Remove tips with less than n neighbors
################################################################################
.removeTips <- function(mask, n = 3) {
    tp <- .countPxNeighbors(mask == 1)
    array(as.numeric(tp > n), dim = dim(tp))
}

################################################################################
### Find neighbors in an image mask
################################################################################
# Find bg (0s) near to pixels (1s)
.findBgEdges <- function(mask) {
    ed <- .expandPxEdges(mask)
    ed[mask == 1] <- 0
    ed
}
# Find pixels (1s) near bg (0s)
.findPxEdges <- function(mask) {
    ed <- .countPxNeighbors(mask)
    ed <- ed > 0 & ed < 8
    ed
}

################################################################################
### Expand pixels (in one unit)
################################################################################
.expandPxEdges <- function(mask) {
    nrc <- dim(mask)
    idx <- which(mask == 1, arr.ind = TRUE)
    ii <- idx[, 1]
    jj <- idx[, 2]
    ii <- c(ii - 1, ii, ii, ii + 1)
    jj <- c(jj, jj - 1, jj + 1, jj)
    ix <- (ii > 0 & ii <= nrc[1]) & (jj > 0 & jj <= nrc[2])
    ii <- ii[ix]
    jj <- jj[ix]
    idx <- mask[cbind(ii, jj)]
    mask[cbind(ii, jj)][idx == 0] <- 1
    return(mask)
}

################################################################################
### Fill cavity of a labeled mask
################################################################################
.fillCavity <- function(mask) {
    xm <- mask
    xm[is.na(xm)] <- 0
    ids <- sort(unique(as.numeric(xm)), decreasing = TRUE)
    ids <- ids[ids > 0]
    for (id in ids) {
        idx <- which(xm == id, arr.ind = TRUE)
        rr <- range(idx[, 1])
        rr <- rr[1]:rr[2]
        rc <- range(idx[, 2])
        rc <- rc[1]:rc[2]
        if (length(rr) > 3 && length(rc) > 3) {
            x1 <- x2 <- xm[rr, rc, drop = FALSE]
            x2[x2 != id] <- 0
            x2 <- x2 == 0
            x2 <- .expandMask(x2, val = 1)
            #--- fill large cavities
            x3 <- .labelMask(x2)
            x3 <- .reduceMask(x3)
            x1[x1 == 0 & x3 > 1] <- id
            #--- fill small spots
            x3 <- .removeTips(x2, n = 3)
            x3 <- .reduceMask(x3)
            x4 <- .reduceMask(x2)
            x1[x3 != x4] <- id
            mask[rr, rc] <- x1
        }
    }
    mask
}
.fillSpots <- function(mask, spot.size = 6) {
  xm <- mask
  xm[is.na(xm)] <- 0
  ids <- sort(unique(as.numeric(xm)), decreasing = TRUE)
  ids <- ids[ids > 0]
  for (id in ids) {
    idx <- which(xm == id, arr.ind = TRUE)
    rr <- range(idx[, 1])
    rr <- rr[1]:rr[2]
    rc <- range(idx[, 2])
    rc <- rc[1]:rc[2]
    if (length(rr) > 3 && length(rc) > 3) {
      x1 <- x2 <- xm[rr, rc, drop = FALSE]
      x2[x2 != id] <- 0
      x2 <- x2 == 0
      x2 <- .expandMask(x2, val = 1)
      #--- fill small spots
      x3 <- .labelMask(x2)
      x3 <- .relabelBySize(x3)
      spot_th <- which(table(x3)[-1] <= spot.size)[1]
      x3 <- .reduceMask(x3)
      x1[x1 == 0 & x3 > spot_th] <- id
      #--- fill small spots
      x3 <- .removeTips(x2, n = 3)
      x3 <- .reduceMask(x3)
      x4 <- .reduceMask(x2)
      x1[x3 != x4] <- id
      mask[rr, rc] <- x1
    }
  }
  mask
}
.countCavity <- function(mask) {
    xm <- mask
    xm[is.na(xm)] <- 0
    ids <- sort(unique(as.numeric(xm)))
    ids <- ids[ids > 0]
    nc <- vapply(ids, function(id) {
        idx <- which(xm == id, arr.ind = TRUE)
        rr <- range(idx[, 1])
        rr <- rr[1]:rr[2]
        rc <- range(idx[, 2])
        rc <- rc[1]:rc[2]
        if (length(rr) > 3 && length(rc) > 3) {
            x1 <- x2 <- xm[rr, rc, drop = FALSE]
            x2[x2 != id] <- 0
            x2 <- x2 == 0
            x2 <- .expandMask(x2, val = 1)
            x2 <- .labelMask(x2)
            n <- length(table(x2)) - 2
        } else {
            n <- 0
        }
        n
    }, numeric(1))
    return(nc)
}

################################################################################
### Make a kernel mask
################################################################################
.kernelMask <- function(r = 5, shape = c("round", "diamond", "square")) {
    shape <- match.arg(shape)
    if (shape == "round") {
        kmask <- .rdKernel(r)
    } else if (shape == "diamond") {
        kmask <- .dmKernel(r)
    } else {
        kmask <- .sqKernel(r)
    }
    kmask
}
.rdKernel <- function(r = 5) {
    len <- r * 2 + 1
    x <- matrix(0, nrow = len, ncol = len)
    cnt <- (len + 1)/2
    r <- cnt - 1
    a <- (cnt - r):(cnt + r)
    b <- (cnt - r):(cnt + r)
    for (i in a) {
        for (j in b) {
            if (sqrt((i - cnt)^2 + (j - cnt)^2) <= r) {
                x[i, j] <- 1
            }
        }
    }
    x
}
.dmKernel <- function(r = 5) {
    len <- r * 2 + 1
    x <- matrix(0, nrow = len, ncol = len)
    cnt <- (len + 1)/2
    x[cnt, cnt] <- 1
    x <- .expandKernel(x)
    x
}
.sqKernel <- function(r = 5) {
    len <- r * 2 + 1
    x <- matrix(0, nrow = len, ncol = len)
    x[, ] <- 1
    x
}
.expandKernel <- function(x) {
    while (!.anyOuterPx(x)) {
        x <- .expandPxEdges(x)
    }
    x
}
.anyOuterPx <- function(x) {
    d <- dim(x)
    b <- c(x[1, ], x[d[1], ], x[, 1], x[, d[2]])
    ifelse(any(b == 1), TRUE, FALSE)
}

################################################################################
### Label contiguous pixels in an image mask
################################################################################
.labelMask <- function(mask) {
    mask[is.na(mask)] <- 0
    mask <- mask > 0
    idxmask <- data.frame(which(mask | !mask, arr.ind = TRUE))
    idxmask$key <- seq_len(nrow(idxmask))
    idxmask$val <- as.numeric(mask)
    idxmask$label <- NA
    label <- 1
    seed <- idxmask[which(idxmask$val == 1)[1], , drop = FALSE]
    while (sum(idxmask$val) > 0) {
        idxmask$label[seed$key] <- label
        idxmask$val[seed$key] <- 0
        seed <- .expandSeed(seed, idxmask, nrc = dim(mask))
        if (nrow(seed) == 0) {
            seed <- idxmask[which(idxmask$val == 1)[1], , drop = FALSE]
            label <- label + 1
        }
    }
    idxmask <- idxmask[!is.na(idxmask$label), , drop = FALSE]
    mask[cbind(idxmask$row, idxmask$col)] <- idxmask$label
    mask
}
.expandSeed <- function(seed, idx, nrc) {
    ii <- seed[, "row"]
    jj <- seed[, "col"]
    ii <- c(ii - 1, ii - 1, ii - 1, ii, ii, ii + 1, ii + 1, ii + 1)
    jj <- c(jj - 1, jj, jj + 1, jj - 1, jj + 1, jj - 1, jj, jj + 1)
    ix <- (ii > 0 & ii <= nrc[1]) & (jj > 0 & jj <= nrc[2])
    ii <- ii[ix]
    jj <- jj[ix]
    ij <- ii + nrc[1] * (jj - 1)
    seed <- idx[unique(ij), , drop = FALSE]
    seed <- seed[seed$val > 0, , drop = FALSE]
    return(seed)
}

################################################################################
### Relabel a mask with labeled objects
################################################################################
.relabel <- function(mask) {
    lbs <- sort(unique(as.numeric(mask)))
    lbs <- lbs[lbs > 0]
    rmask <- mask
    for (i in seq_along(lbs)) {
        rmask[mask == lbs[i]] <- i
    }
    return(rmask)
}
.relabelBySize <- function(mask) {
    objs <- table(mask)
    lbs <- as.integer(names(objs))
    objs <- objs[lbs > 0]
    lbs <- lbs[lbs > 0]
    lbs <- lbs[order(objs, decreasing = TRUE)]
    rmask <- mask
    for (i in seq_along(lbs)) {
        rmask[mask == lbs[i]] <- i
    }
    return(rmask)
}
.relabelBySignal <- function(x, mask, decreasing = TRUE) {
    mask[is.na(mask)] <- 0
    lbs <- sort(unique(as.numeric(mask)))
    lbs <- lbs[lbs > 0]
    sig <- vapply(lbs, function(lb) {
        quantile(x[mask == lb], 0.95, na.rm = TRUE)
    }, numeric(1))
    lbs <- lbs[order(sig, decreasing = decreasing)]
    rmask <- mask
    for (i in seq_along(lbs)) {
        rmask[mask == lbs[i]] <- i
    }
    return(rmask)
}

#' @title PathwaySpace: An S4 class for signal propagation on image spaces
#'
#' @slot nodes A data frame with xy-vertex coordinates.
#' @slot edges A data frame with edges.
#' @slot graph An igraph object.
#' @slot image A raster background image matrix.
#' @slot pars A list with parameters.
#' @slot misc A list with intermediate objects for downstream methods.
#' @slot projections A list with processed objects for downstream methods.
#' @slot status A vector containing the processing status of the
#' PathwaySpace object.
#' @author Sysbiolab Team, Mauro Castro (\email{mauro.castro@@ufpr.br})
#'
#' @method circularProjection \code{\link{circularProjection}}
#' @method polarProjection \code{\link{polarProjection}}
#' @method silhouetteMapping \code{\link{silhouetteMapping}}
#' @method summitMapping \code{\link{summitMapping}}
#' @method getPathwaySpace \code{\link{getPathwaySpace}}
#' @method plotPathwaySpace \code{\link{plotPathwaySpace}}
#' @aliases PathwaySpace-class
#' @importClassesFrom RGraphSpace GraphSpace
#' @return An S4 class object.
#' @section Constructor:
#' see \code{\link{buildPathwaySpace}} constructor.
#' @exportClass PathwaySpace
#' @importClassesFrom RGraphSpace GraphSpace
#' @export
setClass("PathwaySpace",
    contains = "GraphSpace",
    slots = c(
        projections = "list",
        status = "character"
    ),
    prototype = list(
        projections = list(),
        status = character()
    )
)
setValidity("PathwaySpace", function(object) {
    errors <- character()
    
    ## Inherit and check GraphSpace validity
    gs_valid <- getValidity(getClass("GraphSpace"))(object)
    if (!isTRUE(gs_valid)) {
        errors <- c(errors, paste0("GraphSpace validity: ", gs_valid))
    }
    
    ## Check projections slot
    if (!is.list(object@projections)) {
        errors <- c(errors, "'projections' must be a list.")
    }
    
    ## Check status slot
    if (!is.character(object@status)) {
        errors <- c(errors, "'status' must be a character vector.")
    }
    
    ## Return
    if (length(errors) == 0) TRUE else errors
    
})

#-------------------------------------------------------------------------------
.validate.gspace <- function(gs){
  n <- nrow(getGraphSpace(gs, "nodes"))
  if (n<1) {
    stop("'gs' should have at least one vertex.", call. = FALSE) 
  }
}

#-------------------------------------------------------------------------------
.validate.ps.args <- function(check, name, para) {
  if (check == "numeric_vec") {
    msg <- paste0("'", name, "' should be a numeric vector.")
    if (!.is_numericVector(para)) stop(msg, call. = FALSE)
  } else if (check == "numeric_mtx") {
    msg <- paste0("'", name, "' should be a numeric matrix.")
    if (!is.numeric(para) || !is.matrix(para)) 
      stop(msg, call. = FALSE)
  } else if (check == "square_numeric_mtx") {
    msg <- paste0("'", name, "' should be a named numeric square matrix.")
    if (!is.numeric(para) || !is.matrix(para)) 
      stop(msg, call. = FALSE)
    if(ncol(para) != nrow(para)) 
      stop(msg, call. = FALSE)
    if(is.null(colnames(para)) || is.null(rownames(para))) 
      stop(msg, call. = FALSE)
    if(any(colnames(para) != rownames(para)))
      stop(msg, call. = FALSE)
  } else if (check == "allCharacter") {
    msg <- paste0("'", name, "' should be a vector with strings.")
    if (!.all_characterValues(para)) stop(msg, call. = FALSE)
  } else if (check == "allBinary") {
    msg <- paste0("'", name, "' should be a vector with binary values.")
    if (!.all_binaryValues(para)) stop(msg, call. = FALSE)
  } else if (check == "allInteger") {
    msg <- paste0("'", name, "' should be a vector with integer values.")
    if (!.all_integerValues(para)) stop(msg, call. = FALSE)
  } else if (check == "singleString") {
    msg <- paste0("'", name, "' should be a single string.")
    if (!.is_singleString(para)) stop(msg, call. = FALSE)
  } else if (check == "singleInteger") {
    msg <- paste0("'", name, "' should be a single integer value.")
    if (!.is_singleInteger(para)) stop(msg, call. = FALSE)
  } else if (check == "singleNumber") {
    msg <- paste0("'", name, "' should be a single numeric value.")
    if (!.is_singleNumber(para)) stop(msg, call. = FALSE)
  } else if (check == "function") {
    msg <- paste0("'", name, "' should be a function.")
    if (!is.function(para)) stop(msg, call. = FALSE)
  } else if (check == "singleLogical") {
    msg <- paste0("'", name, "' should be a single logical value.")
    if (!.is_singleLogical(para)) stop(msg, call. = FALSE)
  } else {
    warning("Skipped arg validation.", call. = FALSE)
  }
}

#-------------------------------------------------------------------------------
.validate.colors <- function(check, name, para) {
  if (check == "singleColor") {
    if (!.is_singleColor(para)) {
      msg <- paste0("'", name, "' should be a single color.")
      stop(msg, call. = FALSE)
    }
  } else if (check == "allColors") {
    if (!.is_color(para)) {
      msg <- paste0("'", name, "' should be a vector with colors.")
      stop(msg, call. = FALSE)
    }
  } else {
    warning("Skipped color validation.", call. = FALSE)
  }
}

#-------------------------------------------------------------------------------
# Validate custom plot args
.validate.psplot.args <- function(name, para) {
  if (name == "marks") {
    if (!is.null(para)) {
      if (!.all_characterValues(para)) {
        msg <- paste0("'", name, "' should be a vector with strings.")
        stop(msg, call. = FALSE)
      }
    }
  } else {
    warning("Skipped arg validation.", call. = FALSE)
  }
}

#-------------------------------------------------------------------
.is_singleNumber <- function(para) {
  (is.integer(para) || is.numeric(para)) &&
    length(para) == 1L && !is.na(para)
}
.is_singleInteger <- function(para) {
  lg <- (is.integer(para) || is.numeric(para)) &&
    length(para) == 1L && !is.na(para)
  if (lg) {
    para <- abs(para)
    lg <- abs(para - round(para)) <= para
  }
  return(lg)
}
.is_singleString <- function(para) {
  is.character(para) && length(para) == 1L && !is.na(para)
}
.is_singleLogical <- function(para) {
  is.logical(para) && length(para) == 1L && !is.na(para)
}
.all_binaryValues <- function(para) {
  all(para %in% c(0, 1, NA))
}
.all_integerValues <- function(para, notNA = TRUE) {
  lg <- is.integer(para) || is.numeric(para) || all(is.na(para))
  if (lg) {
    para <- abs(para)
    lg <- all(abs(para - round(para)) <= para, na.rm=TRUE)
  }
  if(lg && notNA) lg <- !any(is.na(para))
  return(lg)
}
.all_numericValues <- function(para, notNA = TRUE) {
  lg <- is.numeric(para) || all(is.na(para))
  if(lg && notNA) lg <- !any(is.na(para))
  return(lg)
}
.all_characterValues <- function(para, notNA = TRUE) {
  lg <- is.character(para) || all(is.na(para))
  if(lg && notNA) lg <- !any(is.na(para))
  return(lg)
}
.is_numericVector <- function(para){
  is.vector(para) && .all_numericValues(para)
}
.is_integerVector <- function(para){
  is.vector(para) && .all_integerValues(para)
}
.is_characterVector <- function(para){
  is.vector(para) && .all_characterValues(para)
}
.is_color <- function(x) {
  res <- try(col2rgb(x), silent = TRUE)
  return(!"try-error" %in% class(res))
}
.is_singleColor <- function(para) {
  .is_color(para) && length(para) == 1L && !is.na(para)
}

#-------------------------------------------------------------------------------
#' @title Constructor of Weibull decay functions
#'
#' @description
#' The `weibullDecay()` constructor either creates a decay function or 
#' returns a `ggplot` object for visualizing the decay model. It is a utility 
#' function used internally by \code{\link{circularProjection}} and 
#' \code{\link{polarProjection}}.
#' 
#' @param decay A decay factor (in [0,1]). This term indicates how much a
#' \code{signal} decreases as a function of distance in pathway space. 
#' For example, at a specific distance defined by the \code{pdist} parameter, 
#' the signal intensity will be the initial signal multiplied by \code{decay}.
#' @param pdist A distance normalization term (in (0, 1]) at which the signal 
#' reaches `signal * decay`. This parameter is used to anchor the decay to 
#' a meaningful distance (see `details`). Also, when \code{pdist = 1}, it will
#' represent the diameter of the inscribed circle within the coordinate space
#' of a `PathwaySpace` object.
#' @param shape A parameter (>=1) of a Weibull function. When \code{shape=1}
#' the Weibull decay follows an exponential decay. When \code{shape>1}
#' the function is first convex, then concave with an inflection point.
#' @param plot A logical value indicating whether to return a `ggplot` object.
#' @param demo.signal A numeric value in `[-Inf, Inf]`, only passed when 
#' \code{plot = TRUE} to visualize the decay curve with a specific signal 
#' intensity. The value is ignored by the function constructor, as the decay 
#' function itself is returned without using an initial signal.
#' @return Returns either a function of the form 
#' \code{function(x, signal) \{ ... \}} or, if \code{plot = TRUE}, a `ggplot`
#' object illustrating the decay model.
#' @author Sysbiolab Team
#' @seealso \code{\link{linearDecay}}, \code{\link{expDecay}}
#' @details 
#' The `weibullDecay()` constructor creates a decay model based on the Weibull 
#' distribution. It describes how a signal decreases as a function of distance, 
#' controlled by both a decay rate and a shape parameter.
#' 
#' The decay function is defined as:
#' 
#' \deqn{y = signal \times decay^{\left(\frac{x}{pdist}\right)^{shape}}}
#' 
#' where \eqn{signal} represents the initial intensity, \eqn{decay} controls 
#' the rate of attenuation, \eqn{x} is a vector of normalized distances,
#' and \eqn{shape} adjusts the curvature of the decay. When \eqn{shape = 1}, 
#' the function follows an exponential decay. For \eqn{shape > 1}, the curve 
#' transitions from convex to concave, exhibiting an inflection point. The 
#' \eqn{pdist} parameter anchors the model such that:
#' \itemize{
#'   \item \eqn{y = signal} when \eqn{x = 0}
#'   \item \eqn{y = signal \times decay} when \eqn{x = pdist}
#' }
#' 
#' @examples
#' # Return a decay function
#' decay_fun <- weibullDecay(decay = 0.5, pdist = 0.4, shape = 2)
#'
#' # Plot decay model parameters
#' # weibullDecay(decay = 0.5, pdist = 0.4, shape = 2, plot = TRUE)
#' 
#' @importFrom ggplot2 geom_point geom_hline rel
#' @aliases weibullDecay
#' @export
#' 
weibullDecay <- function(decay = 0.001, pdist = 0.15, shape = 1.05,
  plot = FALSE, demo.signal = 1) {
  
  .validate.ps.args("singleNumber", "decay", decay)
  .validate.ps.args("singleNumber", "pdist", pdist)
  .validate.ps.args("singleNumber", "shape", shape)
  .validate.ps.args("singleNumber", "demo.signal", demo.signal)

  if(decay < 0 || decay > 1){
    stop("'decay' must be in [0,1]", call. = FALSE)
  }
  if(shape < 1){
    stop("'shape' must be >=1", call. = FALSE)
  }
  if(pdist <= 0 || pdist > 1){
    stop("'pdist' must be in (0,1]", call. = FALSE)
  }
  
  if(decay==0) decay <- .Machine$double.xmin
  if(decay==1) decay <- 1 - (1/.Machine$longdouble.max.exp)
  f <- function(x, signal){
    y <- signal * decay^( (x/pdist)^shape )
    return(y)
  }
  body(f) <- do.call("substitute", list(body(f),
    list(decay = decay, shape = shape, pdist = pdist)))
  attributes(f)$name <- "weibullDecay"
  
  if(plot){
    x <- seq(0, 1, 0.02)
    y <- f(x, demo.signal)
    p <- .plot_decay(x, y, demo.signal, decay, pdist, name = "weibullDecay", shape)
    return(p)
  } else {
    if(!missing(demo.signal)){
      warning("The value of 'demo.signal' is ignored by the function constructor.")
    }
    return(f)
  }
  
}

#-------------------------------------------------------------------------------
#' @title Constructor of exponential decay functions
#'
#' @description
#' The `expDecay()` constructor either creates a decay function or 
#' returns a `ggplot` object for visualizing the decay model. It is a utility 
#' function used internally by \code{\link{circularProjection}} and 
#' \code{\link{polarProjection}}.
#' 
#' @param decay A decay factor (in [0,1]). This term indicates how much a
#' \code{signal} decreases as a function of distance in pathway space. 
#' For example, at a specific distance defined by the \code{pdist} parameter, 
#' the signal intensity will be the initial signal multiplied by \code{decay}.
#' @param pdist A distance normalization term (in (0, 1]) at which the signal 
#' reaches `signal * decay`. This parameter is used to anchor the decay to 
#' a meaningful distance (see `details`). Also, when \code{pdist = 1}, it will
#' represent the diameter of the inscribed circle within the coordinate space
#' of a `PathwaySpace` object.
#' @param plot A logical value indicating whether to return a `ggplot` object.
#' @param demo.signal A numeric value in `[-Inf, Inf]`, only passed when 
#' \code{plot = TRUE} to visualize the decay curve with a specific signal 
#' intensity. The value is ignored by the function constructor, as the decay 
#' function itself is returned without using an initial signal.
#' @return Returns either a function of the form 
#' \code{function(x, signal) \{ ... \}} or, if \code{plot = TRUE}, a `ggplot`
#' object illustrating the decay model.
#' @author Sysbiolab Team
#' @seealso \code{\link{linearDecay}}, \code{\link{weibullDecay}}
#' @details 
#' The `expDecay()` constructor creates an exponential decay model. It describes 
#' how a signal decreases as a function of distance, controlled by a decay 
#' rate parameter.
#' 
#' The decay function is defined as:
#' 
#' \deqn{y = signal \times decay^{\left(\frac{x}{pdist}\right)}}
#' 
#' where \eqn{signal} represents the initial intensity, \eqn{decay} controls 
#' the rate of attenuation, and \eqn{x} is a vector of normalized distances.
#' The \eqn{pdist} parameter anchors the model such that:
#' \itemize{
#'   \item \eqn{y = signal} when \eqn{x = 0}
#'   \item \eqn{y = signal \times decay} when \eqn{x = pdist}
#' }
#' @examples
#' # Return a decay function
#' decay_fun <- expDecay(decay = 0.25, pdist = 0.5)
#'
#' # Plot decay model parameters
#' # expDecay(decay = 0.25, pdist = 0.5, plot = TRUE)
#' 
#' @aliases expDecay
#' @export
#'
expDecay <- function(decay = 0.001, pdist = 0.15, plot = FALSE, demo.signal = 1) {
  
  .validate.ps.args("singleNumber", "decay", decay)
  .validate.ps.args("singleNumber", "pdist", pdist)
  .validate.ps.args("singleLogical", "plot", plot)
  .validate.ps.args("singleNumber", "demo.signal", demo.signal)
  
  if(decay < 0 || decay > 1){
    stop("'decay' must be in [0,1]", call. = FALSE)
  }
  if(pdist <= 0 || pdist > 1){
    stop("'pdist' must be in [0,1]", call. = FALSE)
  }
  
  if(decay==0) decay <- .Machine$double.xmin
  if(decay==1) decay <- 1 - (1/.Machine$longdouble.max.exp)
  f <- function(x, signal){
    y <- signal * decay^(x/pdist)
    return(y)
  }
  body(f) <- do.call("substitute", list(body(f), list(decay = decay, 
    pdist = pdist)))
  attributes(f)$name <- "expDecay"
  
  if(plot){
    x <- seq(0, 1, 0.02)
    y <- f(x, demo.signal)
    p <- .plot_decay(x, y, demo.signal, decay, pdist, name = "expDecay")
    return(p) 
  } else {
    if(!missing(demo.signal)){
      warning("The value of 'demo.signal' is ignored by the function constructor.")
    }
    return(f)
  }
  
}

#-------------------------------------------------------------------------------
#' @title Constructor of linear decay functions
#'
#' @description
#' The `linearDecay()` constructor either creates a decay function or 
#' returns a `ggplot` object for visualizing the decay model. It is a utility 
#' function used internally by \code{\link{circularProjection}} and 
#' \code{\link{polarProjection}}.
#' 
#' @param decay A decay factor (in [0,1]). This term indicates how much a
#' \code{signal} decreases as a function of distance in pathway space. 
#' For example, at a specific distance defined by the \code{pdist} parameter, 
#' the signal intensity will be the initial signal multiplied by \code{decay}.
#' @param pdist A distance normalization term (in (0, 1]) at which the signal 
#' reaches `signal * decay`. This parameter is used to anchor the decay to 
#' a meaningful distance (see `details`). Also, when \code{pdist = 1}, it will
#' represent the diameter of the inscribed circle within the coordinate space
#' of a `PathwaySpace` object.
#' @param plot A logical value indicating whether to return a `ggplot` object.
#' @param demo.signal A numeric value in `[-Inf, Inf]`, only passed when 
#' \code{plot = TRUE} to visualize the decay curve with a specific signal 
#' intensity. The value is ignored by the function constructor, as the decay 
#' function itself is returned without using an initial signal.
#' @return Returns either a function of the form 
#' \code{function(x, signal) \{ ... \}} or, if \code{plot = TRUE}, a `ggplot`
#' object illustrating the decay model.
#' @author Sysbiolab Team
#' @seealso \code{\link{expDecay}}, \code{\link{weibullDecay}}
#' @details
#' The `linearDecay()` constructor creates a simple linear decay model. It 
#' describes how a signal decreases proportionally with distance.
#'
#' The decay function is defined as:
#' \deqn{y = signal \times \left(1 - (1 - decay) \times \frac{x}{pdist}\right)}
#'
#' where \eqn{signal} represents the initial intensity, \eqn{decay} defines 
#' the relative signal level at \eqn{pdist}, and \eqn{x} is a vector of 
#' normalized distances. The signal decreases uniformly from its initial 
#' value to \eqn{pdist}, which is a reference distance that anchors 
#' the model such that:
#' \itemize{
#'   \item \eqn{y = signal} when \eqn{x = 0}
#'   \item \eqn{y = signal \times decay} when \eqn{x = pdist}
#' }
#' 
#' This makes the linear form consistent with the exponential and Weibull decay
#' functions, both of which also reach \eqn{signal \times decay} at the
#' reference distance.
#' 
#' @examples
#' # Return a decay function
#' decay_fun <- linearDecay(decay = 0.5, pdist = 0.25)
#'
#' # Plot decay model parameters
#' # linearDecay(decay = 0.5, pdist = 0.25, plot = TRUE)
#'
#' @aliases linearDecay
#' @export
#'
linearDecay <- function(decay = 0.001, pdist = 0.15, plot = FALSE, 
  demo.signal = 1) {
  
  .validate.ps.args("singleNumber", "decay", decay)
  .validate.ps.args("singleNumber", "pdist", pdist)
  .validate.ps.args("singleLogical", "plot", plot)
  .validate.ps.args("singleNumber", "demo.signal", demo.signal)
  
  if(decay < 0 || decay > 1){
    stop("'decay' must be in [0,1]", call. = FALSE)
  }
  if(pdist <= 0 || pdist > 1){
    stop("'pdist' must be in [0,1]", call. = FALSE)
  }
  
  f <- function(x, signal){
    # linear decay
    y <- signal * ( 1 - ( (1 - decay) * x/pdist ) )
    # clip 'y' to prevent flipping sign
    y <- pmax(y * sign(signal), 0) * sign(signal)
    return(y)
  }
  body(f) <- do.call("substitute", list(body(f), 
    list(decay = decay, pdist = pdist)))
  attributes(f)$name <- "linearDecay"
  
  if(plot){
    x <- seq(0, 1, 0.02)
    y <- f(x, demo.signal)
    p <- .plot_decay(x, y, demo.signal, decay, pdist, name = "linearDecay")
    return(p)
  } else {
    if(!missing(demo.signal)){
      warning("The value of 'demo.signal' is ignored by the function constructor.")
    }
    return(f)
  }
  
}

#-------------------------------------------------------------------------------
.plot_decay <- function(x, y, signal, decay, pdist, name, shape){
  if(missing(shape)){
    call <- paste0(name,"(...)\n", 
      "decay = ", format(decay, digits=2), "\n",
      "pdist = ", format(pdist, digits=2)
    )
  } else {
    call <- paste0(name,"(...)\n", 
      "decay = ", format(decay, digits=2), "\n", 
      "pdist = ", format(pdist, digits=2), "\n", 
      "shape = ", format(shape, digits=2)
    )
  }
  si <- ifelse(signal>=0, TRUE, FALSE)
  pi <- ifelse(pdist<0.5, TRUE, FALSE)
  data <- data.frame(x = x, y = y)
  if(signal>=0 && signal <= 1){
    limits <- c(0, 1.025)
    breaks <- pretty(c(0, 1))
  } else if(signal >= -1 && signal < 0){
    limits <- c(-1.025, 0)
    breaks <- pretty(c(-1, 0))
  } else {
    mx <- signal*1.025
    limits <- range(c(0, 1.025 * sign(mx),mx))
    breaks <- pretty(limits)
    breaks <- breaks[abs(breaks)<abs(mx)]
  }
  et1 <- element_text(size=12)
  et2 <- element_text(size=13)
  p <- ggplot(aes(x = x, y = y), data = data) +
    scale_y_continuous(limits = limits, breaks = breaks) + 
    scale_x_continuous(breaks = seq(0, 1, 0.2)) + 
    labs(y="Signal Intensity", x="Normalized Distance") + 
    # geom_vline(xintercept=0, color="grey40") +
    # geom_hline(yintercept=0, color="grey40") +
    annotate("segment", x = 0, xend = pdist, 
      y = decay*signal, yend = decay*signal, 
      color = "green4", linetype = "21", linewidth = 0.75) +
    annotate("segment", x = pdist, xend = pdist, 
      y = decay*signal*ifelse(si,0,1), yend = signal*ifelse(si,decay,1), 
      color = "green4", linetype = "21", linewidth = 0.75) +
    annotate("text", 
      x = 0, y = decay*signal, 
      label = "italic(S)[0] %*% italic(decay)", parse = TRUE,
      color = "green4", vjust = -0.3, hjust = 0, size=4) + 
    annotate("text", 
      x = pdist*ifelse(pi,1.05,0.95), y = signal * ifelse(si,0.05,0.95), 
      label="italic(pdist)", parse = TRUE, color = "green4", vjust = 0,
      hjust = ifelse(pi,0,1), size=4) + 
    annotate("text", 
      x = max(x), y = signal, label=call, 
      vjust = ifelse(si,1,0), hjust = 1, size=4) + 
    geom_point(color = "grey50") + 
    theme(aspect.ratio = 0.85, axis.text = et1, axis.title = et2,
      panel.background = element_rect(fill = "grey95"), 
      panel.grid.major = element_line(linewidth = rel(1.2), colour = "white"),
      panel.grid.minor = element_line(linewidth = rel(0.6), colour = "white")
      )
  
  # Add the new point to the plot
  p <- p + 
    geom_point(data = data.frame(x = 0, y = signal), 
      aes(x = x, y = y), color = "red", size = 6, shape = 18) +
    geom_point(data = data.frame(x = pdist, y = signal * decay),
      aes(x = x, y = y), color = "green3", size = 4, shape = 19) +
    annotate("text", x = 0.02, y = signal, 
      label = "italic(S)[0] * ' (initial signal)'", parse = TRUE,
      color = "red", hjust = 0, vjust = ifelse(si,0,1), size = 4)
  
  return(p)
}

#-------------------------------------------------------------------------------
#' @title Signal aggregation functions
#'
#' @description Signal aggregation functions for \code{\link{circularProjection}}
#' and \code{\link{polarProjection}} internal calls. The aggregation should be 
#' symmetric with respect to signal polarity, ensuring that opposite signals 
#' produce corresponding outputs.
#' 
#' @param method A character string specifying the method for
#' signal aggregation, returning either a customized \code{\link{mean}} or 
#' \code{\link{weighted.mean}} function.
#' @return Returns a function of the form: \code{function(x) { ... }}
#' @author Sysbiolab Team
#' @seealso \code{\link{circularProjection}}, \code{\link{polarProjection}}, 
#' \code{\link{weighted.mean}}
#' @examples
#' aggregate.fun <- signalAggregation()
#' 
#' @importFrom stats weighted.mean
#' @rdname signalAggregation
#' @export
#' 
signalAggregation <- function(method = c("mean", "wmean", "log.wmean", 
    "exp.wmean")){
    method <- match.arg(method)
    if(method=="mean"){
        f <- function(x){
            y <- mean(x, na.rm = TRUE)
            return(y)
        }
        attributes(f)$name <- "meanSignal"
        return(f)
    } else if(method=="wmean"){
        f <- function(x){
            y <- weighted.mean(x, abs(x), na.rm = TRUE)
            return(y)
        }
        attributes(f)$name <- "weightedSignal"
        return(f)
    } else if(method=="log.wmean"){
        f <- function(x){
            y <- weighted.mean(x, log1p(abs(x)), na.rm = TRUE)
            return(y)
        }
        attributes(f)$name <- "logWeightedSignal"
        return(f)
    } else if(method=="exp.wmean"){
        f <- function(x){
            w <- abs(x)
            w <- (w / sum(w, na.rm = TRUE))^2
            y <- weighted.mean(x, w, na.rm = TRUE)
            return(y)
        }
        attributes(f)$name <- "expWeightedSignal"
        return(f)
    }

}
attributes(signalAggregation)$name <- "signalAggregation"


#-------------------------------------------------------------------------------
#' @title Polar transformation functions
#'
#' @description Creates polar transformation functions for 
#' \code{\link{polarProjection}} internal calls. These functions are used to 
#' adjusts signal decay according to point-to-edge angular distances, 
#' with options to attenuate angular shapes.
#' 
#' @param method String indicating the transformation to apply. 
#' Must be one of: "power", "gaussian", or "logistic".
#' @param s Single numeric value in \code{[0, 1]}. Controls the spread around 
#' the \code{x} mean of the Gaussian function.
#' @param k Single numeric value \code{>=1}. Controls the steepness of 
#' the logistic function.
#' @param m Single numeric value in \code{[0, 1]}. Specifies the midpoint of 
#' the logistic function.
#' @return Returns a function of the form: \code{function(x, beta) { ... }}, 
#' that applies the specified shape-based transformation.
#' @author Sysbiolab Team
#' @seealso \code{\link{polarProjection}}
#' @details 
#' The polar transformation controls how much the projected signal decays as 
#' a function of the angular distance between a point in pathway space and 
#' a reference edge axis. The function returned by \code{polarDecay()} expects 
#' two arguments, with the following signature: 
#' \code{function(x, beta) { ... }}.
#' 
#' **Power:**
#' \deqn{x^{\beta}}
#' where \eqn{x} is a vector of normalized angular distances (in \code{[0, 1]})
#' and \eqn{beta} is a non-negative exponent that controls the rate of signal 
#' decay. Increasing \eqn{beta} results in a steeper decay rate, modulating 
#' the angular span of the projection.
#' 
#' **Gaussian:**
#' \deqn{\exp\left(-\frac{(1-x)^2}{2\sigma^2}\right)^{\beta}}
#' where \eqn{sigma} controls the spread around the mean, creating 
#' fuzzier effect on projections.
#' 
#' **Logistic:**
#' \deqn{(1 / (1 + \exp(k (x - m))))^{\beta}}
#' where \eqn{k} is the steepness and \eqn{m} is the function's midpoint, 
#' making more gradual transitions.
#'   
#' These transformations are intended to be plugged into the higher-level 
#' \code{\link{polarProjection}} function, allowing user control over the 
#' polar projection profiles. 
#' 
#' @examples
#' polar.fun <- polarDecay("power")
#' 
#' @md
#' @rdname polarDecay
#' @export
#' 
polarDecay <- function(method = c("power", "gaussian", "logistic"), 
    s = 0.5, k = 10, m = 0.5) {
    
    .validate.ps.args("singleNumber", "s", s)
    .validate.ps.args("singleNumber", "k", k)
    .validate.ps.args("singleNumber", "m", m)
    method <- match.arg(method)
    
    if(s < 0 || s>1)stop("'s' must be in [0,1]", call. = FALSE)
    if(k < 1)stop("'k' must be >=1", call. = FALSE)
    if(m < 0 || m>1)stop("'m' must be in [0,1]", call. = FALSE)
    
    if (method == "power") {
        if(!missing(s)){
            warning("'s' ignored unless method is 'gaussian'.")
        }
        if(!missing(k)){
            warning("'k' ignored unless method is 'logistic'.")
        }
        if(!missing(m)){
            warning("'m' ignored unless method is 'logistic'.")
        }
        f <- function(x, beta){
            y <- x ^ beta
            return(y)
        }
        attributes(f)$name <- "power"
        return(f)
        
    } else if (method == "gaussian") {
        if(!missing(k)){
            warning("'k' ignored unless method is 'logistic'.")
        }
        if(!missing(m)){
            warning("'m' ignored unless method is 'logistic'.")
        }
        f <- function(x, beta){
            y <- exp( - ((1 - x)^2) / (2*s^2) ) ^ beta
            return(y)
        }
        body(f) <- do.call("substitute", list(body(f), list(s = s)))
        attributes(f)$name <- "gaussian"
        return(f)
        
    } else if (method == "logistic") {
        if(!missing(s)){
            warning("'s' ignored unless method is 'gaussian'.")
        }
        f <- function(x, beta){
            y <- ( 1 / (1 + exp(-k * (x - m))) ) ^ beta
            return(y)
        }
        body(f) <- do.call("substitute", list(body(f), list(k = k, m = m)))
        attributes(f)$name <- "logistic"
        return(f)
        
    }
}

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#' @title Deprecated function
#'
#' @description 
#' Use \code{\link{weibullDecay}}, \code{\link{expDecay}}, 
#' and \code{\link{linearDecay}}.
#' 
#' @param ... Deprecated arguments
#' @return Stop unconditionally
#' @author Sysbiolab Team
#' @examples
#' decay.fun <- weibullDecay()
#' 
#' @rdname signalDecay
#' @export
#'
signalDecay <- function(...){
  lifecycle::deprecate_stop("1.0.3", "signalDecay()", "weibullDecay()")
}

setGeneric("circularProjection", 
  function(ps, ...) {
    standardGeneric("circularProjection")
  }, package = "PathwaySpace"
)

setGeneric("polarProjection", 
  function(ps, ...) {
    standardGeneric("polarProjection")
  }, package = "PathwaySpace"
)

setGeneric("silhouetteMapping", 
  function(ps, ...) {
    standardGeneric("silhouetteMapping")
  }, package = "PathwaySpace"
)

setGeneric("summitMapping", 
  function(ps, maxset = 30, minsize = 30, 
    threshold = 0.5, ...) {
    standardGeneric("summitMapping")
  }, package = "PathwaySpace"
)

setGeneric("plotPathwaySpace", 
  function(ps, ...) {
    standardGeneric("plotPathwaySpace")
  }, package = "PathwaySpace"
)

setGeneric("getPathwaySpace", 
  function(ps, what = "status") {
    standardGeneric("getPathwaySpace")
  }, package = "PathwaySpace"
)

setGeneric("vertexSignal", function(x, value) standardGeneric("vertexSignal"))
setGeneric("vertexSignal<-", function(x, value) standardGeneric("vertexSignal<-"))

setGeneric("vertexDecay", function(x, value) standardGeneric("vertexDecay"))
setGeneric("vertexDecay<-", function(x, value) standardGeneric("vertexDecay<-"))

#' @title Plotting 2D-landscape images for the PathwaySpace package
#'
#' @description \code{plotPathwaySpace} is a wrapper function to 
#' create dedicated ggplot graphics for PathwaySpace-class objects.
#'
#' @param ps A \linkS4class{PathwaySpace} class object.
#' @param colors A vector of colors.
#' @param bg.color A single color for background.
#' @param si.color A single color for silhouette.
#' (see \code{\link{silhouetteMapping}}).
#' @param si.alpha A transparency level in [0, 1], used to adjust the 
#' opacity of the silhouette. This parameter is useful for improving the 
#' perception of a background image, when one is available.
#' @param theme Name of a custom PathwaySpace theme. These themes 
#' (from 'th0' to 'th3') consist mainly of preconfigured ggplot settings, 
#' which the user can subsequently refine using \code{\link[ggplot2]{ggplot2}}.
#' @param title A string for the title.
#' @param xlab The title for the 'x' axis of a 2D-image space.
#' @param ylab The title for the 'y' axis of a 2D-image space.
#' @param zlab The title for the 'z' axis of the image signal.
#' @param font.size A single numeric value passed to plot annotations.
#' @param font.color A single color passed to plot annotations.
#' @param zlim The 'z' limits of the plot (a numeric vector with two numbers).
#' If NULL, limits are determined from the range of the input values.
#' @param slices A single positive integer value used to split 
#' the image signal into equally-spaced intervals.
#' @param add.grid A logical value indicating whether to add gridlines to 
#' the image space. However, gridlines will only appear when the image 
#' is decorated with graph silhouettes (see \code{\link{silhouetteMapping}}).
#' @param grid.color A color passed to \code{\link[ggplot2]{geom_point}}.
#' @param add.summits A logical value indicating whether to add contour 
#' lines to 'summits' (when summits are available; 
#' see \code{\link{summitMapping}}).
#' @param label.summits A logical value indicating whether to label summits.
#' @param summit.color A color passed to 'summits'.
#' @param add.marks A logical value indicating whether to plot vertex labels.
#' @param marks A vector of vertex names to be highlighted in the 
#' image space. This argument overrides 'add.labels'.
#' @param mark.size A size argument passed to \code{\link[ggplot2]{geom_text}}.
#' @param mark.color A color passed to \code{\link[ggplot2]{geom_text}}.
#' @param mark.padding A box padding argument passed to 
#' \code{\link[ggrepel]{geom_text_repel}}.
#' @param mark.line.width A line width argument passed to 
#' \code{\link[ggrepel]{geom_text_repel}}.
#' @param use.dotmark A logical value indicating whether "marks" should be 
#' represented as dots.
#' @param add.image A logical value indicating whether to add a background 
#' image, when one is available (see \code{\link[RGraphSpace]{GraphSpace}}).
#' @return A ggplot-class object.
#' @author Sysbiolab Team, Mauro Castro.
#' @seealso \code{\link{circularProjection}}
#' @examples
#' # Load a demo igraph
#' data('gtoy1', package = 'RGraphSpace')
#'
#' # # Check graph validity
#' gs <- GraphSpace(gtoy1)
#' 
#' gs <- normalizeGraphSpace(gs)
#' 
#' # Create a PathwaySpace object
#' ps <- buildPathwaySpace(gs, nrc = 300)
#' # note: adjust 'nrc' to increase image resolution
#'
#' # Set '1s' as vertex signal
#' vertexSignal(ps) <- 1
#'
#' # Create a 2D-landscape image
#' ps <- circularProjection(ps, k = 2,
#'    decay.fun = weibullDecay(pdist = 0.4))
#'
#' # Plot a 2D-landscape image
#' plotPathwaySpace(ps)
#' 
#' @import methods
#' @docType methods
#' @importFrom ggplot2 ggplot annotate element_text theme theme_bw theme_gray
#' @importFrom ggplot2 scale_x_continuous scale_y_continuous expansion
#' @importFrom ggplot2 aes scale_fill_gradientn annotation_raster
#' @importFrom ggplot2 coord_fixed geom_raster labs
#' @importFrom ggplot2 margin element_blank element_rect element_line
#' @importFrom grid unit
#' @importFrom ggrepel geom_text_repel
#' @importFrom grDevices convertColor col2rgb rgb as.raster
#' @importFrom grDevices adjustcolor colorRampPalette
#' @importFrom stats runif
#' @importFrom RGraphSpace plotGraphSpace theme_gspace_coords
#' @importFrom patchwork wrap_plots
#' @rdname plotPathwaySpace-methods
#' @aliases plotPathwaySpace
#' @export
#'
setMethod("plotPathwaySpace", "PathwaySpace", 
  function(ps, colors = pspace.cols(), bg.color = "grey95", 
    si.color = "grey85", si.alpha = 1,
    theme = c("th0", "th1", "th2", "th3"),
    title = "PathwaySpace", 
    xlab = "Pathway coordinates 1", 
    ylab = "Pathway coordinates 2", 
    zlab = "Density", 
    font.size = 1, font.color = "white",
    zlim = NULL, slices = 25, 
    add.grid = TRUE, grid.color = "white", 
    add.summits = TRUE, label.summits = TRUE, summit.color = "white",
    add.marks = FALSE, marks = NULL, mark.size = 3, mark.color = "white", 
    mark.padding = 0.5, mark.line.width = 0.5, use.dotmark = FALSE, 
    add.image = FALSE) {
    
    #--- validate the ps object and args
    if (!.checkStatus(ps, "Projection") && !.checkStatus(ps, "Silhouette")) {
      stop("NOTE: 'ps' needs to be evaluated by a 'projection' method!",
        call. = FALSE)
    }
    .validate.ps.args("singleNumber", "si.alpha", si.alpha)
    .validate.ps.args("singleString", "title", title)
    .validate.ps.args("singleString", "xlab", xlab)
    .validate.ps.args("singleString", "ylab", ylab)
    .validate.ps.args("singleString", "zlab", zlab)
    .validate.ps.args("singleNumber", "font.size", font.size)
    .validate.ps.args("singleInteger", "slices", slices)
    .validate.ps.args("singleLogical", "add.grid", add.grid)
    .validate.ps.args("singleLogical", "add.summits", add.summits)
    .validate.ps.args("singleLogical", "label.summits", label.summits)
    .validate.ps.args("singleLogical", "add.marks", add.marks)
    .validate.ps.args("singleNumber", "mark.size", mark.size)
    .validate.ps.args("singleNumber", "mark.padding", mark.padding)
    .validate.ps.args("singleNumber","mark.line.width", mark.line.width)
    .validate.ps.args("singleLogical", "use.dotmark", use.dotmark)
    .validate.ps.args("singleLogical", "add.image", add.image)
    .validate.colors("singleColor", "mark.color", mark.color)
    .validate.colors("allColors","colors", colors)
    .validate.colors("singleColor", "si.color", si.color)
    .validate.colors("singleColor", "font.color", font.color)
    .validate.colors("singleColor", "grid.color", grid.color)
    .validate.colors("singleColor", "summit.color", summit.color)
    .validate.psplot.args("marks", marks)
    if(!is.na(bg.color) ){
      .validate.colors("singleColor", "bg.color", bg.color)
    }
    theme <- match.arg(theme)
    if(!is.null(zlim)) {
      .validate.ps.args("numeric_vec", "zlim", zlim)
      if(length(zlim)!=2) 
        stop("'zlim' should be a numeric vector of lenght 2.", call. = FALSE)
    }
    if (si.alpha < 0 || si.alpha > 1) {
      stop("'si.alpha' should be in [0,1]", call. = FALSE)
    }
    #--- get slots from ps
    summits <- getPathwaySpace(ps, "summits")
    cset <- getPathwaySpace(ps, "summit_contour")
    silstatus <- .checkStatus(ps, "Silhouette")
    gxy <- getPathwaySpace(ps, "projections")$gxy
    gxyz <- getPathwaySpace(ps, "projections")$gxyz
    pars <- getPathwaySpace(ps, "projections")$pars
    
    #--- set colors
    colors <- colorRampPalette(colors)(25)
    if(pars$ps$configs$scale.type=="negpos"){
      slices <- ceiling(slices/2) * 2
    }
    
    # set scales
    if(is.null(zlim)){
      zlim <- pars$ps$configs$zlim
    } else {
      gxyz[gxyz < zlim[1]] <- zlim[1]
      gxyz[gxyz > zlim[2]] <- zlim[2]
    }
    if (all(zlim == 0)) zlim[2] <- 1
    
    #--- set gspace theme
    gs_theme <- theme_gspace_coords(theme = theme, 
      is_norm = TRUE, xlab = xlab, ylab = ylab, 
      txt_size = font.size, leg_size = font.size, 
      bg_color = "grey95")
    
    gs_pars <- attributes(gs_theme)$gspace_pars
    
    #--- trim colors and set zlim args
    cl <- .trimcols(colors, bg.color, zlim, pars)
    cl <- .set_theme_zlim(cl, zlim)
    bks <- seq(zlim[1], zlim[2], length.out = slices)
    gxyz[, ] <- bks[cut(as.numeric(gxyz), breaks = sort(unique(bks)),
      include.lowest = TRUE)]
    
    #--- get grid
    gridln <- .getGrid(gxyz, gs_pars$axis.ticks)
    gridln <- as.numeric(gridln)
    gridln <- data.frame(
      arrayInd(seq_along(gridln), .dim = dim(gxyz)), gridln)
    colnames(gridln) <- c("Y", "X", "L")
    
    #--- set main input for ggplot
    gxyz <- data.frame(arrayInd(seq_along(gxyz), .dim = dim(gxyz)),
      as.numeric(gxyz))
    colnames(gxyz) <- c("Y", "X", "Z")
    
    #--- scale coordinates to plot space
    gxyz$X <- scales::rescale(gxyz$X)
    gxyz$Y <- scales::rescale(gxyz$Y)
    gxyz$L <- gridln$L
    
    #--- set a bg color effect, scaling alpha to z
    if(si.alpha < 1){
      si.color <- adjustcolor(si.color, si.alpha)
      gz.alpha <- .scale_alpha(si.alpha, gxyz, zlim, pars)
    } else {
      gz.alpha <- 1
    }
    
    #--- initialize ggplot
    ggp <- .set_pspace(gxyz, zlab, cl, si.color)
    
    #--- adjust gs_theme
    ggp <- ggp + gs_theme
    if(theme == "th2"){
      ggp <- ggp + ggplot2::theme(panel.grid = element_blank())
    } else if(theme == "th3"){
      ggp <- ggp + ggplot2::theme(panel.grid = element_blank(),
        legend.position = "bottom")
    }
    #--- add image
    if(pars$image.layer){
      img <- getPathwaySpace(ps, "image")
      if(add.image){
        ggp <- .add_image(ggp, img)
      } else {
        ggi <- .add_image(ggp, img)
        if(add.grid) ggi <- .add_grid(ggi, gxyz, grid.color)
      }
    }
    ggp <- ggp + ggplot2::labs(fill = zlab)
      
    #--- add main projection
    ggp <- ggp + ggplot2::geom_raster(interpolate = FALSE, 
      na.rm=TRUE, alpha = gz.alpha)
    
    #--- add a grid
    if(add.grid) ggp <- .add_grid(ggp, gxyz, grid.color)
    
    #--- add contour lines if available
    bl <- add.summits || label.summits
    if (bl && !is.null(summits) && length(summits) > 0 && sum(cset) > 0) {
      ggp <- .add_contour(ggp, gxyz, summits, cset, 
        summit.color, mark.size, add.summits, label.summits)
    }
    
    #--- add marks if available
    if (!is.null(marks)) {
      ggp <- .add_marks(ggp, gxy, pars, marks, mark.size,
        mark.color, mark.padding, mark.line.width, use.dotmark)
    } else if(add.marks){
      ggp <- .add_marks(ggp, gxy, pars, marks=rownames(gxy), 
        mark.size, mark.color, mark.padding, mark.line.width, use.dotmark)
    }
    
    #--- add annotations
    if(.checkStatus(ps, "Projection")){
      ggp <- .custom_annotations(ggp, title, pars, font.size, 
        font.color, silstatus, si.color)
    }
    
    if(pars$image.layer && !add.image){
      ggl <- list(graph = ggp, image = ggi)
      return(ggl)
    } else {
      return(ggp)
    }
    
  }
)

#-------------------------------------------------------------------------------
.scale_alpha <- function(si.alpha, gxyz, zlim, pars){
  az <- gxyz$Z
  mxz <- max(abs(zlim))
  if(pars$ps$configs$scale.type == "negpos") {
    slim <- 0.5 * (1 - si.alpha)
    slim <- slim * mxz
    az[az < 0 & az < -slim] <- mxz
    az[az > 0 & az > slim] <- mxz
    az <- abs(az)
  } else if(pars$ps$configs$scale.type == "neg") {
    az[az < zlim[2]] <- zlim[1]
  } else {
    az[az > zlim[1]] <- zlim[2]
  }
  pars$ps$configs$zlim
  az <- az/mxz
  alpha <- az^10 + si.alpha
  return(alpha)
}

#-------------------------------------------------------------------------------
.custom_annotations <- function(ggp, title, pars, font.size, 
  font.color, silstatus, si.color){
  if(silstatus){
    if(si.color=="grey85"){
      fcol <- "grey20"
    } else {
      fcol <- font.color
    }
    sep <- "; "
    xlab <- 0.01
    hjust <- 0
  } else {
    fcol <- font.color
    sep <- "\n"
    xlab <- 0.99
    hjust <- 1
  }
  ggp <- ggp + ggplot2::annotate("text", label = title,
    colour = fcol, size = font.size*4, x = 0, y = 0.99, 
    hjust = 0, vjust = 1)
  dfun <- pars$ps$decay$fun
  if(!is.null(dfun)){
    if(dfun == "weibullDecay"){
      dfun <- "Weibull decay"
    } else if(dfun == "expDecay"){
      dfun <- "Exponential decay"
    } else if(dfun == "linearDecay"){
      dfun <- "Linear decay"
    } else {
      dfun <- "Custom decay"
    }
    pars$ps$dfun <- dfun
  } else {
    pars$ps$dfun <- "Custom decay"
  }
  if(pars$ps$projection=="Polar"){
    annot <- pars$ps[c("projection", "dfun", "k", "beta")]
    annot$k <- paste0("k = ", annot$k, "; ")
    annot$beta <- paste0("beta = ", pars$ps$beta)
  } else {
    annot <- pars$ps[c("projection", "dfun", "k")]
    annot$k <- paste0("k = ", annot$k)
  }
  annot$projection <- paste0(annot$projection, " projection", sep)
  annot$dfun <- paste0(annot$dfun, sep)
  annot <- paste(unlist(annot), collapse = "")
  ggp <- ggp + ggplot2::annotate("text", label = annot,
    colour = fcol, size = font.size*3, x = xlab, 
    y = 0.01, hjust = hjust, vjust = 0)
  return(ggp)
}

#-------------------------------------------------------------------------------
.set_pspace <- function(gxyz, zlab, cl, si.color){
  X <- Y <- Z <- NULL
  ggp <- ggplot2::ggplot(gxyz, ggplot2::aes(X, Y, fill = Z)) +
    ggplot2::scale_fill_gradientn(limits = cl$zlim,
      breaks = cl$breaks, labels = names(cl$breaks),
      colours = cl$pal, aesthetics = "fill", na.value = si.color) +
    ggplot2::coord_fixed()
  return(ggp)
}

#-------------------------------------------------------------------------------
.add_grid <- function(ggp, gxyz, grid.color){
  dt <- gxyz[gxyz$L == 1, c("X", "Y")]
  ggp <- ggp + ggplot2::annotate(geom = "point", x = dt$X, 
    y = dt$Y, color = grid.color, size = 0.2, pch = 15)
  return(ggp)
}

#-------------------------------------------------------------------------------
.add_image <- function(ggp, image){
  ggp <- ggp + annotation_raster(image, xmin = 0, xmax = 1, ymin = 0, ymax = 1)
  return(ggp)
}

#-------------------------------------------------------------------------------
.add_contour <- function(ggp, gxyz, summits, cset, summit.color, 
  mark.size, add.summits, label.summits) {
  setnames <- names(summits)
  if (is.null(setnames)) {
    setnames <- seq_along(setnames)
  }
  cset <- data.frame(which(!is.na(cset) | is.na(cset), arr.ind = TRUE),
    as.numeric(cset))
  colnames(cset) <- c("Y", "X", "C")
  gxyz <- cbind(gxyz, C = cset$C)
  xy.tx <- NULL
  concav <- sort(unique(gxyz$C))[-1]
  for (i in seq_along(concav)) {
    xy.cv <- gxyz[gxyz$C == i, c("X", "Y")]
    xy.tx <- rbind(xy.tx, colMeans(xy.cv))
    if(add.summits){
      ggp <- ggp + ggplot2::annotate(geom = "tile", x = xy.cv[, 1], 
        y = xy.cv[, 2], color = summit.color,
        fill = summit.color, linewidth = 0.2)
    }
  }
  rownames(xy.tx) <- setnames
  xy.tx <- as.data.frame(xy.tx)
  if(label.summits){
    if(add.summits){
      ggp <- ggp + ggplot2::annotate(geom = "text", x = xy.tx[, 1],
        y = xy.tx[, 2], label = rownames(xy.tx), vjust = 0.5, hjust = 0.5,
        color = summit.color, size = mark.size,
        fontface = "bold")
    } else {
      nudgex <- sign(xy.tx$X - 0.5) * xy.tx$X
      nudgey <- sign(xy.tx$Y - 0.5) * xy.tx$Y
      ggp <- ggp + ggrepel::geom_text_repel(
        mapping = aes(label = rownames(xy.tx),segment.size = 0.4), 
        xlim = c(0.1, 0.9), ylim = c(0.1, 0.9),
        data = xy.tx, min.segment.length = 0.1,
        fontface = "bold", force = 3, segment.linetype = 1,
        max.overlaps = nrow(xy.tx) + 5, point.padding = 0, seed = 123,
        max.iter = 20000, max.time = 30, nudge_x = nudgex * 0.075,
        nudge_y = nudgey * 0.075, size = mark.size, colour = summit.color,
        segment.colour = summit.color)
    }
  }
  return(ggp)
}

#-------------------------------------------------------------------------------
.add_marks <- function(ggp, gxy, pars, marks, mark.size,
  mark.color, mark.padding, mark.line.width, use.dotmark) {
  
  # scale coordinates to plot space
  gxy_df <- as.data.frame(gxy)
  gxy_df <- gxy_df[, c("X", "Y")]
  gxy_df$X <- scales::rescale(gxy_df$X, from = c(1, pars$ps$nrc))
  gxy_df$Y <- scales::rescale(gxy_df$Y, from = c(1, pars$ps$nrc))
  
  # match  marks
  marks <- marks[!duplicated(marks)]
  if (is.null(names(marks))) names(marks) <- marks
  names(marks) <- ifelse(is.na(names(marks)), marks, names(marks))
  names(marks) <- ifelse(names(marks) == "", marks, names(marks))
  idx_df <- .get_mark_idx(marks, gxy_df)
  if(any(is.na(idx_df$idx))){
    stop("All 'marks' should be annotated in the 'PathwaySpace' object.",
      call. = FALSE)
  }
  
  # set df
  gxy_df <- gxy_df[idx_df$idx, , drop = FALSE]
  gxy_df$ID <- idx_df$label
  ID <- NULL
  
  # call to ggplot
  if (use.dotmark) {
    ggp <- ggp + ggplot2::annotate("point", shape = 5, 
      colour = mark.color, size = mark.size * 0.4, stroke = 0.4, 
      x = gxy_df$X, y = gxy_df$Y)
  }
  
  nudgex <- sign(gxy_df$X - 0.5) * gxy_df$X * 0.1
  nudgey <- sign(gxy_df$Y - 0.5) * gxy_df$Y * 0.1
  nudgex <- pmax(nudgex,-0.1)
  nudgey <- pmax(nudgey,-0.1)
  nudgex <- pmin(nudgex, 0.1)
  nudgey <- pmin(nudgey, 0.1)
  
  ggp <- ggp + ggrepel::geom_text_repel(
    mapping = aes(label = ID,segment.size = mark.line.width), 
    xlim = c(0.1, 0.9), ylim = c(0.1, 0.9),
    data = gxy_df, min.segment.length = 0.1,
    fontface = "bold", force = 3, segment.linetype = "2121", 
    max.overlaps = nrow(gxy_df) + 5, point.padding = 0, seed = 123, 
    max.iter = 20000, max.time = 30, nudge_x = nudgex, 
    nudge_y = nudgey, size = mark.size, colour = mark.color, 
    segment.colour = mark.color, box.padding = mark.padding)
  
  return(ggp)
}
.get_mark_idx <- function(marks, gxy_df){
  idx_df <- data.frame(name=marks, label=names(marks))
  idx_df$idx1 <- match(marks, rownames(gxy_df))
  idx_df$idx2 <- match(names(marks), rownames(gxy_df))
  idx_df$idx <- sapply(seq_len(nrow(idx_df)), function(i){
    idx <- idx_df[i, c(3,4)]
    idx[!is.na(idx)][1]
  })
  idx_df <- idx_df[,c(1,2,5)]
  return(idx_df)
}

#-------------------------------------------------------------------------------
.set_theme_zlim <- function(cl, zlim){
  # adjust labels for z-axis midle and tips
  bks_names <- cl$breaks
  bks_names <- format(bks_names,  trim = TRUE)
  n <- length(bks_names)
  bks_names[!seq_len(n) %in% c(1, ceiling(n/2), n)] <- ""
  # bks_names <- format(bks_names, justify=cl$justify)
  names(cl$breaks) <- bks_names
  # expand 'zlim' and palette tips
  expand <- TRUE
  if(expand){
    tips <- (zlim[2] - zlim[1]) * 0.1
    cl$zlim <- c(zlim[1] - tips, zlim[2] + tips)
    cl$pal <- c(cl$pal[1], cl$pal, cl$pal[length(cl$pal)])
  } else {
    cl$zlim <- zlim
  }
  return(cl)
}
#-------------------------------------------------------------------------------
#--- grid lines
.getGrid <- function(gxyz, ticks = c(0.2, 0.4, 0.6, 0.8), ndots = 100) {
  ticks <- ticks[ticks>0 & ticks <1]
  nc <- ncol(gxyz)
  nr <- nrow(gxyz)
  by <- ceiling(nc/ndots)
  ic <- ceiling(ticks * nc)
  ir <- ceiling(ticks * nr)
  grid1 <- grid2 <- array(0, dim = dim(gxyz))
  grid1[ir, seq(1, ncol(grid1), by = by)] <- 1L
  grid2[seq(1, nrow(grid2), by = by), ic] <- 1L
  grid1 <- (grid1 + grid2) > 0
  grid1[grid1] <- 1
  grid1[!is.na(gxyz)] <- 0
  return(grid1)
}

#-------------------------------------------------------------------------------
.trimcols <- function(colors, bg.color, zlim, pars) {
  if(pars$ps$configs$scale.type == "negpos") {
    
    if (is.na(bg.color)) {
      if (length(colors) %% 2 == 1){
        bg.color <- colors[(length(colors)+1)/2]
      } else {
        bg.color <- colors[(length(colors)/2)+1]
      } 
    } else {
      if (length(colors) %% 2 == 1){
        colors[(length(colors)+1)/2] <- bg.color
      } else {
        n <- length(colors)/2
        colors <- c(colors[seq_len(n)], bg.color, 
          colors[(n+1):(n*2)])
      } 
    }
    cols <- colorRampPalette(colors)(17)
    bg <- cols[9]
    
  } else if(pars$ps$configs$scale.type=="neg") {
    
    if (is.na(bg.color)) {
      bg.color <- colors[length(colors)]
      colors <- colors[-length(colors)]
    }
    cols <- colorRampPalette(c(colors,bg.color))(16)
    bg <- cols[length(cols)]
    cols <- cols[-length(cols)]
    
  } else {
    
    if (is.na(bg.color)) {
      bg.color <- colors[1]
      colors <- colors[-1]
    }
    cols <- colorRampPalette(c(bg.color, colors))(16)
    bg <- cols[1]
    cols <- cols[-1]
    
  }
  
  bkIn <- seq(zlim[1], zlim[2], length.out = length(cols))
  bkOut <- pretty(zlim, n = 11)
  pal <- .trimRamp(bkIn, bkOut, cols)
  
  return(list(breaks = bkOut, pal = pal, bg = bg))
}
.trimRamp <- function(bkIn, bkOut, cols) {
  cols <- t(col2rgb(cols)/255)
  bkOut <- ifelse(bkOut < bkIn[1], bkIn[1], bkOut)
  bkOut <- ifelse(bkOut > bkIn[length(bkIn)], bkIn[length(bkIn)], bkOut)
  bnout <- .bincode(bkOut, bkIn, right = TRUE, include.lowest = TRUE)
  rcol <- lapply(unique(bnout), function(i){
    j <- bnout == i
    .getcolor(bkOut[j], bkIn[i], bkIn[i+1], cols[i, ], cols[i+1, ])
  })
  rcol <- unlist(rcol)
  return(rcol)
}
.getcolor <- function (x, bk1, bk2, c1, c2) {
  c1 <- grDevices::convertColor(c1, "sRGB", "Lab")
  c2 <- grDevices::convertColor(c2, "sRGB", "Lab")
  rcol <- matrix(ncol = 3, nrow = length(x))
  for (i in seq_len(3)) {
    xx <- (x - bk2) * (c2[i] - c1[i]) / (bk2 - bk1) + c2[i]
    rcol[, i] <- xx
  }
  rcol <- grDevices::convertColor(rcol, "Lab", "sRGB")
  .xlim <- function(x) {
    x[x < 0] <- 0; x[x > 1] <- 1
    return(x)
  }
  rcol[, ] <- .xlim(as.numeric(rcol))
  .rgb2hex <- function(r, g, b){
    grDevices::rgb(r, g, b, maxColorValue = 1)
  }
  rcol <- .rgb2hex(rcol[, 1], rcol[, 2], rcol[, 3])
  return(rcol)
}

#' @title Variation of the watershed algorithm for summit detection
#'
#' @description The \code{summitWatershed} function implements a segmentation
#' strategy to identify summits within a landscape image generated by the
#' PathwaySpace package. This function is entirely coded in R, which helps
#' alleviating users from the task of loading an excessive number of
#' dependencies. Nonetheless, while this novel implementation prevents
#' the burden a 'dependency heaviness', it still requires optimization
#' as it currently exhibits slower performance compared to well-established
#' implementations such as the watershed function from the EBImage package.
#' The \code{summitWatershed} maintain a certain level of compatibility
#' with the EBImage's watershed function, and both can be used in the
#' PathwaySpace package.
#'
#' @param x A 2D-numeric array in which each point represents the coordinates
#' of a signal in a landscape image.
#' @param tolerance The minimum signal intensity of a summit (in [0,1]),
#' representing a fraction of the maximum signal intensity.
#' @param ext Radius (in pixels) for detecting neighboring objects.
#' @return A matrix with labeled summits.
#' @author Sysbiolab Team, Mauro Castro.
#' @seealso \code{\link{summitMapping}}
#' @examples
#' # Load a demo landscape image
#' data('gimage', package = 'PathwaySpace')
#'
#' # Scale down the image for a quicker demonstration
#' gimage <- gimage[200:300, 200:300]
#' 
#' # Check signal range
#' range(gimage, na.rm = TRUE)
#' # [1] 0 1
#'
#' # Check image
#' \donttest{
#' image(gimage)
#' }
#'
#' # Threshold the signal intensity, for example:
#' gimage[gimage < 0.5] <- 0
#'
#' # Run summit segmentation
#' gmask <- summitWatershed(x = gimage)
#'
#' # Check resulting image mask
#' \donttest{
#' image(gimage)
#' }
#'
#' @aliases summitWatershed
#' @export
#'
summitWatershed <- function(x, tolerance = 0.1, ext = 1) {
    .validate.ps.args("numeric_mtx", "x", x)
    .validate.ps.args("singleNumber", "tolerance", tolerance)
    .validate.ps.args("singleInteger", "ext", ext)
    xmask <- .summitWatershed(x = x, ext = ext)
    xmask <- .mergeSummits(x = x, xmask, tolerance)
    return(xmask)
}
.summitWatershed <- function(x, ext = 1) {
    nr <- nrow(x)
    nc <- ncol(x)
    nrc <- max(nr, nc)
    x[x == 0] <- NA
    # make indices
    x.idx <- x
    x.idx[, ] <- seq_len(prod(dim(x)))
    c.idx <- arrayInd(seq_len(prod(dim(x))), dim(x), useNames = TRUE)
    c.idx <- cbind(c.idx, val = as.numeric(x), key = seq_len(nrow(c.idx)),
        label = NA)
    seeds <- c.idx[!is.na(c.idx[, "val"]), , drop = FALSE]
    seeds <- seeds[sort.list(seeds[, "val"], decreasing = TRUE), , 
        drop = FALSE]
    # make kernel masks
    r <- ext
    km1 <- .kernelMask(r)
    km1 <- t(which(km1 == 1, arr.ind = TRUE)) - (r + 1)
    a <- (pi * r^2) * max(1, nrc * 0.01)
    rr <- ceiling(sqrt(a / pi))
    km2 <- .kernelMask(rr)
    km2 <- t(which(km2 == 1, arr.ind = TRUE)) - (rr + 1)
    # start flooding
    lcount <- 1
    for (i in seq_len(nrow(seeds))) {
        lab <- c.idx[seeds[i, "key"], "label"]
        if (is.na(lab)) {
            si <- seeds[i, ]
            idx.si <- .getSidx(si, km2, nr, nc, c.idx, x.idx)
            lab <- idx.si[!is.na(idx.si[, "label"]), , drop = FALSE]
            if (nrow(lab) == 0) {
                lab <- lcount
                lcount <- lcount + 1
            } else if (nrow(lab) == 1) {
                lab <- lab[, "label"]
            } else {
                label <- lab[, "label"]
                lb <- table(label)[1] / length(label) > 0.5
                if (lb) {
                    lab <- .mode(label)
                } else {
                    lab <- label[which.max(lab[, "val"])]
                }
            }
            idx.si <- .getSidx(si, km1, nr, nc, c.idx, x.idx)
            c.idx[idx.si[, "key"], "label"] <- lab
        }
    }
    xmask <- x
    xmask[c.idx[, c("row", "col")]] <- c.idx[, "label"]
    xmask[is.na(xmask)] <- 0
    # keep masks of at least 3 pixels
    mz <- table(as.numeric(xmask))
    mz <- as.numeric(names(mz[mz < 3]))
    for (m in mz) {
        xmask[xmask == m] <- 0
    }
    xmask <- .relabel(xmask)
    xmask <- .fillCavity(xmask)
    # prune dams
    ml <- .findOutlines(xmask)
    dl <- .findDams(x, ml, xmask)$dl
    dl <- .dilatePxEdges(dl, .kernelMask(1))
    xmask[xmask == 0] <- NA
    xmask[dl > 0] <- 0
    xmask <- .expandPartitions(x, xmask)
    return(xmask)
}
.getSidx <- function(si, km, nr, nc, c.idx, x.idx, si.val = TRUE) {
    ix <- t(km + si[c(1, 2)])
    bl <- (ix[, 1] > 0 & ix[, 1] <= nr) & (ix[, 2] > 0 & ix[, 2] <= nc)
    ix <- ix[bl, , drop = FALSE]
    idx.si <- c.idx[x.idx[ix], , drop = FALSE]
    idx.si <- idx.si[!is.na(idx.si[, "val"]), , drop = FALSE]
    if (si.val) {
        bl <- (si["val"] - idx.si[, "val"]) <= 0
        idx.si <- idx.si[bl, , drop = FALSE]
    }
    idx.si
}
.mode <- function(c) {
    uc <- unique(c)
    ic <- match(c, uc)
    uc[which.max(tabulate(ic))]
}
#-------------------------------------------------------------------------------
.mergeSummits <- function(x, xmask, tolerance = 0.1) {
    if (tolerance < 0 || tolerance > 1) {
        stop("'tolerance' should be in [0,1]", call. = FALSE)
    }
    rsignal <- range(x[x > 0], na.rm = TRUE)
    ml <- .findOutlines(xmask)
    dams <- .findDams(x, ml, xmask)
    xmask <- .openDams(x, xmask, dams, ml, rsignal, tolerance)
    xmask <- .fillCavity(xmask)
    xmask <- .relabel(xmask)
    return(xmask)
}
.findDams <- function(x, ml, xmask) {
    mlabs <- sort(unique(as.numeric(xmask)))
    mlabs <- mlabs[mlabs > 0]
    xx <- mml <- dl <- array(0, dim = dim(x) + 2)
    nr <- nrow(xx)
    nc <- ncol(xx)
    mml[-c(1, nr), -c(1, nc)] <- ml
    xx[-c(1, nr), -c(1, nc)] <- x
    ij <- c(-1, 0, 1)
    km <- .kernelMask(1, shape = "square")
    km[2, 2] <- 0
    km <- which(km == 1, arr.ind = TRUE) - 2
    sts <- data.frame(neighbor.id = NA, n.signal = NA)[-1, ]
    dlstat <- lapply(mlabs, function(l) {
        sts
    })
    for (l in mlabs) {
        lp <- which(mml == l, arr.ind = TRUE)
        if (nrow(lp) > 0) {
            for (i in seq_len(nrow(lp))) {
                tp <- t(t(km) + lp[i, ])
                idx <- mml[tp]
                idx <- idx > 0 & idx != l
                if (any(idx)) {
                    tp <- tp[idx, , drop = FALSE]
                    dl[tp] <- mml[tp]
                    tpp <- data.frame(neighbor.id = mml[tp], n.signal = xx[tp])
                    dlstat[[l]] <- rbind(dlstat[[l]], tpp)
                }
            }
        }
    }
    dl <- dl[-c(1, nr), -c(1, nc)]
    dams <- list(dl = dl, dlstat = dlstat)
    return(dams)
}
.openDams <- function(x, xmask, dams, ml, rsignal, tolerance) {
    mlabs <- sort(unique(as.numeric(xmask)))
    mlabs <- mlabs[mlabs > 0]
    nlabs <- mlabs
    for (mid in mlabs) {
        if (mid %in% nlabs) {
            dlstat <- dams$dlstat[[mid]]
            if (nrow(dlstat) > 0) {
                neighbor.ids <- sort(unique(dlstat$neighbor.id))
                neighbor.ids <- neighbor.ids[neighbor.ids %in% nlabs]
                while (length(neighbor.ids) > 0) {
                    nid <- neighbor.ids[1]
                    dam.n <- dlstat[dlstat$neighbor.id == nid, ]
                    dam.signal <- max(dam.n$n.signal, na.rm = TRUE)
                    top.summit <- max(x[xmask == mid], na.rm = TRUE)
                    bot.summit <- max(x[xmask == nid], na.rm = TRUE)
                    dam.t1 <- (bot.summit - dam.signal) / 
                        (rsignal[2] - rsignal[1])
                    dam.t2 <- 1 - (dam.signal / bot.summit)
                    if (dam.t1 < tolerance || dam.t2 < tolerance) {
                        nlabs[nlabs == nid] <- mid
                        xmask[xmask == nid] <- mid
                        dlstat <- dlstat[dlstat$neighbor.id != nid, ]
                        dlstat <- rbind(dlstat, dams$dlstat[[nid]])
                        neighbor.ids <- sort(unique(dlstat$neighbor.id))
                        neighbor.ids <- neighbor.ids[neighbor.ids %in% nlabs]
                    } else {
                        neighbor.ids <- neighbor.ids[-1]
                    }
                }
            }
        }
    }
    xmask <- .relabel(xmask)
    return(xmask)
}
.expandPartitions <- function(x, xmask) {
    nr <- nrow(x)
    nc <- ncol(x)
    seeds <- xmask
    ol <- .findOutlines(seeds)
    seeds[seeds != ol & seeds > 0] <- NA
    # make indices with seeds
    x.idx <- x
    x.idx[, ] <- seq_len(prod(dim(x)))
    c.idx <- arrayInd(seq_len(prod(dim(x))), dim(x), useNames = TRUE)
    c.idx <- cbind(c.idx, val = as.numeric(x), key = seq_len(nrow(c.idx)),
        label = NA)
    c.idx <- cbind(c.idx, seed = as.numeric(seeds))
    c.idx[is.na(c.idx[, "seed"]), "val"] <- NA
    c.idx[, "label"] <- c.idx[, "seed"]
    # sort seeds by signal intensity
    seeds <- c.idx[!is.na(c.idx[, "val"]), , drop = FALSE]
    if (nrow(seeds) == 0) {
        return(xmask)
    }
    ixd <- order(seeds[, "val"], seeds[, "seed"], decreasing = TRUE)
    seeds <- seeds[ixd, , drop = FALSE]
    seeds <- cbind(seeds, keyseed = seq_len(nrow(seeds)), count = 1)
    seeds[seeds[, "seed"] == 0, "count"] <- 0
    c.idx <- cbind(c.idx, keyseed = NA)
    c.idx[seeds[, "key"], "keyseed"] <- seeds[, "keyseed"]
    km <- .kernelMask(1, "square")
    km <- t(which(km == 1, arr.ind = TRUE)) - 2
    # start expanding seeds
    while (sum(seeds[, "count"]) > 0) {
        si <- seeds[match(1, seeds[, "count"]), ]
        idx.si <- .getSidx(si, km, nr, nc, c.idx, x.idx, si.val = FALSE)
        idx.si <- idx.si[idx.si[, "label"] == 0, , drop = FALSE]
        if (nrow(idx.si) > 0) {
            c.idx[idx.si[, "key"], "label"] <- si["label"]
            seeds[idx.si[, "keyseed"], "label"] <- si["label"]
            seeds[idx.si[, "keyseed"], "count"] <- 1
        }
        seeds[si["keyseed"], "count"] <- 0
    }
    c.idx <- c.idx[!is.na(c.idx[, "label"]), , drop = FALSE]
    xmask[c.idx[, c("row", "col")]] <- c.idx[, "label"]
    xmask[is.na(xmask)] <- 0
    # keep masks of at least 3 pixels
    mz <- table(as.numeric(xmask))
    mz <- as.numeric(names(mz[mz < 3]))
    for (m in mz) { xmask[xmask == m] <- 0 }
    xmask <- .relabel(xmask)
    return(xmask)
}

---
title: "Getting started with *PathwaySpace* projection methods"
author: "Sysbiolab Team"
date: "`r Sys.Date()`"
bibliography: bibliography.bib
output: 
  html_document:
    theme: cerulean
    self_contained: yes
    toc: true
    toc_float: true
    toc_depth: 2
    css: custom.css
vignette: >
  %\VignetteIndexEntry{"PathwaySpace: signals along geodesic paths"}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE, purl=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

**Package**: PathwaySpace `r packageVersion('PathwaySpace')`

# Highlights
* Produces landscape images representing graphs by geodesic paths
* Projects signals using a decay function to model signal attenuation
* Applies a convolution algorithm to combine signals from neighboring vertices

# Overview

For a given *igraph* object containing vertices, edges, and a signal associated with the vertices, *PathwaySpace* performs a convolution operation, which involves a weighted combination of neighboring signals on a graph. **Figure 1A** illustrates the convolution operation problem. Each vertex's signal is positioned on a grid at specific `x` and `y` coordinates, represented by cones (for available signals) or question marks (for null or missing values).

```{r fig1, echo=FALSE, fig.cap="**Figure 1.** Signal processing addressed by the *PathwaySpace* package. **A**) Graph overlaid on a 2D coordinate system. Each projection cone represents the signal associated with a graph vertex (referred to as *vertex-signal positions*), while question marks indicate positions with no signal information (referred to as *null-signal positions*). **Inset**: Graph layout of the toy example used in this vignette. **B**) Illustration of signal projection from two neighboring vertices, simplified to one dimension. **Right**: Signal profiles from aggregation and decay functions.", out.width = '100%', purl=FALSE}
knitr::include_graphics("figures/fig1.png")
```

<br/>

*PathwaySpace* model considers the vertex-signal positions as source points (or transmitters) and the null-signal positions as end points (or receivers). The signal values from vertex-signal positions are then projected to the null-signal positions according to a decay function, which will control how the signal values attenuate as they propagate across the 2D space. For a given null-signal position, the k-top signals are used to define the contributing vertices for the convolution operation, which will aggregate the signals from these contributing vertices considering their intensities reaching the end points. Users can adjust both the aggregation and decay functions; the aggregation function can be any arithmetic rule that reduces a numeric vector into a single scalar value (*e.g.*, mean, weighted mean), while available decay functions include linear, exponential, and Weibull models (**Fig.1B**). Additionally, users can assign vertex-specific decay functions to model signal projections for subsets of vertices that may exhibit distinct behaviors. The resulting image forms geodesic paths in which the signal has been projected from vertex- to null-signal positions, using a density metric to measure the signal intensity along these paths.

# Setting basic input data

```{r Load packages - quick start, eval=TRUE, message=FALSE}
#--- Load required packages for this section
library(igraph)
library(ggplot2)
library(RGraphSpace)
library(PathwaySpace)
```

This section will create an *igraph* object containing a binary signal associated to each vertex. The graph layout is configured manually to ensure that users can easily view all the relevant arguments needed to prepare the input data for the *PathwaySpace* package. The *igraph*'s `make_star()` function creates a star-like graph and the `V()` function is used to set attributes for the vertices. The *PathwaySpace* package will require that all vertices have `x`, `y`, and `name` attributes.

```{r Making a toy igraph - 1, eval=TRUE, message=FALSE}
# Make a 'toy' igraph object, either a directed or undirected graph
gtoy1 <- make_star(5, mode="undirected")

# Assign 'x' and 'y' coordinates to each vertex
# ..this can be an arbitrary unit in (-Inf, +Inf)
V(gtoy1)$x <- c(0, 2, -2, -4, -8)
V(gtoy1)$y <- c(0, 0,  2, -4,  0)

# Assign a 'name' to each vertex (here, from n1 to n5)
V(gtoy1)$name <- paste0("n", 1:5)
```

# Checking graph validity

Next, we will create a *GraphSpace-class* object using the `GraphSpace()` constructor, followed by a call to `normalizeGraphSpace()`. The constructor ensures the validity of the input *igraph* object, while `normalizeGraphSpace()` scales the network coordinates. In this example, we set `mar = 0.2` to define the graph's outer margins.

```{r GraphSpace constructor - 1, eval=TRUE, message=FALSE}
# Check graph validity
gs1 <- GraphSpace(gtoy1)
# Normalize node coordinates 
gs1 <- normalizeGraphSpace(gs1, mar = 0.2)
```

Our graph is now ready for the *PathwaySpace* package. We can check its layout using the `plotGraphSpace()` function. 

```{r GraphSpace constructor - 2, eval=FALSE, message=FALSE, out.width="100%"}
# Check the graph layout
plotGraphSpace(gs1, add.labels = TRUE)
```

```{r fig2.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotGraphSpace(gs1, add.labels = TRUE)
# ggsave(filename = "./figures/fig2.png", height=4, width=5,
#   units="in", device="png", dpi=250, plot=gg)
```

```{r fig2, echo=FALSE, out.width = '70%', purl=FALSE}
knitr::include_graphics("figures/fig2.png")
```


# Creating a *PathwaySpace* object

Next, we will create a *PathwaySpace-class* object using the `buildPathwaySpace()` constructor. This will calculate pairwise distances between vertices, subsequently required by the signal projection methods.

```{r PathwaySpace constructor - 1, eval=TRUE, message=FALSE}
# Run the PathwaySpace constructor
p_space1 <- buildPathwaySpace(gs1)
```

As a default behavior, the `buildPathwaySpace()` constructor initializes the signal of each vertex as `0`. We can use the `vertexSignal()` accessor to get and set vertex signals in a *PathwaySpace* object; for example, in order to get vertex names and signal values:

```{r PathwaySpace constructor - 2, eval=TRUE, message=FALSE, results='hide'}
# Check the number of vertices in the PathwaySpace object
length(p_space1)
## [1] 5

# Check vertex names
names(p_space1)
## [1] "n1" "n2" "n3" "n4" "n5"

# Check signal (initialized with '0')
vertexSignal(p_space1)
## n1 n2 n3 n4 n5 
##  0  0  0  0  0
```

...and for setting new signal values in *PathwaySpace* objects:

```{r PathwaySpace constructor - 3, eval=TRUE, message=FALSE, results='hide'}
# Set new signal to all vertices
vertexSignal(p_space1) <- c(1, 4, 2, 4, 3)

# Set a new signal to the 1st vertex
vertexSignal(p_space1)[1] <- 2

# Set a new signal to vertex "n1"
vertexSignal(p_space1)["n1"] <- 6

# Check updated signal values
vertexSignal(p_space1)
## n1 n2 n3 n4 n5 
##  6  4  2  4  3
```

# Signal projection

## Circular projection

Following that, we will use the `circularProjection()` function to project the network signals by the `weibullDecay()` function with `pdist = 0.4`, which is passed by the `decay.fun` argument. This term determines a distance unit for the signal convolution, affecting the extent over which the convolution operation projects the signal. For example, when `pdist = 1`, it will represent the diameter of the inscribed circle within the coordinate space. We also set `k = 1`, which defines the contributing vertices for signal convolution.

```{r Circular projection - 1, eval=FALSE, message=FALSE, out.width="70%"}
# Run signal projection
p_space1 <- circularProjection(p_space1, k = 1, 
  decay.fun = weibullDecay(pdist = 0.4))

# Plot a PathwaySpace image
plotPathwaySpace(p_space1, add.marks = TRUE)
```

```{r fig3.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotPathwaySpace(p_space1, add.marks = TRUE)
# ggsave(filename = "./figures/fig3.png", height=3.5, width=5,
#   units="in", device="png", dpi=350, plot=gg)
```

```{r fig3, echo=FALSE, out.width = '75%', purl=FALSE}
knitr::include_graphics("figures/fig3.png")
```

Next, we reassess the same *PathwaySpace* object, using `pdist = 0.2`, `k = 2` and adjusting the `shape` of the decay function (for further details, see the [**online tutorials**](https://sysbiolab.github.io/PathwaySpace/)).

```{r Circular projection - 3, eval=FALSE, message=FALSE, out.width="70%"}
# Re-run signal projection, adjusting Weibull's shape
p_space1 <- circularProjection(p_space1, k = 2, 
  decay.fun = weibullDecay(shape = 2, pdist = 0.2))

# Plot PathwaySpace
plotPathwaySpace(p_space1, marks = "n1", theme = "th2")
```

```{r fig4.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotPathwaySpace(p_space1, marks = "n1", theme = "th2")
# ggsave(filename = "./figures/fig4.png", height=3.5, width=5,
#   units="in", device="png", dpi=350, plot=gg)
```

```{r fig4, echo=FALSE, out.width = '75%', purl=FALSE}
knitr::include_graphics("figures/fig4.png")
```

The `shape` parameter allows a projection to take a variety of shapes. When `shape = 1` the projection follows an exponential decay, and when `shape > 1` the projection is first convex, then concave with an inflection point along the decay path. For additional examples see the [**modeling signal decay**](https://sysbiolab.github.io/PathwaySpace/articles/modeling-signal-decay.html) tutorial.

## Polar projection

In this section we will project network signals using a polar coordinate system. This representation may be useful for certain types of data, for example, to highlight patterns of signal propagation on directed graphs, especially to explore the orientation aspect of signal flow. To demonstrate this feature we will used the `gtoy2` directed graph, available in the *RGraphSpace* package.

```{r Polar projection - 1, eval=TRUE, message=FALSE, out.width="100%"}
# Load a pre-processed directed igraph object
data("gtoy2", package = "RGraphSpace")
# Check graph validity
gs2 <- GraphSpace(gtoy2)
gs2 <- normalizeGraphSpace(gs2, mar = 0.2)
```

```{r Polar projection - 2, eval=FALSE, message=FALSE, out.width="100%"}
# Check the graph layout
plotGraphSpace(gs2, add.labels = TRUE)
```

```{r fig5.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotGraphSpace(gs2, add.labels = TRUE)
# ggsave(filename = "./figures/fig5.png", height=4, width=5,
#   units="in", device="png", dpi=250, plot=gg)
```

```{r fig5, echo=FALSE, out.width = '75%', purl=FALSE}
knitr::include_graphics("figures/fig5.png")
```

```{r Polar projection - 3, eval=TRUE, message=FALSE}
# Build a PathwaySpace for the 'gs2'
p_space2 <- buildPathwaySpace(gs2)

# Set '1s' as vertex signal
vertexSignal(p_space2) <- 1
```

For fine-grained modeling of signal decay, the `vertexDecay()` accessor allows assigning decay functions at the level of individual vertices. For example, adjusting Weibull's `shape` argument for node `n6`:

```{r Polar projection - 4, eval=TRUE, message=FALSE}
# Modify decay function
# ..for all vertices
vertexDecay(p_space2) <- weibullDecay(shape=2, pdist = 1)
# ..for individual vertices
vertexDecay(p_space2)[["n6"]] <- weibullDecay(shape=3, pdist = 1)
```

In polar projections, the `pdist` term defines a reference distance related to edge length, aiming to constrain signal projections within edge bounds. Here we set `pdist = 1` to reach full edge lengths. Next, we run the signal projection using polar coordinates. The `beta` exponent will control the angular span; for values greater than zero, `beta` will progressively narrow the projection along the edge axis.

```{r Polar projection - 5, eval=FALSE, message=FALSE, out.width="70%"}
# Run signal projection using polar coordinates
p_space2 <- polarProjection(p_space2, beta = 10)

# Plot PathwaySpace
plotPathwaySpace(p_space2, theme = "th2", add.marks = TRUE)
```

```{r fig6.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotPathwaySpace(p_space2, theme = "th2", add.marks = TRUE)
# ggsave(filename = "./figures/fig6.png", height=3.5, width=5,
#   units="in", device="png", dpi=350, plot=gg)
```

```{r fig6, echo=FALSE, out.width = '75%', purl=FALSE}
knitr::include_graphics("figures/fig6.png")
```

Note that this projection distributes signals on the edges regardless of direction. To incorporate edge orientation, we set `directional = TRUE`, which channels the projection along the paths:

```{r Polar projection - 6, eval=FALSE, message=FALSE, out.width="70%"}
# Re-run signal projection using 'directional = TRUE'
p_space2 <- polarProjection(p_space2, 
  beta = 10, directional = TRUE)

# Plot PathwaySpace
plotPathwaySpace(p_space2, theme = "th2", 
  marks = c("n1","n3","n4","n5"))
```

```{r fig7.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotPathwaySpace(p_space2, theme = "th2", marks = c("n1","n3","n4","n5"))
# ggsave(filename = "./figures/fig7.png", height=3.5, width=5,
#   units="in", device="png", dpi=350, plot=gg)
```

```{r fig7, echo=FALSE, out.width = '75%', purl=FALSE}
knitr::include_graphics("figures/fig7.png")
```

This *PathwaySpace* polar projection emphasizes the signal flow along the directional pattern of a directed graph (see the *igraph* plot above). When interpreting, users should note that this approach introduces simplifications; for example, depending on the network topology, the polar projection may fail to capture complex features of directed graphs, such as cyclic dependencies, feedforward and feedback loops, or other intricate interactions.

# Signal types

The *PathwaySpace* accepts binary, integer, and numeric signal types, including `NAs`. If a vertex signal is assigned with `NA`, it will be ignored by the convolution algorithm. Logical values are also allowed, but it will be treated as binary. Next, we show the projection of a signal that includes negative values, using the `p_space1` object created previously.

```{r Signal types, eval=FALSE, message=FALSE, out.width="70%"}
# Set a negative signal to vertices "n3" and "n4"
vertexSignal(p_space1)[c("n3","n4")] <- c(-2, -4)

# Check updated signal vector
vertexSignal(p_space1)
# n1 n2 n3 n4 n5 
#  6  4 -2 -4  3 

# Re-run signal projection
p_space1 <- circularProjection(p_space1, 
  decay.fun = weibullDecay(shape = 2))

# Plot PathwaySpace
plotPathwaySpace(p_space1, bg.color = "white", 
  font.color = "grey20", add.marks = TRUE, 
  mark.color = "magenta", theme = "th3")
```

```{r fig8.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotPathwaySpace(p_space1, bg.color = "white", font.color = "grey20", add.marks = TRUE, mark.color = "magenta", theme = "th3")
# ggsave(filename = "./figures/fig8.png", height=3.5, width=5,
#   units="in", device="png", dpi=350, plot=gg)
```

```{r fig8, echo=FALSE, out.width = '75%', purl=FALSE}
knitr::include_graphics("figures/fig8.png")
```

Note that the original signal vector was rescale to `[-1, +1]`. If the signal vector is `>=0`, then it will be rescaled to `[0, 1]`; if the signal vector is `<=0`, it will be rescaled to `[-1, 0]`; and if the signal vector is in `(-Inf, +Inf)`, then it will be rescaled to `[-1, +1]`. To override this signal processing, simply set `rescale = FALSE` in the projection function.

# Online tutorials

https://sysbiolab.github.io/PathwaySpace/

# Citation

If you use *PathwaySpace*, please cite:

* Tercan & Apolonio et al. Protocol for assessing distances in pathway space for classifier feature sets from machine learning methods. *STAR Protocols* 6(2):103681, 2025. https://doi.org/10.1016/j.xpro.2025.103681

* Ellrott et al. Classification of non-TCGA cancer samples to TCGA molecular subtypes using compact feature sets. *Cancer Cell* 43(2):195-212.e11, 2025. https://doi.org/10.1016/j.ccell.2024.12.002


# Other useful links

* RGraphSpace: A Lightweight Interface Between 'ggplot2' and 'igraph' Objects
https://cran.r-project.org/package=RGraphSpace

* RedeR: Interactive visualization and manipulation of nested networks
https://bioconductor.org/packages/RedeR/


# Session information
```{r label='Session information', eval=TRUE, echo=FALSE}
sessionInfo()
```

---
title: "Getting started with *PathwaySpace* projection methods"
author: "Sysbiolab Team"
date: "`r Sys.Date()`"
bibliography: bibliography.bib
output: 
  html_document:
    theme: cerulean
    self_contained: yes
    toc: true
    toc_float: true
    toc_depth: 2
    css: custom.css
vignette: >
  %\VignetteIndexEntry{"PathwaySpace: signals along geodesic paths"}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE, purl=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

**Package**: PathwaySpace `r packageVersion('PathwaySpace')`

# Highlights
* Produces landscape images representing graphs by geodesic paths
* Projects signals using a decay function to model signal attenuation
* Applies a convolution algorithm to combine signals from neighboring vertices

# Overview

For a given *igraph* object containing vertices, edges, and a signal associated with the vertices, *PathwaySpace* performs a convolution operation, which involves a weighted combination of neighboring signals on a graph. **Figure 1A** illustrates the convolution operation problem. Each vertex's signal is positioned on a grid at specific `x` and `y` coordinates, represented by cones (for available signals) or question marks (for null or missing values).

```{r fig1, echo=FALSE, fig.cap="**Figure 1.** Signal processing addressed by the *PathwaySpace* package. **A**) Graph overlaid on a 2D coordinate system. Each projection cone represents the signal associated with a graph vertex (referred to as *vertex-signal positions*), while question marks indicate positions with no signal information (referred to as *null-signal positions*). **Inset**: Graph layout of the toy example used in the *quick start* section of this vignette. **B**) Illustration of signal projection from two neighboring vertices, simplified to one dimension. **Right**: Signal profiles from aggregation and decay functions.", out.width = '100%', purl=FALSE}
knitr::include_graphics("figures/fig1.png")
```

<br/>

*PathwaySpace* model considers the vertex-signal positions as source points (or transmitters) and the null-signal positions as end points (or receivers). The signal values from vertex-signal positions are then projected to the null-signal positions according to a decay function, which will control how the signal values attenuate as they propagate across the 2D space. For a given null-signal position, the k-top signals are used to define the contributing vertices for the convolution operation, which will aggregate the signals from these contributing vertices considering their intensities reaching the end points. Users can adjust both the aggregation and decay functions; the aggregation function can be any arithmetic rule that reduces a numeric vector into a single scalar value (*e.g.*, mean, weighted mean), while available decay functions include linear, exponential, and Weibull models (**Fig.1B**). Additionally, users can assign vertex-specific decay functions to model signal projections for subsets of vertices that may exhibit distinct behaviors. The resulting image forms geodesic paths in which the signal has been projected from vertex- to null-signal positions, using a density metric to measure the signal intensity along these paths.

# Setting basic input data

```{r Load packages - quick start, eval=TRUE, message=FALSE}
#--- Load required packages for this section
library(igraph)
library(ggplot2)
library(RGraphSpace)
library(PathwaySpace)
```

This section will create an *igraph* object containing a binary signal associated to each vertex. The graph layout is configured manually to ensure that users can easily view all the relevant arguments needed to prepare the input data for the *PathwaySpace* package. The *igraph*'s `make_star()` function creates a star-like graph and the `V()` function is used to set attributes for the vertices. The *PathwaySpace* package will require that all vertices have `x`, `y`, and `name` attributes.

```{r Making a toy igraph - 1, eval=TRUE, message=FALSE}
# Make a 'toy' igraph object, either a directed or undirected graph
gtoy1 <- make_star(5, mode="undirected")

# Assign 'x' and 'y' coordinates to each vertex
# ..this can be an arbitrary unit in (-Inf, +Inf)
V(gtoy1)$x <- c(0, 2, -2, -4, -8)
V(gtoy1)$y <- c(0, 0,  2, -4,  0)

# Assign a 'name' to each vertex (here, from n1 to n5)
V(gtoy1)$name <- paste0("n", 1:5)
```

# Checking graph validity

Next, we will create a *GraphSpace-class* object using the `GraphSpace()` constructor, followed by a call to `normalizeGraphSpace()`. The constructor ensures the validity of the input *igraph* object, while `normalizeGraphSpace()` scales the network coordinates. In this example, we set `mar = 0.2` to define the graph's outer margins.

```{r GraphSpace constructor - 1, eval=TRUE, message=FALSE}
# Check graph validity
gs1 <- GraphSpace(gtoy1)
# Normalize node coordinates 
gs1 <- normalizeGraphSpace(gs1)
```

Our graph is now ready for the *PathwaySpace* package. We can check its layout using the `plotGraphSpace()` function. 

```{r GraphSpace constructor - 2, eval=FALSE, message=FALSE, out.width="100%"}
# Check the graph layout
plotGraphSpace(gs1, add.labels = TRUE)
```

```{r fig2.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotGraphSpace(gs1, add.labels = TRUE)
# ggsave(filename = "./figures/fig2.png", height=4, width=5,
#   units="in", device="png", dpi=250, plot=gg)
```

```{r fig2, echo=FALSE, out.width = '70%', purl=FALSE}
knitr::include_graphics("figures/fig2.png")
```


# Creating a *PathwaySpace* object

Next, we will create a *PathwaySpace-class* object using the `buildPathwaySpace()` constructor. This will calculate pairwise distances between vertices, subsequently required by the signal projection methods.

```{r PathwaySpace constructor - 1, eval=TRUE, message=FALSE}
# Run the PathwaySpace constructor
p_space1 <- buildPathwaySpace(gs1)
```

As a default behavior, the `buildPathwaySpace()` constructor initializes the signal of each vertex as `0`. We can use the `vertexSignal()` accessor to get and set vertex signals in a *PathwaySpace* object; for example, in order to get vertex names and signal values:

```{r PathwaySpace constructor - 2, eval=TRUE, message=FALSE, results='hide'}
# Check the number of vertices in the PathwaySpace object
length(p_space1)
## [1] 5

# Check vertex names
names(p_space1)
## [1] "n1" "n2" "n3" "n4" "n5"

# Check signal (initialized with '0')
vertexSignal(p_space1)
## n1 n2 n3 n4 n5 
##  0  0  0  0  0
```

...and for setting new signal values in *PathwaySpace* objects:

```{r PathwaySpace constructor - 3, eval=TRUE, message=FALSE, results='hide'}
# Set new signal to all vertices
vertexSignal(p_space1) <- c(1, 4, 2, 4, 3)

# Set a new signal to the 1st vertex
vertexSignal(p_space1)[1] <- 2

# Set a new signal to vertex "n1"
vertexSignal(p_space1)["n1"] <- 6

# Check updated signal values
vertexSignal(p_space1)
## n1 n2 n3 n4 n5 
##  6  4  2  4  3
```

# Signal projection

## Circular projection

Following that, we will use the `circularProjection()` function to project the network signals by the `weibullDecay()` function with `pdist = 0.4`, which is passed by the `decay.fun` argument. This term determines a distance unit for the signal convolution, affecting the extent over which the convolution operation projects the signal. For example, when `pdist = 1`, it will represent the diameter of the inscribed circle within the coordinate space. We also set `k = 1`, which defines the contributing vertices for signal convolution.

```{r Circular projection - 1, eval=FALSE, message=FALSE, out.width="70%"}
# Run signal projection
p_space1 <- circularProjection(p_space1, k = 1, 
  decay.fun = weibullDecay(pdist = 0.4))

# Plot a PathwaySpace image
plotPathwaySpace(p_space1, add.marks = TRUE)
```

```{r fig3.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotPathwaySpace(p_space1, add.marks = TRUE)
# ggsave(filename = "./figures/fig3.png", height=3.5, width=5,
#   units="in", device="png", dpi=350, plot=gg)
```

```{r fig3, echo=FALSE, out.width = '75%', purl=FALSE}
knitr::include_graphics("figures/fig3.png")
```

Next, we reassess the same *PathwaySpace* object, using `pdist = 0.2`, `k = 2` and adjusting the `shape` of the decay function (for further details, see the [**online tutorials**](https://sysbiolab.github.io/PathwaySpace/)).

```{r Circular projection - 3, eval=FALSE, message=FALSE, out.width="70%"}
# Re-run signal projection, adjusting Weibull's shape
p_space1 <- circularProjection(p_space1, k = 2, 
  decay.fun = weibullDecay(shape = 2, pdist = 0.2))

# Plot PathwaySpace
plotPathwaySpace(p_space1, marks = "n1", theme = "th2")
```

```{r fig4.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotPathwaySpace(p_space1, marks = "n1", theme = "th2")
# ggsave(filename = "./figures/fig4.png", height=3.5, width=5,
#   units="in", device="png", dpi=350, plot=gg)
```

```{r fig4, echo=FALSE, out.width = '75%', purl=FALSE}
knitr::include_graphics("figures/fig4.png")
```

The `shape` parameter allows a projection to take a variety of shapes. When `shape = 1` the projection follows an exponential decay, and when `shape > 1` the projection is first convex, then concave with an inflection point along the decay path. For additional examples see the [**modeling signal decay**](https://sysbiolab.github.io/PathwaySpace/modeling-signal-decay.html) tutorial.

## Polar projection

In this section we will project network signals using a polar coordinate system. This representation may be useful for certain types of data, for example, to highlight patterns of signal propagation on directed graphs, especially to explore the orientation aspect of signal flow. To demonstrate this feature we will used the `gtoy2` directed graph, available in the *RGraphSpace* package.

```{r Polar projection - 1, eval=TRUE, message=FALSE, out.width="100%"}
# Load a pre-processed directed igraph object
data("gtoy2", package = "RGraphSpace")
# Check graph validity
gs2 <- GraphSpace(gtoy2)
gs2 <- normalizeGraphSpace(gs2, mar = 0.2)
```

```{r Polar projection - 2, eval=FALSE, message=FALSE, out.width="100%"}
# Check the graph layout
plotGraphSpace(gs2, add.labels = TRUE)
```

```{r fig5.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotGraphSpace(gs2, add.labels = TRUE)
# ggsave(filename = "./figures/fig5.png", height=4, width=5,
#   units="in", device="png", dpi=250, plot=gg)
```

```{r fig5, echo=FALSE, out.width = '75%', purl=FALSE}
knitr::include_graphics("figures/fig5.png")
```

```{r Polar projection - 3, eval=TRUE, message=FALSE}
# Build a PathwaySpace for the 'gs2'
p_space2 <- buildPathwaySpace(gs2)

# Set '1s' as vertex signal
vertexSignal(p_space2) <- 1
```

For fine-grained modeling of signal decay, the `vertexDecay()` accessor allows assigning decay functions at the level of individual vertices. For example, adjusting Weibull's `shape` argument for node `n6`:

```{r Polar projection - 4, eval=TRUE, message=FALSE}
# Modify decay function
# ..for all vertices
vertexDecay(p_space2) <- weibullDecay(shape=2, pdist = 1)
# ..for individual vertices
vertexDecay(p_space2)[["n6"]] <- weibullDecay(shape=3, pdist = 1)
```

In polar projections, the `pdist` term defines a reference distance related to edge length, aiming to constrain signal projections within edge bounds. Here we set `pdist = 1` to reach full edge lengths. Next, we run the signal projection using polar coordinates. The `beta` exponent will control the angular span; for values greater than zero, `beta` will progressively narrow the projection along the edge axis.

```{r Polar projection - 5, eval=FALSE, message=FALSE, out.width="70%"}
# Run signal projection using polar coordinates
p_space2 <- polarProjection(p_space2, beta = 10)

# Plot PathwaySpace
plotPathwaySpace(p_space2, theme = "th2", add.marks = TRUE)
```

```{r fig6.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotPathwaySpace(p_space2, theme = "th2", add.marks = TRUE)
# ggsave(filename = "./figures/fig6.png", height=3.5, width=5,
#   units="in", device="png", dpi=350, plot=gg)
```

```{r fig6, echo=FALSE, out.width = '75%', purl=FALSE}
knitr::include_graphics("figures/fig6.png")
```

Note that this projection distributes signals on the edges regardless of direction. To incorporate edge orientation, we set `directional = TRUE`, which channels the projection along the paths:

```{r Polar projection - 6, eval=FALSE, message=FALSE, out.width="70%"}
# Re-run signal projection using 'directional = TRUE'
p_space2 <- polarProjection(p_space2, 
  beta = 10, directional = TRUE)

# Plot PathwaySpace
plotPathwaySpace(p_space2, theme = "th2", 
  marks = c("n1","n3","n4","n5"))
```

```{r fig7.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotPathwaySpace(p_space2, theme = "th2", marks = c("n1","n3","n4","n5"))
# ggsave(filename = "./figures/fig7.png", height=3.5, width=5,
#   units="in", device="png", dpi=350, plot=gg)
```

```{r fig7, echo=FALSE, out.width = '75%', purl=FALSE}
knitr::include_graphics("figures/fig7.png")
```

This *PathwaySpace* polar projection emphasizes the signal flow along the directional pattern of a directed graph (see the *igraph* plot above). When interpreting, users should note that this approach introduces simplifications; for example, depending on the network topology, the polar projection may fail to capture complex features of directed graphs, such as cyclic dependencies, feedforward and feedback loops, or other intricate interactions.

# Signal types

The *PathwaySpace* accepts binary, integer, and numeric signal types, including `NAs`. If a vertex signal is assigned with `NA`, it will be ignored by the convolution algorithm. Logical values are also allowed, but it will be treated as binary. Next, we show the projection of a signal that includes negative values, using the `p_space1` object created previously.

```{r Signal types, eval=FALSE, message=FALSE, out.width="70%"}
# Set a negative signal to vertices "n3" and "n4"
vertexSignal(p_space1)[c("n3","n4")] <- c(-2, -4)

# Check updated signal vector
vertexSignal(p_space1)
# n1 n2 n3 n4 n5 
#  6  4 -2 -4  3 

# Re-run signal projection
p_space1 <- circularProjection(p_space1, 
  decay.fun = weibullDecay(shape = 2))

# Plot PathwaySpace
plotPathwaySpace(p_space1, bg.color = "white", 
  font.color = "grey20", add.marks = TRUE, 
  mark.color = "magenta", theme = "th3")
```

```{r fig8.png, eval=FALSE, message=FALSE, echo=FALSE, include=FALSE, purl=FALSE}
# gg <- plotPathwaySpace(p_space1, bg.color = "white", font.color = "grey20", add.marks = TRUE, mark.color = "magenta", theme = "th3")
# ggsave(filename = "./figures/fig8.png", height=3.5, width=5,
#   units="in", device="png", dpi=350, plot=gg)
```

```{r fig8, echo=FALSE, out.width = '75%', purl=FALSE}
knitr::include_graphics("figures/fig8.png")
```

Note that the original signal vector was rescale to `[-1, +1]`. If the signal vector is `>=0`, then it will be rescaled to `[0, 1]`; if the signal vector is `<=0`, it will be rescaled to `[-1, 0]`; and if the signal vector is in `(-Inf, +Inf)`, then it will be rescaled to `[-1, +1]`. To override this signal processing, simply set `rescale = FALSE` in the projection function.

# Online tutorials

https://sysbiolab.github.io/PathwaySpace/

# Citation

If you use *PathwaySpace*, please cite:

* Tercan & Apolonio et al. Protocol for assessing distances in pathway space for classifier feature sets from machine learning methods. *STAR Protocols* 6(2):103681, 2025. https://doi.org/10.1016/j.xpro.2025.103681

* Ellrott et al. Classification of non-TCGA cancer samples to TCGA molecular subtypes using compact feature sets. *Cancer Cell* 43(2):195-212.e11, 2025. https://doi.org/10.1016/j.ccell.2024.12.002


# Other useful links

* RGraphSpace: A Lightweight Interface Between 'ggplot2' and 'igraph' Objects
https://cran.r-project.org/package=RGraphSpace

* RedeR: Interactive visualization and manipulation of nested networks
https://bioconductor.org/packages/RedeR/


# Session information
```{r label='Session information', eval=TRUE, echo=FALSE}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMethods.R
\docType{methods}
\name{polarProjection,PathwaySpace-method}
\alias{polarProjection,PathwaySpace-method}
\alias{polarProjection}
\title{Polar Projection of Graph-Associated Signals}
\usage{
\S4method{polarProjection}{PathwaySpace}(
  ps,
  k = 2,
  beta = 10,
  decay.fun = weibullDecay(pdist = 1),
  aggregate.fun = signalAggregation(),
  polar.fun = polarDecay(),
  directional = FALSE,
  edge.norm = TRUE,
  rescale = TRUE,
  verbose = TRUE,
  theta = deprecated(),
  pdist = deprecated()
)
}
\arguments{
\item{ps}{A \linkS4class{PathwaySpace} class object.}

\item{k}{A single positive integer determining the k-top signals for the 
convolution operation.}

\item{beta}{An exponent (in \code{[0, +Inf)}) used in the polar 
projection functions (see \code{\link{polarDecay}}). It controls the  
shape of the polar projection by modulating the angular span.
For example, \eqn{beta = 0} yields a circular projection, \eqn{beta = 1} 
produces a cardioid-like shape, and \code{beta > 1} progressively narrows 
the projection along a reference edge axis.}

\item{decay.fun}{A signal decay function. Available options include 
'Weibull', 'exponential', and 'linear' (see \code{\link{weibullDecay}}).
Users may also define a custom decay model with at least two arguments, 
e.g., \code{function(x, signal) \{ ... \}}, which should returns a vector of  
projected signals of the same length as \code{x}. Additional arguments may  
include any variable available as a graph vertex attribute.}

\item{aggregate.fun}{A function used to aggregate the projected signals. 
It must be provided as a unary function, e.g., \code{function(x) { ... }}, 
which should aggregate a vector of signals to a scalar value. 
Available options include 'mean', 'wmean', 'log.wmean', and 'exp.wmean' 
(See \code{\link{signalAggregation}}).}

\item{polar.fun}{A polar decay function (see \code{\link{polarDecay}}).}

\item{directional}{If directional edges are available, this argument can 
be used to orientate the signal projection on directed graphs.}

\item{edge.norm}{Scale distances based on edge lengths 
(when \code{edge.norm=TRUE}) or based on full coordinate space 
(when \code{edge.norm=FALSE}).}

\item{rescale}{A logical value indicating whether to rescale 
the signal. If the signal \code{>=0}, then it will be rescaled to 
\code{[0, 1]}; if the signal \code{<=0}, then it will be rescaled to 
\code{[-1, 0]}; and if the signal in \code{(-Inf, +Inf)}, then it will be 
rescaled to \code{[-1, 1]}.}

\item{verbose}{A logical value specifying to display detailed 
messages (when \code{verbose=TRUE}) or not (when \code{verbose=FALSE}).}

\item{theta}{Deprecated as of PathwaySpace 1.0.2; use 'beta' instead.}

\item{pdist}{Deprecated as of PathwaySpace 1.0.2; this parameter is now 
passed internally through \code{decay.fun}.}
}
\value{
A preprocessed \linkS4class{PathwaySpace} class object.
}
\description{
\code{polarProjection} implements a convolution algorithm
to project signals across a 2D-coordinate system.
}
\examples{
# Load a demo igraph
data('gtoy2', package = 'RGraphSpace')

# Create a new PathwaySpace object
ps <- buildPathwaySpace(gtoy2, nrc = 100)
# note: adjust 'nrc' to increase image resolution

# Set '1s' as vertex signal
vertexSignal(ps) <- 1

# Set edge weight
# gs_edge_attr(ps, "weight") <- c(-1, 1, 1, 1, 1, 1)

# Create a 2D-landscape image
ps <- polarProjection(ps, pdist=1)

}
\seealso{
\code{\link{buildPathwaySpace}}
}
\author{
Sysbiolab Team
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMisc.R
\docType{data}
\name{CGC_20211118}
\alias{CGC_20211118}
\title{COSMIC-CGC genes mapped to PathwaySpace images}
\format{
data.frame
}
\source{
COSMIC-CGC database (release v95, tier 1 collection).
}
\usage{
data(CGC_20211118)
}
\value{
A data.frame object.
}
\description{
A data frame listing 'GeneSymbol' and 'Entrez' IDs from the
COSMIC-CGC database (Sondka et al., 2020). These genes are used to
demonstrate the PathwaySpace's summit mapping pipeline, which assigns 
summits to an image space.
}
\examples{
data(CGC_20211118)
}
\references{
Sondka et al. The COSMIC Cancer Gene Census: describing genetic
dysfunction across all human cancers. Nat Rev Cancer 18, 696-705, 2018.
Doi: 10.1038/s41568-018-0060-1.
}
\keyword{CGC_20211118}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMisc.R
\name{pspace.pals}
\alias{pspace.pals}
\title{Create interpolated color palettes for PathwaySpace images}
\usage{
pspace.pals(
  colors = c("#303f9d", "#578edb", "#63b946", "#f3930c", "#a60d0d"),
  trim.colors = c(3, 2, 1, 2, 3),
  offset = 0.5,
  n = 25
)
}
\arguments{
\item{colors}{A vector of five base colors used to construct the custom
diverging palette. These colors are interpolated according to the
`trim.colors` values.}

\item{trim.colors}{A vector of five positive integers that control the
relative weight of each hue in the five-color diverging palette.}

\item{offset}{Adjusts brightness by shifting hues toward the center,
either brighter (`offset > 0`) or darker (` offset < 0`).}

\item{n}{The number of colors to generate in the output palette.}
}
\value{
A vector with hexadecimal color codes.
}
\description{
Creates mixed color palettes by interpolating and offsetting hues, useful for
generating transitions between hues.
}
\examples{
pspace.pals()

}
\seealso{
\code{\link{plotPathwaySpace}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMethods.R
\docType{methods}
\name{summitMapping,PathwaySpace-method}
\alias{summitMapping,PathwaySpace-method}
\alias{summitMapping}
\title{Mapping Summits on PathwaySpace Images}
\usage{
\S4method{summitMapping}{PathwaySpace}(
  ps,
  maxset = 30,
  minsize = 30,
  threshold = 0.5,
  verbose = TRUE,
  segm_fun = summitWatershed,
  ...
)
}
\arguments{
\item{ps}{A \linkS4class{PathwaySpace} class object.}

\item{maxset}{A single positive integer indicating the maximum number 
of summits to be returned by the segmentation function.}

\item{minsize}{A single positive integer indicating the minimum size 
of the summits.}

\item{threshold}{A threshold provided as a fraction (in \code{[0,1]}) of the
max signal intensity.}

\item{verbose}{A logical value specifying to display detailed 
messages (when \code{verbose=TRUE}) or not (when \code{verbose=FALSE}).}

\item{segm_fun}{A segmentation function used to detect summits
(see \code{\link{summitWatershed}}).}

\item{...}{Additional arguments passed to the segmentation function.}
}
\value{
A preprocessed \linkS4class{PathwaySpace} class object.
}
\description{
The \code{summitMapping} method implements a segmentation
strategy to identify summits on a 2D-landscape image 
(see \code{\link{summitWatershed}}).
}
\examples{
# Load a large igraph
data("PCv12_pruned_igraph", package = "PathwaySpace")

# Continue this example from the PathwaySpace vignette,
# in the 'PathwaySpace decoration' section

}
\seealso{
\code{\link{circularProjection}}
}
\author{
Sysbiolab Team
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceDecay.R
\name{expDecay}
\alias{expDecay}
\title{Constructor of exponential decay functions}
\usage{
expDecay(decay = 0.001, pdist = 0.15, plot = FALSE, demo.signal = 1)
}
\arguments{
\item{decay}{A decay factor (in [0,1]). This term indicates how much a
\code{signal} decreases as a function of distance in pathway space. 
For example, at a specific distance defined by the \code{pdist} parameter, 
the signal intensity will be the initial signal multiplied by \code{decay}.}

\item{pdist}{A distance normalization term (in (0, 1]) at which the signal 
reaches `signal * decay`. This parameter is used to anchor the decay to 
a meaningful distance (see `details`). Also, when \code{pdist = 1}, it will
represent the diameter of the inscribed circle within the coordinate space
of a `PathwaySpace` object.}

\item{plot}{A logical value indicating whether to return a `ggplot` object.}

\item{demo.signal}{A numeric value in `[-Inf, Inf]`, only passed when 
\code{plot = TRUE} to visualize the decay curve with a specific signal 
intensity. The value is ignored by the function constructor, as the decay 
function itself is returned without using an initial signal.}
}
\value{
Returns either a function of the form 
\code{function(x, signal) \{ ... \}} or, if \code{plot = TRUE}, a `ggplot`
object illustrating the decay model.
}
\description{
The `expDecay()` constructor either creates a decay function or 
returns a `ggplot` object for visualizing the decay model. It is a utility 
function used internally by \code{\link{circularProjection}} and 
\code{\link{polarProjection}}.
}
\details{
The `expDecay()` constructor creates an exponential decay model. It describes 
how a signal decreases as a function of distance, controlled by a decay 
rate parameter.

The decay function is defined as:

\deqn{y = signal \times decay^{\left(\frac{x}{pdist}\right)}}

where \eqn{signal} represents the initial intensity, \eqn{decay} controls 
the rate of attenuation, and \eqn{x} is a vector of normalized distances.
The \eqn{pdist} parameter anchors the model such that:
\itemize{
  \item \eqn{y = signal} when \eqn{x = 0}
  \item \eqn{y = signal \times decay} when \eqn{x = pdist}
}
}
\examples{
# Return a decay function
decay_fun <- expDecay(decay = 0.25, pdist = 0.5)

# Plot decay model parameters
# expDecay(decay = 0.25, pdist = 0.5, plot = TRUE)

}
\seealso{
\code{\link{linearDecay}}, \code{\link{weibullDecay}}
}
\author{
Sysbiolab Team
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMethods.R
\docType{methods}
\name{getPathwaySpace,PathwaySpace-method}
\alias{getPathwaySpace,PathwaySpace-method}
\alias{getPathwaySpace}
\title{Accessors for Fetching Slots from a PathwaySpace Object}
\usage{
\S4method{getPathwaySpace}{PathwaySpace}(ps, what = "status")
}
\arguments{
\item{ps}{A preprocessed \linkS4class{PathwaySpace} class object}

\item{what}{A character value specifying which information should 
be retrieved from the slots.
Options: "nodes", "edges", "graph", "image", "pars", "misc", 
"signal","projections", "status", "silhouette", "summits", 
"summit_mask", "summit_contour"}
}
\value{
Content from slots in the \linkS4class{PathwaySpace} object.
}
\description{
\code{getPathwaySpace} retrives information from
individual slots available in a PathwaySpace object.
}
\examples{
# Load a demo igraph
data('gtoy1', package = 'RGraphSpace')

# Create a new PathwaySpace object
ps <- buildPathwaySpace(gtoy1, nrc = 100)
# note: adjust 'nrc' to increase image resolution

# Get the 'status' slot in ps
status <- getPathwaySpace(ps, what = 'status')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMisc.R
\docType{data}
\name{gimage}
\alias{gimage}
\title{An image matrix}
\format{
matrix
}
\source{
This package.
}
\usage{
data(gimage)
}
\value{
An image matrix.
}
\description{
An image matrix used for workflow demonstrations.
}
\examples{
data(gimage)
}
\keyword{gimage}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceDecay.R
\name{weibullDecay}
\alias{weibullDecay}
\title{Constructor of Weibull decay functions}
\usage{
weibullDecay(
  decay = 0.001,
  pdist = 0.15,
  shape = 1.05,
  plot = FALSE,
  demo.signal = 1
)
}
\arguments{
\item{decay}{A decay factor (in [0,1]). This term indicates how much a
\code{signal} decreases as a function of distance in pathway space. 
For example, at a specific distance defined by the \code{pdist} parameter, 
the signal intensity will be the initial signal multiplied by \code{decay}.}

\item{pdist}{A distance normalization term (in (0, 1]) at which the signal 
reaches `signal * decay`. This parameter is used to anchor the decay to 
a meaningful distance (see `details`). Also, when \code{pdist = 1}, it will
represent the diameter of the inscribed circle within the coordinate space
of a `PathwaySpace` object.}

\item{shape}{A parameter (>=1) of a Weibull function. When \code{shape=1}
the Weibull decay follows an exponential decay. When \code{shape>1}
the function is first convex, then concave with an inflection point.}

\item{plot}{A logical value indicating whether to return a `ggplot` object.}

\item{demo.signal}{A numeric value in `[-Inf, Inf]`, only passed when 
\code{plot = TRUE} to visualize the decay curve with a specific signal 
intensity. The value is ignored by the function constructor, as the decay 
function itself is returned without using an initial signal.}
}
\value{
Returns either a function of the form 
\code{function(x, signal) \{ ... \}} or, if \code{plot = TRUE}, a `ggplot`
object illustrating the decay model.
}
\description{
The `weibullDecay()` constructor either creates a decay function or 
returns a `ggplot` object for visualizing the decay model. It is a utility 
function used internally by \code{\link{circularProjection}} and 
\code{\link{polarProjection}}.
}
\details{
The `weibullDecay()` constructor creates a decay model based on the Weibull 
distribution. It describes how a signal decreases as a function of distance, 
controlled by both a decay rate and a shape parameter.

The decay function is defined as:

\deqn{y = signal \times decay^{\left(\frac{x}{pdist}\right)^{shape}}}

where \eqn{signal} represents the initial intensity, \eqn{decay} controls 
the rate of attenuation, \eqn{x} is a vector of normalized distances,
and \eqn{shape} adjusts the curvature of the decay. When \eqn{shape = 1}, 
the function follows an exponential decay. For \eqn{shape > 1}, the curve 
transitions from convex to concave, exhibiting an inflection point. The 
\eqn{pdist} parameter anchors the model such that:
\itemize{
  \item \eqn{y = signal} when \eqn{x = 0}
  \item \eqn{y = signal \times decay} when \eqn{x = pdist}
}
}
\examples{
# Return a decay function
decay_fun <- weibullDecay(decay = 0.5, pdist = 0.4, shape = 2)

# Plot decay model parameters
# weibullDecay(decay = 0.5, pdist = 0.4, shape = 2, plot = TRUE)

}
\seealso{
\code{\link{linearDecay}}, \code{\link{expDecay}}
}
\author{
Sysbiolab Team
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMethods.R
\docType{methods}
\name{vertexSignal,PathwaySpace-method}
\alias{vertexSignal,PathwaySpace-method}
\alias{vertexSignal}
\alias{vertexSignal<-}
\alias{vertexDecay}
\alias{vertexDecay<-}
\alias{vertexSignal<-,PathwaySpace-method}
\alias{vertexDecay,PathwaySpace-method}
\alias{vertexDecay<-,PathwaySpace-method}
\title{Accessor Functions for PathwaySpace Objects}
\usage{
\S4method{vertexSignal}{PathwaySpace}(x)

\S4method{vertexSignal}{PathwaySpace}(x) <- value

\S4method{vertexDecay}{PathwaySpace}(x)

\S4method{vertexDecay}{PathwaySpace}(x) <- value
}
\arguments{
\item{x}{A \linkS4class{PathwaySpace} class object.}

\item{value}{The new value of the attribute.}
}
\value{
Updated \linkS4class{PathwaySpace} object.
}
\description{
Get or set 'signal' and 'decay' functions in a 
\linkS4class{PathwaySpace} class object.
}
\examples{
data('gtoy1', package = 'RGraphSpace')
ps <- buildPathwaySpace(gtoy1, nrc = 100)

# Check vertex names
names(ps)

# Access signal values from all vertices
vertexSignal(ps)

# Modify signal value of a specific vertex
vertexSignal(ps)[1] <- 1

# Modify signal value of specific vertices
vertexSignal(ps)[c("n2","n3")] <- 1

# Set '1s' to all vertices
vertexSignal(ps) <- 1

#----

# Access decay function of a specific vertex
vertexDecay(ps)[["n3"]]

# Modify decay function of a specific vertex
vertexDecay(ps)[["n3"]] <- linearDecay()

# Modify decay functions of two vertices
vertexDecay(ps)[c("n1","n3")] <- list( weibullDecay() )

# Modify decay functions of all vertices
vertexDecay(ps) <- weibullDecay(shape = 2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMethods.R
\name{buildPathwaySpace}
\alias{buildPathwaySpace}
\title{Constructor of PathwaySpace-class Objects}
\usage{
buildPathwaySpace(gs, nrc = 500, verbose = TRUE, g = deprecated())
}
\arguments{
\item{gs}{A \code{\link[RGraphSpace]{GraphSpace}} object. Alternatively, 
an \code{\link[igraph]{igraph}} object with node coordinates assigned 
to \code{x} and \code{y} vertex attributes, and node labels assigned 
to \code{name} vertex attribute.}

\item{nrc}{A single positive integer indicating the number of rows and 
columns (in pixels) for a square image matrix. This argument will 
affect the resulting image size and resolution.}

\item{verbose}{A logical value specifying to display detailed 
messages (when \code{verbose=TRUE}) or not (when \code{verbose=FALSE}).}

\item{g}{Deprecated from PathwaySpace 1.0.1; use 'gs' instead.}
}
\value{
A pre-processed \linkS4class{PathwaySpace} class object.
}
\description{
\code{buildPathwaySpace} is a constructor of
PathwaySpace-class objects.
}
\examples{
# Load a demo igraph
data('gtoy1', package = 'RGraphSpace')

# Check graph validity
gs <- GraphSpace(gtoy1)

gs <- normalizeGraphSpace(gs)

# Create a new PathwaySpace object
ps <- buildPathwaySpace(gs, nrc = 100)
# note: adjust 'nrc' to increase image resolution

}
\seealso{
\code{\link[igraph]{undirected_graph}}
}
\author{
Sysbiolab Team
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMisc.R
\name{getNearestNode}
\alias{getNearestNode}
\title{getNearestNode}
\usage{
getNearestNode(ps)
}
\arguments{
\item{ps}{Either a \linkS4class{PathwaySpace} or 
\link[RGraphSpace]{GraphSpace} object.}
}
\description{
Retrieves the nearest neighbor for each node from a 
\linkS4class{PathwaySpace} object using Euclidean distance.
}
\examples{
# See examples in the PathwaySpace's tutorials:
# https://sysbiolab.github.io/PathwaySpace/

}
\seealso{
\code{\link[RANN]{nn2}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceWatershed.R
\name{summitWatershed}
\alias{summitWatershed}
\title{Variation of the watershed algorithm for summit detection}
\usage{
summitWatershed(x, tolerance = 0.1, ext = 1)
}
\arguments{
\item{x}{A 2D-numeric array in which each point represents the coordinates
of a signal in a landscape image.}

\item{tolerance}{The minimum signal intensity of a summit (in [0,1]),
representing a fraction of the maximum signal intensity.}

\item{ext}{Radius (in pixels) for detecting neighboring objects.}
}
\value{
A matrix with labeled summits.
}
\description{
The \code{summitWatershed} function implements a segmentation
strategy to identify summits within a landscape image generated by the
PathwaySpace package. This function is entirely coded in R, which helps
alleviating users from the task of loading an excessive number of
dependencies. Nonetheless, while this novel implementation prevents
the burden a 'dependency heaviness', it still requires optimization
as it currently exhibits slower performance compared to well-established
implementations such as the watershed function from the EBImage package.
The \code{summitWatershed} maintain a certain level of compatibility
with the EBImage's watershed function, and both can be used in the
PathwaySpace package.
}
\examples{
# Load a demo landscape image
data('gimage', package = 'PathwaySpace')

# Scale down the image for a quicker demonstration
gimage <- gimage[200:300, 200:300]

# Check signal range
range(gimage, na.rm = TRUE)
# [1] 0 1

# Check image
\donttest{
image(gimage)
}

# Threshold the signal intensity, for example:
gimage[gimage < 0.5] <- 0

# Run summit segmentation
gmask <- summitWatershed(x = gimage)

# Check resulting image mask
\donttest{
image(gimage)
}

}
\seealso{
\code{\link{summitMapping}}
}
\author{
Sysbiolab Team, Mauro Castro.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspacePlots.R
\docType{methods}
\name{plotPathwaySpace,PathwaySpace-method}
\alias{plotPathwaySpace,PathwaySpace-method}
\alias{plotPathwaySpace}
\title{Plotting 2D-landscape images for the PathwaySpace package}
\usage{
\S4method{plotPathwaySpace}{PathwaySpace}(
  ps,
  colors = pspace.cols(),
  bg.color = "grey95",
  si.color = "grey85",
  si.alpha = 1,
  theme = c("th0", "th1", "th2", "th3"),
  title = "PathwaySpace",
  xlab = "Pathway coordinates 1",
  ylab = "Pathway coordinates 2",
  zlab = "Density",
  font.size = 1,
  font.color = "white",
  zlim = NULL,
  slices = 25,
  add.grid = TRUE,
  grid.color = "white",
  add.summits = TRUE,
  label.summits = TRUE,
  summit.color = "white",
  add.marks = FALSE,
  marks = NULL,
  mark.size = 3,
  mark.color = "white",
  mark.padding = 0.5,
  mark.line.width = 0.5,
  use.dotmark = FALSE,
  add.image = FALSE
)
}
\arguments{
\item{ps}{A \linkS4class{PathwaySpace} class object.}

\item{colors}{A vector of colors.}

\item{bg.color}{A single color for background.}

\item{si.color}{A single color for silhouette.
(see \code{\link{silhouetteMapping}}).}

\item{si.alpha}{A transparency level in [0, 1], used to adjust the 
opacity of the silhouette. This parameter is useful for improving the 
perception of a background image, when one is available.}

\item{theme}{Name of a custom PathwaySpace theme. These themes 
(from 'th0' to 'th3') consist mainly of preconfigured ggplot settings, 
which the user can subsequently refine using \code{\link[ggplot2]{ggplot2}}.}

\item{title}{A string for the title.}

\item{xlab}{The title for the 'x' axis of a 2D-image space.}

\item{ylab}{The title for the 'y' axis of a 2D-image space.}

\item{zlab}{The title for the 'z' axis of the image signal.}

\item{font.size}{A single numeric value passed to plot annotations.}

\item{font.color}{A single color passed to plot annotations.}

\item{zlim}{The 'z' limits of the plot (a numeric vector with two numbers).
If NULL, limits are determined from the range of the input values.}

\item{slices}{A single positive integer value used to split 
the image signal into equally-spaced intervals.}

\item{add.grid}{A logical value indicating whether to add gridlines to 
the image space. However, gridlines will only appear when the image 
is decorated with graph silhouettes (see \code{\link{silhouetteMapping}}).}

\item{grid.color}{A color passed to \code{\link[ggplot2]{geom_point}}.}

\item{add.summits}{A logical value indicating whether to add contour 
lines to 'summits' (when summits are available; 
see \code{\link{summitMapping}}).}

\item{label.summits}{A logical value indicating whether to label summits.}

\item{summit.color}{A color passed to 'summits'.}

\item{add.marks}{A logical value indicating whether to plot vertex labels.}

\item{marks}{A vector of vertex names to be highlighted in the 
image space. This argument overrides 'add.labels'.}

\item{mark.size}{A size argument passed to \code{\link[ggplot2]{geom_text}}.}

\item{mark.color}{A color passed to \code{\link[ggplot2]{geom_text}}.}

\item{mark.padding}{A box padding argument passed to 
\code{\link[ggrepel]{geom_text_repel}}.}

\item{mark.line.width}{A line width argument passed to 
\code{\link[ggrepel]{geom_text_repel}}.}

\item{use.dotmark}{A logical value indicating whether "marks" should be 
represented as dots.}

\item{add.image}{A logical value indicating whether to add a background 
image, when one is available (see \code{\link[RGraphSpace]{GraphSpace}}).}
}
\value{
A ggplot-class object.
}
\description{
\code{plotPathwaySpace} is a wrapper function to 
create dedicated ggplot graphics for PathwaySpace-class objects.
}
\examples{
# Load a demo igraph
data('gtoy1', package = 'RGraphSpace')

# # Check graph validity
gs <- GraphSpace(gtoy1)

gs <- normalizeGraphSpace(gs)

# Create a PathwaySpace object
ps <- buildPathwaySpace(gs, nrc = 300)
# note: adjust 'nrc' to increase image resolution

# Set '1s' as vertex signal
vertexSignal(ps) <- 1

# Create a 2D-landscape image
ps <- circularProjection(ps, k = 2,
   decay.fun = weibullDecay(pdist = 0.4))

# Plot a 2D-landscape image
plotPathwaySpace(ps)

}
\seealso{
\code{\link{circularProjection}}
}
\author{
Sysbiolab Team, Mauro Castro.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMisc.R
\name{plotPathDistances}
\alias{plotPathDistances}
\title{Accessory function to plot pathway space distances}
\usage{
plotPathDistances(pdist, z.transform = FALSE)
}
\arguments{
\item{pdist}{A list generated by the \code{\link{pathDistances}} function.}

\item{z.transform}{A single logical value specifying to convert pathway 
distances into z-score values.}
}
\value{
A 'ggplot' object.
}
\description{
Accessory function to plot pathway space distances
}
\examples{

# Load a vertex-wise distance matrix (distance between nodes in a graph)
data("gdist.toy", package = "PathwaySpace")

# Get two gene lists
from <- sample(colnames(gdist.toy), 50)
to <- sample(colnames(gdist.toy), 50)

# Calculate distances between lists, and between random lists
res <- pathDistances(gdist.toy, from, to)

# Plot observed and null distances
plotPathDistances(res)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceClasses.R
\docType{class}
\name{PathwaySpace-class}
\alias{PathwaySpace-class}
\title{PathwaySpace: An S4 class for signal propagation on image spaces}
\value{
An S4 class object.
}
\description{
PathwaySpace: An S4 class for signal propagation on image spaces
}
\section{Slots}{

\describe{
\item{\code{nodes}}{A data frame with xy-vertex coordinates.}

\item{\code{edges}}{A data frame with edges.}

\item{\code{graph}}{An igraph object.}

\item{\code{image}}{A raster background image matrix.}

\item{\code{pars}}{A list with parameters.}

\item{\code{misc}}{A list with intermediate objects for downstream methods.}

\item{\code{projections}}{A list with processed objects for downstream methods.}

\item{\code{status}}{A vector containing the processing status of the
PathwaySpace object.}
}}

\section{Constructor}{

see \code{\link{buildPathwaySpace}} constructor.
}

\author{
Sysbiolab Team, Mauro Castro (\email{mauro.castro@ufpr.br})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMisc.R
\docType{package}
\name{PathwaySpace-package}
\alias{PathwaySpace}
\alias{PathwaySpace-package}
\title{PathwaySpace: Spatial Projection of Network Signals along Geodesic Paths}
\description{
For a given graph containing vertices, edges, and a signal associated with
the vertices, the PathwaySpace package performs a convolution operation,
which involves a weighted combination of neighboring vertices and their
associated signals. The package then uses a decay function to propagate
these signals, creating geodesic paths on a 2D-image space.
}
\details{
\tabular{ll}{
Package: \tab PathwaySpace\cr
Type: \tab Software\cr
License: \tab Artistic-2.0\cr
Maintainer: \tab Mauro Castro \email{mauro.a.castro@gmail.com}\cr
}
}
\section{Index}{

\tabular{ll}{
\link{PathwaySpace-class}: 
\tab An S4 class for signal propagation on pathway spaces.\cr
\link{buildPathwaySpace}: 
\tab Constructor of PathwaySpace-class objects.\cr
\link{circularProjection}: 
\tab Creating 2D-landscape images from graph objects.\cr
\link{polarProjection}: 
\tab Creating 2D-landscape images from graph objects.\cr
\link{silhouetteMapping}: 
\tab Mapping graph silhouettes on PathwaySpace images.\cr
\link{summitMapping}: 
\tab Mapping summits on a 2D-landscape image.\cr
\link{getPathwaySpace}: 
\tab Accessory method for fetching slots from a PathwaySpace object.\cr
\link{plotPathwaySpace}: 
\tab Plotting 2D-landscape images for the PathwaySpace package.\cr
}
Further information is available in the vignettes by typing
\code{vignette('PathwaySpace')}. Documented topics are also available in
HTML by typing \code{help.start()} and selecting the PathwaySpace package
from the menu.
}

\references{
The Cancer Genome Atlas Analysis Network (2023). PathwaySpace: Spatial propagation 
of network signals along geodesic paths. R package version 0.99.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://sysbiolab.github.io/PathwaySpace/}
  \item \url{https://github.com/sysbiolab/PathwaySpace}
  \item Report bugs at \url{https://github.com/sysbiolab/PathwaySpace/issues}
}

}
\author{
The Cancer Genome Atlas Analysis Network
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMisc.R
\name{pathDistances}
\alias{pathDistances}
\title{Calculate a pathway space distance between two vectors}
\usage{
pathDistances(gdist, from, to, nperm = 1000, verbose = TRUE)
}
\arguments{
\item{gdist}{A distance matrix computed by the igraph's \code{distances} 
function. Rows and columns must be named with vertex labels as listed in
the 'igraph' object.}

\item{from}{A vector with valid vertex names.}

\item{to}{A vector with valid vertex names.}

\item{nperm}{Number of permutations.}

\item{verbose}{A single logical value specifying to display detailed 
messages (when \code{verbose=TRUE}) or not (when \code{verbose=FALSE}).}
}
\value{
A list with pathway space distances and a 'ggplot' object.
}
\description{
Calculate a pathway space distance between two vectors
}
\examples{

# Load a vertex-wise distance matrix (distance between nodes in a graph)
data("gdist.toy", package = "PathwaySpace")

# Get two vertex lists
from <- sample(colnames(gdist.toy), 50)
to <- sample(colnames(gdist.toy), 50)

# Calculate distances between lists, and between random lists
res <- pathDistances(gdist.toy, from, to)
names(res)
# "p_dist"  "z_score"

}
\seealso{
\code{\link{plotPathwaySpace}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMisc.R
\docType{data}
\name{Hallmarks_v2023_1_Hs_symbols}
\alias{Hallmarks_v2023_1_Hs_symbols}
\alias{hallmarks}
\title{A list with Hallmark gene sets (v2023.1)}
\format{
list
}
\source{
MSigDB database (v2023.1).
}
\usage{
data(Hallmarks_v2023_1_Hs_symbols)
}
\value{
A list object.
}
\description{
A list with Human gene symbols from the MSigDB's Hallmark gene set 
collection (Liberzon et al., 2015). These gene sets are used to
demonstrate the PathwaySpace's summit mapping pipeline, which assigns 
summits to an image space.
}
\examples{
data(Hallmarks_v2023_1_Hs_symbols)
}
\references{
Liberzon et al. The Molecular Signatures Database (MSigDB) hallmark 
gene set collection. Cell Systems 1(5):417-425, 2015
Doi: 10.1016/j.cels.2015.12.004
}
\keyword{Hallmarks_v2023_1_Hs_symbols}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMethods.R
\name{gs_vertex_attr<-,PathwaySpace-method}
\alias{gs_vertex_attr<-,PathwaySpace-method}
\alias{gs_vertex_attr<-}
\alias{gs_edge_attr<-,PathwaySpace-method}
\alias{gs_edge_attr<-}
\title{Accessor Functions for PathwaySpace Objects}
\usage{
\S4method{gs_vertex_attr}{PathwaySpace}(x, name, ...) <- value

\S4method{gs_edge_attr}{PathwaySpace}(x, name, ...) <- value
}
\arguments{
\item{x}{A \linkS4class{PathwaySpace} class object.}

\item{name}{Name of the attribute.}

\item{...}{Additional arguments passed to igraph methods.}

\item{value}{The new value of the attribute.}
}
\value{
Updated \linkS4class{PathwaySpace} object.
}
\description{
Get or set edge and vertex attributes in
\linkS4class{PathwaySpace} class object.
}
\examples{
data('gtoy1', package = 'RGraphSpace')
ps <- buildPathwaySpace(gtoy1, nrc = 100)

# Get vertex count
gs_vcount(ps)

# Get edge count
gs_ecount(ps)

# Access a specific vertex attribute
gs_vertex_attr(ps, "signal")

# Replace an entire vertex attribute
gs_vertex_attr(ps, "signal") <- 1

# Modify a single value within a vertex attribute
gs_vertex_attr(ps, "signal")["n1"] <- 1

# Access a specific edge attribute
gs_edge_attr(ps, "weight")

# Replace an entire edge attribute
gs_edge_attr(ps, "weight") <- 1

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMisc.R
\name{pspace.cols}
\alias{pspace.cols}
\title{A simple vector of colors for PathwaySpace images}
\usage{
pspace.cols(n = 25, ...)
}
\arguments{
\item{n}{The number of colors to generate in the output palette.}

\item{...}{Additional arguments (not used).}
}
\value{
A vector with hexadecimal color codes.
}
\description{
A simple vector of colors for PathwaySpace images
}
\examples{
pspace.cols()

}
\seealso{
\code{\link{plotPathwaySpace}}, \code{\link{pspace.pals}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceDecay.R
\name{signalAggregation}
\alias{signalAggregation}
\title{Signal aggregation functions}
\usage{
signalAggregation(method = c("mean", "wmean", "log.wmean", "exp.wmean"))
}
\arguments{
\item{method}{A character string specifying the method for
signal aggregation, returning either a customized \code{\link{mean}} or 
\code{\link{weighted.mean}} function.}
}
\value{
Returns a function of the form: \code{function(x) { ... }}
}
\description{
Signal aggregation functions for \code{\link{circularProjection}}
and \code{\link{polarProjection}} internal calls. The aggregation should be 
symmetric with respect to signal polarity, ensuring that opposite signals 
produce corresponding outputs.
}
\examples{
aggregate.fun <- signalAggregation()

}
\seealso{
\code{\link{circularProjection}}, \code{\link{polarProjection}}, 
\code{\link{weighted.mean}}
}
\author{
Sysbiolab Team
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMisc.R
\docType{data}
\name{PCv12_pruned_igraph}
\alias{PCv12_pruned_igraph}
\alias{gdist.toy}
\title{A pruned and laid out igraph object from Pathway Commons V12}
\format{
igraph
}
\source{
Pathway Commons V12.
}
\usage{
data(PCv12_pruned_igraph)
}
\value{
An igraph object.
}
\description{
This igraph object was created from a 'sif' file available from 
the Pathway Commons V12 (Rodchenkov et al., 2020), which was filtered to 
keep interactions from the following sources: CTD, Recon, HumanCyc, 
DrugBank, MSigDB, DIP, BioGRID, IntAct, BIND, and PhosphoSite. The igraph 
was additionally pruned and laid out by a force-directed algorithm aiming 
signal projection on PathwaySpace's images. Edges with the smallest 
betweenness centrality were pruned using 'backward elimination' and
'forward selection' strategies. The resulting graph represents the 
main connected component with the minimum number of edges.
}
\examples{
data(PCv12_pruned_igraph)
## Suggestion to vizualize this igraph in R:
library(RGraphSpace)
plotGraphSpace(PCv12_pruned_igraph)
}
\references{
Rodchenkov et al. Pathway Commons 2019 Update: integration, analysis and
exploration of pathway data. Nucleic Acids Research 48(D1):D489–D497, 2020.
\doi{10.1093/nar/gkz946}
}
\author{
Chris Wong, Mauro Castro, and TCGA Network.
}
\keyword{PCv12_pruned_igraph}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMethods.R
\docType{methods}
\name{silhouetteMapping,PathwaySpace-method}
\alias{silhouetteMapping,PathwaySpace-method}
\alias{silhouetteMapping}
\title{Decorating PathwaySpace Images with Graph Silhouettes}
\usage{
\S4method{silhouetteMapping}{PathwaySpace}(
  ps,
  pdist = 0.05,
  baseline = 0.01,
  fill.cavity = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{ps}{A \linkS4class{PathwaySpace} class object.}

\item{pdist}{A term (in \code{[0,1]}) determining a distance unit for the
silhouette projection.}

\item{baseline}{A fraction (in \code{[0,1]}) of the silhouette projection,
representing the level over which a silhouette will outline the graph layout.
When \code{baseline = 0} (i.e. lower level of the projection), the 
silhouette will extend over the entire image space, so no outline will 
be visible.}

\item{fill.cavity}{A logical value specifying to fill cavities 
in the silhouette mask (when \code{fill.cavity=TRUE}) or not 
(when \code{fill.cavity=FALSE}).}

\item{verbose}{A logical value specifying to display detailed 
messages (when \code{verbose=TRUE}) or not (when \code{verbose=FALSE}).}
}
\value{
A preprocessed \linkS4class{PathwaySpace} class object.
}
\description{
\code{silhouetteMapping} constructs an image baseline used
to outline the graph layout in a PathwaySpace image.
}
\examples{
# Load a demo igraph
data('gtoy1', package = 'RGraphSpace')

# Create a new PathwaySpace object
ps <- buildPathwaySpace(gtoy1, nrc = 100)
# note: adjust 'nrc' to increase image resolution

# Set '1s' as vertex signal
vertexSignal(ps) <- 1

# Map graph silhouette
ps <- silhouetteMapping(ps, pdist = 0.1)

}
\seealso{
\code{\link{circularProjection}}
}
\author{
Sysbiolab Team
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceDecay.R
\name{polarDecay}
\alias{polarDecay}
\title{Polar transformation functions}
\usage{
polarDecay(
  method = c("power", "gaussian", "logistic"),
  s = 0.5,
  k = 10,
  m = 0.5
)
}
\arguments{
\item{method}{String indicating the transformation to apply.
Must be one of: "power", "gaussian", or "logistic".}

\item{s}{Single numeric value in \code{[0, 1]}. Controls the spread around
the \code{x} mean of the Gaussian function.}

\item{k}{Single numeric value \code{>=1}. Controls the steepness of
the logistic function.}

\item{m}{Single numeric value in \code{[0, 1]}. Specifies the midpoint of
the logistic function.}
}
\value{
Returns a function of the form: \code{function(x, beta) { ... }},
that applies the specified shape-based transformation.
}
\description{
Creates polar transformation functions for
\code{\link{polarProjection}} internal calls. These functions are used to
adjusts signal decay according to point-to-edge angular distances,
with options to attenuate angular shapes.
}
\details{
The polar transformation controls how much the projected signal decays as
a function of the angular distance between a point in pathway space and
a reference edge axis. The function returned by \code{polarDecay()} expects
two arguments, with the following signature:
\code{function(x, beta) { ... }}.

\strong{Power:}
\deqn{x^{\beta}}
where \eqn{x} is a vector of normalized angular distances (in \code{[0, 1]})
and \eqn{beta} is a non-negative exponent that controls the rate of signal
decay. Increasing \eqn{beta} results in a steeper decay rate, modulating
the angular span of the projection.

\strong{Gaussian:}
\deqn{\exp\left(-\frac{(1-x)^2}{2\sigma^2}\right)^{\beta}}
where \eqn{sigma} controls the spread around the mean, creating
fuzzier effect on projections.

\strong{Logistic:}
\deqn{(1 / (1 + \exp(k (x - m))))^{\beta}}
where \eqn{k} is the steepness and \eqn{m} is the function's midpoint,
making more gradual transitions.

These transformations are intended to be plugged into the higher-level
\code{\link{polarProjection}} function, allowing user control over the
polar projection profiles.
}
\examples{
polar.fun <- polarDecay("power")

}
\seealso{
\code{\link{polarProjection}}
}
\author{
Sysbiolab Team
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceMethods.R
\docType{methods}
\name{circularProjection,PathwaySpace-method}
\alias{circularProjection,PathwaySpace-method}
\alias{circularProjection}
\title{Circular Projection of Graph-Associated Signals}
\usage{
\S4method{circularProjection}{PathwaySpace}(
  ps,
  k = 8,
  decay.fun = weibullDecay(),
  aggregate.fun = signalAggregation(),
  rescale = TRUE,
  verbose = TRUE,
  pdist = deprecated()
)
}
\arguments{
\item{ps}{A \linkS4class{PathwaySpace} class object.}

\item{k}{A single positive integer determining the k-top signals for the 
convolution operation.}

\item{decay.fun}{A signal decay function. Available options include 
'Weibull', 'exponential', and 'linear' (see \code{\link{weibullDecay}}).
Users may also define a custom decay model with at least two arguments, 
e.g., \code{function(x, signal) \{ ... \}}, which should returns a vector of  
projected signals of the same length as \code{x}. Additional arguments may  
include any variable available as a graph vertex attribute.}

\item{aggregate.fun}{A function used to aggregate the projected signals. 
It must be provided as a unary function, e.g., \code{function(x) { ... }}, 
which should aggregate a vector of signals to a scalar value. 
Available options include 'mean', 'wmean', 'log.wmean', and 'exp.wmean' 
(See \code{\link{signalAggregation}}).}

\item{rescale}{A logical value indicating whether to rescale 
the signal. If the signal \code{>=0}, then it will be rescaled to 
\code{[0, 1]}; if the signal \code{<=0}, then it will be rescaled to 
\code{[-1, 0]}; and if the signal in \code{(-Inf, +Inf)}, then it will be 
rescaled to \code{[-1, 1]}.}

\item{verbose}{A logical value specifying to display detailed 
messages (when \code{verbose=TRUE}) or not (when \code{verbose=FALSE}).}

\item{pdist}{Deprecated as of PathwaySpace 1.0.2; this parameter is now 
passed internally through \code{decay.fun}.}
}
\value{
A preprocessed \linkS4class{PathwaySpace} class object.
}
\description{
\code{circularProjection} implements a convolution
algorithm to project signals onto a 2D-coordinate system.
}
\examples{
# Load a demo igraph
data('gtoy1', package = 'RGraphSpace')

# Create a new PathwaySpace object
ps <- buildPathwaySpace(gtoy1, nrc = 100)
# note: adjust 'nrc' to increase image resolution

# Set '1s' as vertex signal
vertexSignal(ps) <- 1

# Create a 2D-landscape image
ps <- circularProjection(ps)

}
\seealso{
\code{\link{buildPathwaySpace}},  \code{\link{weibullDecay}},  
\code{\link{expDecay}}, \code{\link{linearDecay}}
}
\author{
Sysbiolab Team
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pspaceDecay.R
\name{signalDecay}
\alias{signalDecay}
\title{Deprecated function}
\usage{
signalDecay(...)
}
\arguments{
\item{...}{Deprecated arguments}
}
\value{
Stop unconditionally
}
\description{
Use \code{\link{weibullDecay}}, \code{\link{expDecay}}, 
and \code{\link{linearDecay}}.
}
\examples{
decay.fun <- weibullDecay()

}
\author{
Sysbiolab Team
}
